package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.util.Hashtable;
import java.util.Vector;

import org.w3c.dom.Document;

/**
 * Common data retrieval methods which retrieve data of interest to <code>SVGLineChart, 
 * SVGBarChart, SVGStackBarChart, SVGPieChart</code> and <code>SVGMeter</code> from 
 * documents which conform to the documented data input schemas. 
 * 
 * @version %I%
 */
public class CommonDataRetriever extends DOMDataRetriever implements IDataInputProcessingExceptionCodes, IDataInputConstants {
	/**
	 * Document containing configuration data.
	 */
	protected Document configDocument;
	/**
	 * Document containing data update. May also contain 
	 * configuration data.
	 */
	protected Document dataDocument;
			
	/**
	 * Sole constructor.
	 */		
	CommonDataRetriever() {
		super();
	}
	
	/**
	 * Sets the document which contains the configuration data.
	 * Configuration-related data is retrieved only from this document. 
	 * 
	 * @param newConfigDocument the configuration data document
	 */
	public void setConfigDocument(Document newConfigDocument) {
		configDocument = newConfigDocument;
	}		
	
	/**
	 * Sets the document which contains the data update and may contain
	 * configuration data; however,only in the event that <code>configDocument</code> 
	 * is <code>null</code> will configuration data be retrieved from this document.
	 * 
	 * @param newDataDocument the document providing data updates
	 */
	public void setDataDocument(Document newDataDocument) {
		dataDocument = newDataDocument;
	}
	
	/**
	 * Returns the number of data sets found in the data document.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, 
	 * NO_DATASETS_FOUND</code>.
	 */
	public int getNumberOfDatasets() throws DataInputProcessingException {
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		int number = getNumberOfElements(dataDocument, DATASET);
		if (number == 0) {		
			throwException(NO_DATASETS_FOUND, DATAUPDATE);
		}				
		return number;
	}
	
	/**
	 * Returns the configuration attributes and values (if any) found in the 
	 * configuration document.
	 * 
	 * @throws DataInputProcessingException for code <code>NO_DATA_DOCUMENT</code>.
	 */
	public Hashtable getConfigurationAttributes() throws DataInputProcessingException {
		
		Document document = (configDocument != null) ? configDocument : dataDocument;	
		if (document == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
			
		Hashtable properties = getAttributesFromSingleElement(document, CONFIGURATION);
		return properties;
	}
	
	/**
	 * Returns the ordered array of values found in the data document which comprise 
	 * the legend. Values can be ordered by specifying their ordinal position; otherwise,
	 * the retrieved (document) order is used.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_NAMES_FOUND, MISSING_NAMES_OR_POSITIONS,
	 * POSITION_NUMBER_OUT_OF_BOUNDS, DUPLICATE_POSITION_NUMBER, INVALID_NUMBER_FORMAT</code>
	 */
	public String [] getLegendLabels() throws DataInputProcessingException {		
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
				
		Vector labels = getAttributeFromElements(dataDocument, DATASET, NAME);				
		if (labels == null) {
			throwException(NO_NAMES_FOUND, DATASET);			
		}	
		
		int labelsSize = labels.size();
		String [] result = new String[labelsSize];
		Vector positions = getAttributeFromElements(dataDocument, DATASET, POSITION);
		
		// use document order
		if (positions == null) {
			for (int i = 0;i < labelsSize; i++) {
				result[i] = (String)labels.get(i);
			}
			return result;
		}
					
		if (labelsSize != positions.size()) {
			throwException(MISSING_NAMES_OR_POSITIONS, DATASET);			
		}			
		
		// put the label in the array based on its position
		for (int i = 0;i < labelsSize; i++) {
			short position = 0;			
			try {
				position = Short.parseShort((String)positions.get(i));					
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, DATASET);				
			}
			// make sure the position is within bounds						
			if (position > (labelsSize - 1) || position < 0) {
				throwException(POSITION_NUMBER_OUT_OF_BOUNDS, DATASET);
			}				
			// make sure the position is not already taken
			String test = null;		
			try {
				test = result[position];
			}
			catch (Exception e) {}								
						
			if (test != null) {
				throwException(DUPLICATE_POSITION_NUMBER, DATASET);
			}	
			result[position] = (String)labels.get(i);
		}				
		return result;			
	}	
		
	/**
	 * Returns the timestamp found in the data document.
	 * 
	 * @throws DataInputProcessingException for code <code>NO_DATA_DOCUMENT, NO_TIMESTAMP_FOUND</code>.
	 */	
	public String getTimestamp() throws DataInputProcessingException { 
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		Document document = (configDocument != null) ? configDocument : dataDocument;
		String timestamp = getAttributeFromSingleElement(dataDocument, DATAUPDATE, TIMESTAMP);
		if (timestamp == null) {
			throwException(NO_TIMESTAMP_FOUND, DATAUPDATE);			
		}			
		return timestamp;
	}	
	
	/**
	 * Returns the array of values which mark the segments into which the data range 
	 * is divided.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_VALUES_FOUND, MISSING_VALUES_OR_POSITIONS,
	 * POSITION_NUMBER_OUT_OF_BOUNDS, DUPLICATE_POSITION_NUMBER, INVALID_NUMBER_FORMAT</code>
	 */
	public String [] getRangeMarkers() throws DataInputProcessingException {	
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		// get the segmentMarker values	
		Vector labels = getAttributeValuesFromSingleElementChildren(dataDocument, RANGE, SEGMENT_MARKER, VALUE);
		int labelsSize = (labels == null) ? 0 : labels.size();
		String [] result = null;	
		
		// if there is a max value given, it will be added to the labels		
		String rangeMaxValue = getAttributeFromSingleElement(dataDocument, RANGE, MAXVALUE);			
		if (rangeMaxValue != null) {
			// ensure it's a number
			double j = 0;
			try {
				j = Double.parseDouble(rangeMaxValue);				
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, RANGE);	
			}			
			// restriction to positive integers
			if (j < 1) {
				throwException(INVALID_NUMBER_FORMAT, RANGE);
			}				
			result = new String[labelsSize + 1];
			result[result.length - 1] = rangeMaxValue;
			if (labels == null) {
				return result;
			}	
		} else if (labelsSize == 0) {
			throwException(NO_VALUES_FOUND, SEGMENT_MARKER);			
		}else {
			result = new String[labelsSize];
		}	
		
				
		// get the positions, if any
		Vector positions = getAttributeValuesFromSingleElementChildren(dataDocument, RANGE, SEGMENT_MARKER, POSITION);
		
		// use document order
		if (positions == null) {
			for (int i = 0;i < labelsSize; i++) {
				result[i] = (String)labels.get(i);
				// ensure they are all proper numbers	
				double j = 0;			
				try {
					j = Double.parseDouble(result[i]);
				} catch (Exception exc) {
					// NumberFormatException
					throwException(INVALID_NUMBER_FORMAT, SEGMENT_MARKER);	
				}			
				// restriction to positive integers
				if (j < 1) {
					throwException(INVALID_NUMBER_FORMAT, SEGMENT_MARKER);
				}			
			}
			return result;
		}		
		if (positions.size() != labelsSize) {
			throwException(MISSING_VALUES_OR_POSITIONS, SEGMENT_MARKER);			
		}
					
		// use positions		
		for (int i = 0;i < labelsSize; i++) {
			short position = 0;			
			try {
				position = Short.parseShort((String)positions.get(i));
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, SEGMENT_MARKER);				
			}		
			// make sure the position is within bounds						
			if (position > (labelsSize - 1)  || position < 0) {
				throwException(POSITION_NUMBER_OUT_OF_BOUNDS, DATASET);
			}				
			// make sure the position is not already taken
			String test = null;		
			try {
				test = result[position];
			}
			catch (Exception e) {}			
			if (test != null) {
				throwException(DUPLICATE_POSITION_NUMBER, DATASET);
			}	
			result[position] = (String)labels.get(i);			
			// ensure they are all proper numbers			
			double j = 0;			
			try {
				j = Double.parseDouble(result[position]);
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, SEGMENT_MARKER);
			}			
			// restriction to positive integers
			if (j < 1) {
				throwException(INVALID_NUMBER_FORMAT, SEGMENT_MARKER);
			}			
		}						
		return result;	
	}		
		
	/**
	 * Returns the array of values which comprise the categories into which the
	 * data are assigned.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_NAMES_FOUND, MISSING_NAMES_OR_POSITIONS,
	 * POSITION_NUMBER_OUT_OF_BOUNDS, DUPLICATE_POSITION_NUMBER, INVALID_NUMBER_FORMAT</code>
	 */	
	public String [] getCategories() throws DataInputProcessingException {
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		// get the category names
		Vector labels = getAttributeValuesFromSingleElementChildren(dataDocument, CATEGORIZATION, CATEGORY, NAME);
		if (labels == null) {
			throwException(NO_NAMES_FOUND, CATEGORY);				
		}
		int labelsSize = labels.size();				
		String [] result = new String [labelsSize];
		
		// get the positions, if any
		Vector positions = getAttributeValuesFromSingleElementChildren(dataDocument, CATEGORIZATION, CATEGORY, POSITION);
					
		// use document order
		if (positions == null) {
			for (int i = 0;i < labelsSize; i++) {
				result[i] = (String)labels.get(i);
			}
			return result;
		}		
		if (positions.size() != labelsSize) {
			throwException(MISSING_NAMES_OR_POSITIONS, CATEGORY);			
		}				 		
		
		// use positions		
		for (int i = 0;i < labelsSize; i++) {
			short position = 0;			
			try {
				position = Short.parseShort((String)positions.get(i));
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, CATEGORY);	
			}	
			// make sure the position is within bounds						
			if (position > (labelsSize - 1) || position < 0) {
				throwException(POSITION_NUMBER_OUT_OF_BOUNDS, DATASET);
			}				
			// make sure the position is not already taken
			String test = null;		
			try {
				test = result[position];
			}
			catch (Exception e) {}					
						
			if (test != null) {
				throwException(DUPLICATE_POSITION_NUMBER, DATASET);
			}	
			result[position] = (String)labels.get(i);	
		}						
		return result;	
	}	
		
	/**
	 * Returns an array of values, each of which uniquely identifies a category. These
	 * identifiers are used to map data values to the categories to which they have
	 * been assigned.
	 *  
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_IDS_FOUND, MISSING_IDS_OR_POSITIONS,
	 * POSITION_NUMBER_OUT_OF_BOUNDS, DUPLICATE_POSITION_NUMBER, INVALID_NUMBER_FORMAT</code>
	 */	
	public String [] getCategoryIds() throws DataInputProcessingException {
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		// get the category ids
		Vector labels = getAttributeValuesFromSingleElementChildren(dataDocument, CATEGORIZATION, CATEGORY, ID);
		if (labels == null) {
			throwException(NO_IDS_FOUND, CATEGORY);				
		}
		int labelsSize = labels.size();		
		String [] result = new String[labelsSize];		
		
		// get the positions, if any
		Vector positions = getAttributeValuesFromSingleElementChildren(dataDocument, CATEGORIZATION, CATEGORY, POSITION);		
		if (positions == null) {
			for (int i = 0;i < labelsSize; i++) {
				result[i] = (String)labels.get(i);
			}
			return result;
		}		
		if (positions.size() != labelsSize) {
			throwException(MISSING_IDS_OR_POSITIONS, CATEGORY);			
		}				 		
		
		// use positions		
		for (int i = 0;i < labelsSize; i++) {
			short position = 0;			
			try {
				position = Short.parseShort((String)positions.get(i));
			} catch (Exception exc) {
				throwException(INVALID_NUMBER_FORMAT, CATEGORY);	
			}					
			// make sure the position is within bounds						
			if (position > (labelsSize - 1) || position < 0) {
				throwException(POSITION_NUMBER_OUT_OF_BOUNDS, DATASET);
			}				
			// make sure the position is not already taken
			String test = null;		
			try {
				test = result[position];
			}
			catch (Exception e) {}
			if (test != null) {
				throwException(DUPLICATE_POSITION_NUMBER, DATASET);
			}	
			result[position] = (String)labels.get(i);			
		}						
		return result;	
	}	
		
	/**
	 * Returns the array of values, each of which represents the sum of the data values 
	 * within a category.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_TOTALS_FOUND, MISSING_TOTALS_OR_POSITIONS,
	 * POSITION_NUMBER_OUT_OF_BOUNDS, DUPLICATE_POSITION_NUMBER, INVALID_NUMBER_FORMAT</code>
	 */	
	public String [] getCategoryTotals() throws DataInputProcessingException {
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		// get the category totals
		Vector labels = getAttributeValuesFromSingleElementChildren(dataDocument, CATEGORIZATION, CATEGORY, TOTAL);
		if (labels == null) {
			throwException(NO_TOTALS_FOUND, CATEGORY);		
		}
		int labelsSize = labels.size();
		
		// ensure they are all numbers
		for (int i = 0; i < labelsSize; i++) {
			double j = -1;
			try {
				j = Double.parseDouble((String)labels.get(i));
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, CATEGORY);				
			}	
			// restriction to non-negative integers
			if (j < 0) {
				throwException(INVALID_NUMBER_FORMAT, CATEGORY);
			}		
			
		}		
		String [] result = new String[labelsSize];	
			
		// get the positions, if any
		Vector positions = getAttributeValuesFromSingleElementChildren(dataDocument, CATEGORIZATION, CATEGORY, POSITION);		
		if (positions == null) {
			for (int i = 0;i < labelsSize; i++) {
				result[i] = (String)labels.get(i);
			}
			return result;
		}		
		if (positions.size() != labelsSize) {
			throwException(MISSING_TOTALS_OR_POSITIONS, CATEGORY);				
		}				 		
		
		// use positions		
		for (int i = 0;i < labelsSize; i++) {
			short position = 0;			
			try {
				position = Short.parseShort((String)positions.get(i));
			} catch (Exception exc) {
				throwException(INVALID_NUMBER_FORMAT, CATEGORY);
			}					
			// make sure the position is within bounds						
			if (position > (labelsSize - 1) || position < 0) {
				throwException(POSITION_NUMBER_OUT_OF_BOUNDS, DATASET);	
			}				
			// make sure the position is not already taken
			String test = null;		
			try {				
				test = result[position];
			}
			catch (Exception e) {}				
		
			if (test != null) {
				throwException(DUPLICATE_POSITION_NUMBER, DATASET);		
			}
			result[position] = (String)labels.get(i);				
		}						
		return result;	
	}	
	
	/**
	 * Returns the array of data values for the data set with the specified position
	 * from the data document.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_VALUES_FOUND, NO_IDS_FOUND, 
	 * MISSING_CATEGORY_IDS_OR_DATAPOINT_VALUES, CATEGORY_ID_MISMATCH, INVALID_NUMBER_FORMAT</code>
	 */
	public String [] getDatapointValuesFromDatasetWithPosition(String position) throws DataInputProcessingException {
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		// get the category (key) and value attributes from each datapoint in the dataset
		// first see if the position attribue is specified
		Hashtable values = getChildAttributesFromElementWithAttribute(dataDocument, DATASET, POSITION, position, DATAPOINT, CATEGORY, VALUE);
		if (values == null) {
			values = getAttributeValuesFromChildrenOfElementWithDocOrderOrdinal(dataDocument, DATASET, DATAPOINT, CATEGORY, VALUE, position);					
		}
		// just get the one with the ordinal position
		if (values == null) {			
			throwException(NO_VALUES_FOUND, DATAPOINT);			
		}
		int valuesSize = values.size();
					
		// get the category attribute from each category and order them according 
		// to the category position attribute
		String [] categoryIds = getCategoryIds();
		if (categoryIds == null) {
			throwException(NO_IDS_FOUND, CATEGORY);		
		}
					
		// make sure they are the same size
		if (valuesSize != categoryIds.length) {
			throwException(MISSING_CATEGORY_IDS_OR_DATAPOINT_VALUES, DATAUPDATE);	
		}
		
		String [] result = new String[valuesSize];
		
		// get the datapoint values in the same order as the category ID to which 
		//the datapoint category attribute maps
		for (int i = 0; i < valuesSize; i++) {
			result[i] = (String)values.get(categoryIds[i]);	
			if (result[i] == null) {
				throwException(CATEGORY_ID_MISMATCH, DATAPOINT);	
			}	
			// ensure it's a number
			double j = 0;
			try {
				j = Double.parseDouble(result[i]);
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, DATAPOINT);		
			}	
			// restriction to non-negative integers
			if (j < 0) {
				throwException(INVALID_NUMBER_FORMAT, SEGMENT_MARKER);
			}						
		}		
		return result;	
	}	 	
	
	/**
	 * Returns the array of data values from the first data set found in the data document.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_VALUES_FOUND, NO_IDS_FOUND, 
	 * MISSING_CATEGORY_IDS_OR_DATAPOINT_VALUES, CATEGORY_ID_MISMATCH, INVALID_NUMBER_FORMAT</code>
	 */
	public String [] getDatapointValuesFromFirstDataset() throws DataInputProcessingException {
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		// get the category (key) and value attributes from each datapoint in the dataset
		Hashtable values = getAttributeValuesFromChildrenOfFirstElement(dataDocument, DATASET, DATAPOINT, CATEGORY, VALUE);	
		if (values == null) {
			throwException(NO_VALUES_FOUND, DATAPOINT);	
		}
		int valuesSize = values.size();
					
		// get the category attribute from each category and order them according 
		// to the category position attribute
		String [] categoryIds = getCategoryIds();
		if (categoryIds == null) {
			throwException(NO_IDS_FOUND, CATEGORY);					
		}		
					
		// make sure they are the same size
		if (valuesSize != categoryIds.length) {
			throwException(MISSING_CATEGORY_IDS_OR_DATAPOINT_VALUES, DATAUPDATE);			
		}
		
		// put the datapoint values in the same order as the category ID to which 
		//the datapoint category attribute maps
		String [] result = new String[valuesSize];		
		for (int i = 0; i < valuesSize; i++) {
			result[i] = (String)values.get(categoryIds[i]);	
			if (result[i] == null) {
				throwException(CATEGORY_ID_MISMATCH, DATAPOINT);				
			}					
			// ensure it's a number
			double j = 0;
			try {
				j = Double.parseDouble(result[i]);
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, DATAPOINT);	
			}		
			// restriction to non-negative integers
			if (j < 0) {
				throwException(INVALID_NUMBER_FORMAT, DATAPOINT);				
			}				
		}		
		return result;	
	}	
	
	/**
	 * Convenience method for throwing exceptions.
	 */
	protected void throwException(short code, String element) throws DataInputProcessingException {
		DataInputProcessingException e = new DataInputProcessingException(code, element);
		throw e;		
	}	
	
}

