package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.ByteArrayOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.io.StringWriter;
import java.util.Hashtable;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;

/**
 * <code>GraphicDocumentGenerator</code> is an abstract base class for classes which take
 * content data input as Document Object Model (DOM) objects and then generate DOM objects 
 * which represent graphic visualizations of that content data.
 * 
 * @version %I%
 */
public abstract class GraphicDocumentGenerator implements IGraphicDocumentGenerator, Serializable {
	/**
	 * Configuration document
	 */
	protected Document configDocument;
	
	/**
	 * Data document
	 */	
	protected Document dataDocument;
		
	/**
	 * Graphic type
	 */
	protected String graphicType;	
	
	/**
	 * Graphic document
	 */
	protected Document generatedDocument;
	
	/**
	 * Resource folder location
	 */
	protected String resourcesFolder;
	
	/**
	 * Graphic title
	 */
	protected String graphicTitle;
		
	/**
	 * Timestamp prefix
	 */
	protected String graphicTimestampPrefix;
		
	/**
	 * Legend display indicator
	 */
	protected boolean suppressLegend = false;
	
	/**
	 * Legend title
	 */
	protected String graphicLegendTitle;
		
	/**
	 * Graphic font
	 */
	protected String graphicFont;
		
	/**
	 * Default colour palette
	 */
	protected String graphicColourPaletteName;
		
	/**
	 * Default set of available colour palettes
	 */
	protected Hashtable graphicColourPalette;
		
	/**
	 * Graphic width
	 */
	protected short graphicWidth;
	
	/**
	 * Graphic height
	 */
	protected short graphicHeight;
	
	/**
	 * User preference page
	 */
	protected String preferencesPage;
	
	/**
	 * User preference page pixel width, resolvable to <code>int</code>.
	 * Specific to the preference page.
	 */
	protected String preferencesPageWidth;
	
	/**
	 * User preference page pixel height, resolvable to <code>int</code>.
	 * Specific to the preference page.
	 */
	protected String preferencesPageHeight;
	
		
    /**
	 * @see IGraphicDocumentGenerator#getGeneratedDocument()
	 */
	public Document getGeneratedDocument() {
		return generatedDocument;
	}

	/**
	 * @see IGraphicDocumentGenerator#getConfigDocument()
	 */
	public Document getConfigDocument() {
		return configDocument;
	}

	/**
	 * @see IGraphicDocumentGenerator#getDataDocument()
	 */
	public Document getDataDocument() {
		return dataDocument;
	}

	/**
	 * @see IGraphicDocumentGenerator#getGraphicType()
	 */
	public String getGraphicType() {
		return graphicType;
	}
		
	/**
	 * @see IGraphicDocumentGenerator#getGraphicHeight()
	 */
	public String getGraphicHeight() {
		return Short.toString(graphicHeight);
	}

	/**
	 * @see IGraphicDocumentGenerator#getGraphicWidth()
	 */
	public String getGraphicWidth() {
		return Short.toString(graphicWidth);
	}

	/**
	 * @see IGraphicDocumentGenerator#getGraphicFont()
	 */
	public String getGraphicFont() {
		return graphicFont;
	}
	
	/**
	 * @see IGraphicDocumentGenerator#getGraphicColourPaletteName()
	 */
	public String getGraphicColourPaletteName() {
		return graphicColourPaletteName;
	}
	
	/**
	 * @see IGraphicDocumentGenerator#getGraphicColourPalette()
	 */
	public String [] getGraphicColourPalette() {
		return (String [])graphicColourPalette.get(graphicColourPaletteName);
	}
	
	/**
	 * @see IGraphicDocumentGenerator#getGraphicTitle()
	 */
	public String getGraphicTitle() {
		return graphicTitle;
	}
			
	/**
	 * @see IGraphicDocumentGenerator#isLegendSuppressed()
	 */
	public boolean isLegendSuppressed() {
		return suppressLegend;
	}	
	
	/**
	 * @see IGraphicDocumentGenerator#getGraphicTimestampPrefix()
	 */
	public String getGraphicTimestampPrefix() {
		return graphicTimestampPrefix;
	}		
	
	/**
	 * @see IGraphicDocumentGenerator#getGraphicLegendTitle()
	 */
	public String getGraphicLegendTitle() {
		return graphicLegendTitle;
	}
		
	/**
	 * @see IGraphicDocumentGenerator#getResourcesFolder()
	 */
	public String getResourcesFolder() {
		return resourcesFolder;
	}
	
	
	/**
	 * @see IGraphicDocumentGenerator#setConfigDocument(Document)
	 */
	public void setConfigDocument(Document newConfigDocument) throws DataInputProcessingException {
		configDocument = newConfigDocument;
	}
	
	/**
	 * @see IGraphicDocumentGenerator#setDataDocument(Document)
	 */
	public void setDataDocument(Document newDataDocument) throws DataInputProcessingException {
		dataDocument = newDataDocument;
	}

	/**
	 * @see IGraphicDocumentGenerator#setGraphicHeight(String)
	 */
	public void setGraphicHeight(String newGraphicHeight) {
		try {
			graphicHeight = Short.parseShort(newGraphicHeight);
		}
		catch (Exception exc) {
			graphicHeight = 0;
		}
	}

	/**
	 * @see IGraphicDocumentGenerator#setGraphicWidth(String)
	 */
	public void setGraphicWidth(String newGraphicWidth) {
		try {
			graphicWidth = Short.parseShort(newGraphicWidth);
		}
		catch (Exception exc) {
			graphicWidth = 0; 
		}	
	}

	/**
	 * @see IGraphicDocumentGenerator#setGraphicTitle(String)
	 */
	public void setGraphicTitle(String newGraphicTitle) {
		if (newGraphicTitle != null) {
			graphicTitle = newGraphicTitle.trim();
		}
	}
	
	/**
	 * @see IGraphicDocumentGenerator#setSuppressLegend(String)
	 */
	public void setSuppressLegend(String newSuppressLegend) {
		if (newSuppressLegend != null) {
			if (newSuppressLegend.trim().equals("1") || Boolean.valueOf(newSuppressLegend).booleanValue()) {
				suppressLegend = true;
			}
			else {
				suppressLegend = false;
			}
		}	
	}	
	
	/**
	 * @see IGraphicDocumentGenerator#setGraphicTimestampPrefix(String)
	 */
	public void setGraphicTimestampPrefix(String newGraphicTimestampPrefix) {
		if (newGraphicTimestampPrefix != null) {
			graphicTimestampPrefix = newGraphicTimestampPrefix.trim();;
		}
	}		
	
	/**
	 * @see IGraphicDocumentGenerator#setGraphicLegendTitle(String)
	 */
	public void setGraphicLegendTitle(String newGraphicLegendTitle) {
		if (newGraphicLegendTitle != null) {
			graphicLegendTitle = newGraphicLegendTitle.trim();
		}
	}
	
	/**
	 * @see IGraphicDocumentGenerator#setGraphicFont(String)
	 */
	public void setGraphicFont(String newGraphicFont) {
		if (newGraphicFont != null) {
			graphicFont = newGraphicFont.trim();
		}
	}
	
	/**
	 * @see IGraphicDocumentGenerator#setGraphicColourPaletteName(String)
	 */
	public boolean setGraphicColourPaletteName(String newGraphicColourPaletteName) {
		boolean validName = false;
		if (newGraphicColourPaletteName != null) {			
			String name = newGraphicColourPaletteName.trim();
			validName = graphicColourPalette.containsKey(name);
			if (validName) {
				graphicColourPaletteName = name;
			}
		}
		return validName;				
	}
	
	/**
	 * @see IGraphicDocumentGenerator#setResourcesFolder(String)
	 */
	public void setResourcesFolder(String newResourcesFolder) {
		if (newResourcesFolder != null) {
			resourcesFolder= newResourcesFolder.trim();
		}
	}
		
	
	/**
	 * @see IGraphicDocumentGenerator#addGraphicColourPalette(String paletteName, String [] palette)
	 */
	public boolean addGraphicColourPalette(String paletteName, String [] palette) throws NullPointerException {
		boolean nameAvailable = false;
		if (paletteName != null && palette != null) {
			String name = paletteName.trim();
			nameAvailable = !graphicColourPalette.containsKey(name);
			if (nameAvailable) {
				graphicColourPalette.put(name, palette);					
			}		
		}
		return nameAvailable;		
	}	
	
	/**
	 * @see IGraphicDocumentGenerator#generateGraphicDocument()
	 */
	public Document generateGraphicDocument() throws DataInputProcessingException {
		return generatedDocument;
	}	
	
	/**
	 * @see IGraphicDocumentGenerator#generateAndSerializeTo(String)
	 */
	public boolean generateAndSerializeTo(String uri) throws DataInputProcessingException, IOException {
		generateGraphicDocument();
		if (generatedDocument != null) {
			return serializeGeneratedDocumentToFile(uri);
		}
		else {
			return false;
		}	
	}
	
	/**
	 * @see IGraphicDocumentGenerator#generateAndSerialize()
	 */
	public String generateAndSerialize() throws DataInputProcessingException, IOException {
		generateGraphicDocument();
		return serializeGeneratedDocumentToString();
	}
		
	/**
	 * @see IGraphicDocumentGenerator#serializeGeneratedDocumentToFile(String)
	 */
	public boolean serializeGeneratedDocumentToFile(String relativeUri) throws IOException {
		if (generatedDocument == null) {
			return false;
		}
		
		FileWriter fWriter = new FileWriter(relativeUri);		
		OutputFormat oFormat = new OutputFormat(generatedDocument);
		(new XMLSerializer(fWriter, oFormat)).serialize(generatedDocument);
		return true;		
	}		
		
	/**
	 * @see IGraphicDocumentGenerator#serializeGeneratedDocumentToString()
	 */
	public String serializeGeneratedDocumentToString() throws IOException {
		if (generatedDocument == null) {
			return null;
		}
		
		StringWriter writer = new StringWriter();       
       	(new XMLSerializer(writer, null)).serialize(generatedDocument);        
        return writer.getBuffer().toString();
	}
	
	/**
	 * @see IGraphicDocumentGenerator#serializeGeneratedDocumentToStream()
	 */
	public byte[] serializeGeneratedDocumentToStream() {
        if (generatedDocument == null) {
        	return null;
        }
        	
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        try {
        	(new XMLSerializer(stream, null)).serialize(generatedDocument);
        } catch (Exception e) {
      		System.out.println("Serialize to stream failed");
        }
        return stream.toByteArray();
	}
		
}