package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.IOException;

import org.w3c.dom.Document;

/**
 * The <code>IGraphicDocumentGenerator</code> interface provides users with access 
 * to objects which generate graphic visualizations of (potentially) dynamic input data.
 * These objects encapsulate the state data needed for generating the various 
 * visualizations supported by the implementation.
 *  
 * @version %I%
 */
public interface IGraphicDocumentGenerator {
				
	/**
	 * Returns the type indicator for the graphic generated by this object.
	 * 
	 * @return String  a type indicator	 
	 */
	String getGraphicType(); 
	
	/**
	 * Returns the <code>Document</code> object generated by this object or <code>null</code>
	 * if none has been generated.
	 * 
	 * @return Document  an object, which implements <code>org.w3c.dom.Document</code>
	 */
	Document getGeneratedDocument(); 
		
	/**
	 * Returns the <code>Document</code> object which provides the configuration data 
	 * for the graphic generated by this object or <code>null</code> if none has been
	 * set.
	 * 
	 * @return Document  an object, which implements <code>org.w3c.dom.Document</code> 
	 * and conforms to the configuration schema for the generator. 
	 */
	Document getConfigDocument();
			
	/**
	 * Returns the <code>Document</code> object which provides the content data 
	 * for the graphic generated by this object or <code>null</code> if none has been
	 * set.
	 * 
	 * @return Document  an object, which implements <code>org.w3c.dom.Document</code> 
	 * and conforms to the data schema for the generator. 
	 */
	Document getDataDocument();
			
	/**
	 * Returns the height of the graphic generated by this object.
	 * 
	 * @return String  the pixel height
	 */
	String getGraphicHeight(); 
	
	/**
	 * Returns the width of the graphic generated by this object.
	 * 
	 * @return String  the pixel width
	 */
	String getGraphicWidth(); 

	/**
	 * Returns the font used by the graphic generated by this object.
	 * 
	 * @return String  the font family name
	 */
	String getGraphicFont(); 
	
	/**
	 * Returns the name of the default colour palette used by the graphic 
	 * generated by this object.
	 * 
	 * @return String  the name of the default colour palette
	 * @see IGraphicDocumentStyle
	 */
	String getGraphicColourPaletteName(); 
	
	/**
	 * Returns the set of colours belonging to the default colour palette used by the 
	 * graphic generated by this object.
	 * 
	 * @return String []  the <code>String</code> array of hexadecimal colours
	 * @see IGraphicDocumentStyle
	 */
	String [] getGraphicColourPalette(); 
	
	/**
	 * Returns the title of the graphic generated by this object or <code>null</code> 
	 * if none has been specified.
	 * 
	 * @return String  the display title of the graphic
	 */	
	String getGraphicTitle(); 
	
	/**
	 * Returns the timestamp prefix of the graphic generated by this object
	 * or <code>null</code> if none has been specified.
	 * 
	 * @return String  the prefix which is prepended to the timestamp
	 */
	String getGraphicTimestampPrefix(); 

	/**
	 * Returns the legend suppression indicator for the graphic generated by this object.
	 * 
	 * @return boolean  <code>false</code>, if legend is to be displayed or is not supported by the graphic type.
	 */
	boolean isLegendSuppressed(); 
	
	/**
	 * Returns the legend title of the graphic generated by this object or <code>null</code> 
	 * if the graphic does not support a legend or no title has been specified.
	 * 
	 * @return String  the legend title 
	 */
	String getGraphicLegendTitle(); 
	
	/**
	 * Returns the folder location for any resources needed by the 
	 * generated graphic or <code>null</code> if none has been specified .
	 * 
	 * @return String the folder location on the local machine
	 */
	String getResourcesFolder(); 

	/**
	 * Returns the user preferences page name associated with the
	 * generated graphic or <code>null</code> if none has been specified.
	 * 
	 * @return String the name of the user preferences page
	 */
	String getPreferencesPage(); 

	/**
	 * Returns the width of the window which displays the user preferences page 
	 * associated with the generated graphic.
	 * 
	 * @return String the pixel width
	 */
	String getPreferencesPageWidth(); 

	/**
	 * Returns the height of the window which displays the user preferences page 
	 * associated with the generated graphic.
	 * 
	 * @return String the pixel height
	 */
	String getPreferencesPageHeight(); 

	/**
	 * Specifies the XML document which is to provide the configuration data for the graphic
	 * generated by this object. Note that since this information is generated into the graphic, 
	 * this method must be called before generating the graphic. 
	 * 
	 * @param newConfigDocument an object, which implements <code>org.w3c.dom.Document</code> 
	 * and conforms to the configuration schema for the generator. 
	 */
	void setConfigDocument(Document newConfigDocument) throws DataInputProcessingException;
		
	/**
	 * Specifies the XML document which is to provide the content data for the graphic
	 * generated by this object. Note that since this information is generated into the 
	 * graphic, this method must be called before generating the graphic. 
	 * 
	 * @param newDataDocument an object, which implements <code>org.w3c.dom.Document</code> 
	 * and conforms to the content data schema for the generator. 
	 */
	void setDataDocument(Document newDataDocument) throws DataInputProcessingException;
		
	/**
	 * Specifies the height of the graphic generated by this object. If the value 
	 * cannot be resolved to an <code>int</code> zero will be substituted. Extending
	 * classes should override this behaviour and provide a default value in this
	 * case. Note that since this information is generated into the graphic, this method 
	 * must be called before generating the graphic. 
	 * 
	 * @param newGraphicHeight the pixel height of the generated graphic
	 */
	void setGraphicHeight(String newGraphicHeight);

	/**
	 * Specifies the width of the graphic generated by this object. If the value 
	 * cannot be resolved to an <code>int</code> zero will be substituted. Extending
	 * classes should override this behaviour and provide a default value in this
	 * case. Note that since this information is generated into the graphic, this method 
	 * must be called before generating the graphic. 
	 * 
	 * @param newGraphicWidth the pixel width of the generated graphic
	 */
	void setGraphicWidth(String newGraphicWidth); 
	
	/**
	 * Specifies the font to be used by the graphic generated by this object. 
	 * If the value is not a supported font, the resulting behaviour is dictated
	 * by the browser (plugin) in which the graphic is being rendered. Extending
	 * classes should override this behaviour and provide a default value in this
	 * case. Note that since this information is generated into the graphic, this 
	 * method must be called before generating the graphic. 
	 * 
	 * @param newGraphicFont the font family name
	 */
	void setGraphicFont(String newGraphicFont); 
	
	/**
	 * Specifies the name of the default colour palette to be used by the graphic generated 
	 * by this object. If the name is not recognized, it is not used. Extending classes should
	 * provide a set of documented, named, default palettes from which to choose. Note that since 
	 * this information is generated into the graphic, this method must be called before generating 
	 * the graphic. 
	 *  
	 * @param newGraphicColourPaletteName one of the documented or user-supplied names
	 * @return boolean <code>true</code>, if the supplied name is recognized	 
	 */
	boolean setGraphicColourPaletteName(String newGraphicColourPaletteName); 
	
	/**
	 * Specifies the title of the graphic generated by this object. Note that since this 
	 * information is generated into the graphic, this method must be called before 
	 * generating the graphic. 
	 * 
	 * @param newGraphicTitle the title, which will be trimmed.
	 */
	void setGraphicTitle(String newGraphicTitle); 
	
	/**
	 * Specifies the prefix for the timestamp of the graphic generated by this object. Note that 
	 * since this information is generated into the graphic, this method must be called before 
	 * generating the graphic. 
	 * 
	 * @param newGraphicTimestampPrefix the prefix, which will be trimmed and prepended to the timestamp.
	 */
	void setGraphicTimestampPrefix(String newGraphicTimestampPrefix); 	
	
	/**
	 * Specifies whether the legend should be suppressed in the graphic generated by this object.
	 * If the value passed is invalid, <code>false</code> will be substituted. Note that since this information 
	 * is generated into the graphic, this method must be called before generating the graphic. 
	 * 
	 * @param newSuppressLegend must be one of {"true", "false", "1", "0"}.
	 */
	void setSuppressLegend(String newSuppressLegend); 
	
	/**
	 * Specifies the legend title of the graphic generated by this object. Will be ignored
	 * if a legend is not supported for the graphic. Note that since this information is 
	 * generated into the graphic, this method must be called before generating the graphic. 
	 * 
	 * @param newGraphicLegendTitle the title, which will be trimmed and displayed over the legend
	 */
	void setGraphicLegendTitle(String newGraphicLegendTitle); 
	
		
	/**
	 * Specifies the folder URL on the local machine where this object is executing in
	 * which any resources needing to be accessed by the generated graphic are located. 
	 * If the full URL is not being specified, then the folder location must be relative
	 * to the location of the page which is to draw in the graphic. Note that this information 
	 * is generated into the graphic, so this method must be called before generating the 
	 * graphic.
	 * 
	 * @param newResourcesFolder the folder URL location of the resources, which will be trimmed. 	 
	 */
	void setResourcesFolder(String newResourcesFolder); 
	
	/**
	 * Specifies the user preferences page to be used. The page is expected to be 
	 * located in the resources folder; that is, the page name is appended to any 
	 * non-null value passed using "setResourcesFolder" before being generated into
	 * the graphic. Note that the if the composed location of the preferences page is
	 * not full-path, then it should be relative to the location of the page which 
	 * draws in the generated graphic. Note also that since this information is 
	 * generated into the graphic, this method must be called before generating the 
	 * graphic.
	 * 
	 * @param newPreferencesPage the folder location of the page which is to replace the
	 * existing preferences page. This value will be trimmed. 	
	 */
	void setPreferencesPage(String newPreferencesPage);
	
	/**
	 * Specifies the width for the window which displays the user preferences page. 
	 * Note that since this information is generated into the graphic, this method 
	 * must be called before generating the graphic.
	 * 
	 * @param newPreferencesPageWidth the pixel width	 
	 */
	void setPreferencesPageWidth(String newPreferencesPageWidth);
	
	/**
	 * Specifies the height for the window which displays the user preferences page. 
	 * Note that since this information is generated into the graphic, this method 
	 * must be called before generating the graphic. 
	 *  
	 * @param newPreferencesPageHeight the pixel heights	
	 */
	void setPreferencesPageHeight(String newPreferencesPageHeight);
	
	/**
	 * Adds the specified colour palette to the set of colour palettes which are available 
	 * for the graphic generated by this object. If the palette name already exists, the
	 * palette is not added. No checking is done to ensure the palette is valid or that
	 * the specified colours are supported in the target environment. Note that adding a 
	 * palette does not select that palette as the default palette. Also note that since 
	 * this information is generated into the graphic, this method must be called before 
	 * generating the graphic. 
	 * 
	 * @param paletteName the name of the palette
	 * @param palette the colours (hexadecimal codes)
	 * @return boolean <code>true</code>, if the operation is successful	
	 */
	boolean addGraphicColourPalette(String paletteName, String [] palette) throws NullPointerException;
	
	/**
	 * Generates and returns a graphic <code>Document</code> object based on input data	 
	 * 
	 * @return Document the generated graphic object
	 */
	Document generateGraphicDocument() throws DataInputProcessingException;
	
	/**
	 * Generates a graphic <code>Document</code> object based on the input data and
	 * returns it XML-serialized to a <code>String</code>. 
	 * 
	 * @return String the XML-serialized form of the generated object 
	 */
	String generateAndSerialize() throws DataInputProcessingException, IOException;
		
	/**
	 * Generates a graphic <code>Document</code> object by based on the input data and then 
	 * XML-serializes it to the specified filename on the local machine where this object
	 * is executing.
	 * 
	 * @param filename the location to where the generated object is to be XML-serialized.
	 * @return boolean <code>true</code>, if the operation is successful
	 */
	boolean generateAndSerializeTo(String filename) throws DataInputProcessingException, IOException;
		
	/**
	 * XML-serializes the generated <code>Document</code> object to the specified location
	 * on the physical machine where this object is executing.
	 * 
	 * @param filename The location to which the object is serialized.
	 * @return boolean <code>true</code>, if the operation is successful.
	 */
	boolean serializeGeneratedDocumentToFile(String filename) throws IOException; 
	
	/**
	 * XML-serializes and returns the generated <code>Document</code> object as a <code>String</code>.
	 * 
	 * @return String the XML-serialized form of the <code>Document</code>
	 */	
	String serializeGeneratedDocumentToString() throws IOException; 
	
	/**
	 * XML-serializes and returns the generated <code>Document</code> object as a byte array.
	 * 
	 * @return byte[] The XML-serialized form of the <code>Document</code>
	 */	
	byte[] serializeGeneratedDocumentToStream(); 

}

