package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.util.Hashtable;
import java.util.Vector;

import org.w3c.dom.Document;
/**
 * Data retrieval methods which retrieve data of interest to <code>SVGPieChart</code> from 
 * documents which conform to the documented data input schemas. 
 * 
 * @version %I%
 */
public class PieChartDataRetriever extends CommonDataRetriever {
	/**
	 * Sole constructor.
	 */
	PieChartDataRetriever() {
		super();
	}	
	
	/**
	 * Returns from the data document the text labels which indicate the 
	 * contextual definition of the data values and the summary (total value) 
	 * for the pie, both of which are displayed by the meter graphic.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_CONTEXT_FOUND, NO_SUMMARY_FOUND</code>.
	 * @see IDataInputProcessingExceptionCodes
	 */	
	public Hashtable getTitles() throws DataInputProcessingException {	
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
			
		Hashtable titles = new Hashtable(2);		
		try {			
			titles.put(VALUE_TITLE, getAttributeFromSingleElement(dataDocument, RANGE, CONTEXT));					
		} catch (Exception e) {
			// NullPointerException
			throwException(NO_CONTEXT_FOUND, RANGE);			
		}	
		try {			
			titles.put(TOTAL_TITLE, getAttributeFromSingleElement(dataDocument, DATAUPDATE, SUMMARY));					
		} catch (Exception e) {
			// NullPointerException
			throwException(NO_SUMMARY_FOUND, DATAUPDATE);			
		}			
		return titles;		
	}	
		
	/**
	 * Returns the ordered array of values found in the data document each of which represents
	 * the total value of a data set and each is rendered as a pie slice value. Values can be 
	 * ordered by specifying their ordinal position; otherwise, the retrieved (document) 
	 * order is used.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_TOTALS_FOUND, MISSING_TOTALS_OR_POSITIONS, 
	 * POSITION_NUMBER_OUT_OF_BOUNDS, DUPLICATE_POSITION_NUMBER, INVALID_NUMBER_FORMAT</code>
	 * @see IDataInputProcessingExceptionCodes
	 */	
	public String [] getDatasetTotals() throws DataInputProcessingException {		
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		Vector labels = getAttributeFromElements(dataDocument, DATASET, TOTAL);
		if (labels == null) {
			throwException(NO_TOTALS_FOUND, DATASET);			
		}
		int labelsSize = labels.size();	
		
		// ensure they are numbers
		for (int i = 0; i < labelsSize; i++) {
			double j = 0;
			try {
				j = Double.parseDouble((String)labels.get(i));
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, DATASET);	
			}	
			// restriction to non-negative numbers
			if (j < 0) {
				throwException(INVALID_NUMBER_FORMAT, SEGMENT_MARKER);
			}					
		}
		
		String [] result = new String [labelsSize];		
		// get the positions, if any
		Vector positions = getAttributeFromElements(dataDocument, DATASET, POSITION);
				
		// use document order
		if (positions == null) {
			for (int i = 0;i < labelsSize; i++) {
				result[i] = (String)labels.get(i);
			}
			return result;
		}		
		if (positions.size() != labelsSize) {
			throwException(MISSING_TOTALS_OR_POSITIONS, DATASET);			
		}				 		
		
		// use positions		
		for (int i = 0;i < labelsSize; i++) {
			short position = 0;			
			try {
				position = Short.parseShort((String)positions.get(i));
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, DATASET);				
			}					
			// make sure the position is within bounds						
			if (position > (labelsSize - 1)  || position < 0) {
				throwException(POSITION_NUMBER_OUT_OF_BOUNDS, DATASET);
			}				
			// make sure the position is not already taken
			String test = null;		
			try {
				test = result[position];
			}
			catch (Exception e) {}								
				
			if (test != null) {
				throwException(DUPLICATE_POSITION_NUMBER, DATASET);
			}		
			result[position] = (String)labels.get(i);		
		}						
		return result;			
	}			
	
	/**
	 * Returns the ordered array of values found in the data document each of which 
	 * represents the total value of a data set as a percentage of the sum of the total
	 * values of all the data sets. Each of these values is rendered as a pie 
	 * slice percentage value. Values can be ordered by specifying their ordinal 
	 * position; otherwise, the retrieved (document) order is used.
	 * 
	 * @throws DataInputProcessingException for codes <code>NO_DATA_DOCUMENT, NO_PERCENTAGE_OF_DATAPOINT_TOTAL_FOUND,
	 * MISSING_PERCENTAGE_OF_DATAPOINT_TOTAL_OR_POSITION, INVALID_NUMBER_FORMAT</code>
	 * @see IDataInputProcessingExceptionCodes
	 */	
	public String [] getDatasetPercentages() throws DataInputProcessingException {		
		if (dataDocument == null) {
			throwException(NO_DATA_DOCUMENT, EMPTY_STRING);
		}	
		
		Vector labels = getAttributeFromElements(dataDocument, DATASET, PERCENTAGE_OF_DATAPOINT_TOTAL);
		if (labels == null) {
			throwException(NO_PERCENTAGE_OF_DATAPOINT_TOTAL_FOUND, DATASET);				
		}			
		int labelsSize = labels.size();	
		// ensure they are numbers
		for (int i = 0; i < labelsSize; i++) {
			double j = 0;
			try {
				j = Double.parseDouble((String)labels.get(i));
			} catch (Exception exc) {
				// NumberFormatException
				throwException(INVALID_NUMBER_FORMAT, DATASET);	
			}	
			// restriction to non-negative numbers
			if (j < 0) {
				throwException(INVALID_NUMBER_FORMAT, SEGMENT_MARKER);
			}					
		}
		
		String [] result = new String [labelsSize];
				
		// get the positions, if any
		Vector positions = getAttributeFromElements(dataDocument, DATASET, POSITION);
				
		// use document order
		if (positions == null) {
			for (int i = 0;i < labelsSize; i++) {
				result[i] = (String)labels.get(i);
			}
			return result;
		}
		
		if (positions.size() != labelsSize) {
			throwException(MISSING_PERCENTAGE_OF_DATAPOINT_TOTAL_OR_POSITION, DATASET);				
		}				 		
		
		// use positions		
		for (int i = 0;i < labelsSize; i++) {
			short position = 0;			
			try {
				position = Short.parseShort((String)positions.get(i));
			} catch (Exception exc) {
				throwException(INVALID_NUMBER_FORMAT, DATASET);				
			}					
			result[position] = (String)labels.get(i);
			
		}						
		return result;			
	}	
	
}

