package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;
import org.w3c.dom.CDATASection;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.ProcessingInstruction;
/**
 * Abstract base class <code>SVGDocumentGenerator</code> implements document-building methods 
 * which are common to SVG document-based charts. 
 * 
 * @version %I%
 */
public abstract class SVGDocumentGenerator extends GraphicDocumentGenerator implements IGraphicDocumentStyle, Serializable {
	/**
	 * The SVG version is "-//W3C//DTD SVG 1.0//EN".
	 */	
	static final String SVG_VERSION = "-//W3C//DTD SVG 1.0//EN";
	
	/**
	 * The SVG DTD is "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd".
	 */	
	static final String SVG_DTD = "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd";
	
	/**
	 * The JavaScript functions conform to ECMAScript.
	 */	
	static final String JAVASCRIPT_VERSION = "text/ecmascript";
	
	/**
	 * Default font size, BASIC_FONTSIZE.
	 * 
	 * @see IGraphicDocumentStyle
	 */	
	static final int DEFAULT_FONTSIZE = BASIC_FONTSIZE;
	
	/**
	 * Default font family and generic family, BASIC_FONT.
	 * 
	 * @see IGraphicDocumentStyle
	 */	
	static final String DEFAULT_FONT = BASIC_FONT;
	
	/**
	 * Default colour palette, SEASIDE.
	 * 
	 * @see IGraphicDocumentStyle
	 */
	static final String DEFAULT_PALETTE = SEASIDE;
	
	/**
	 * Default graphic pixel width, BASIC_WITH.
	 * 
	 * @see IGraphicDocumentStyle
	 */
	static final String DEFAULT_WIDTH = BASIC_WIDTH;
	
	/**
	 * Default graphic pixel height, BASIC_HEIGHT.
	 * 
	 * @see IGraphicDocumentStyle
	 */	
	static final String DEFAULT_HEIGHT = BASIC_HEIGHT;
	
	/**
	 *  generated DOM root element
	 */
	protected Element svgRoot;
		
	
	/**
	 * Sole constructor
	 */ 
	public SVGDocumentGenerator() {
		super();
		setGraphicFont(DEFAULT_FONT);	
		setGraphicWidth(DEFAULT_WIDTH);
		setGraphicHeight(DEFAULT_HEIGHT);
		createGraphicColourPalettes();
		setGraphicColourPaletteName(DEFAULT_PALETTE);		
	}
			
	/**
	 * @see IGraphicDocumentGenerator#getPreferencesPage()
	 */
	public String getPreferencesPage() {
		return preferencesPage;
	}
	
	/**
	 * @see IGraphicDocumentGenerator#getPreferencesPageWidth()
	 */
	public String getPreferencesPageWidth() {
		return preferencesPageWidth;
	}

	/**
	 * @see IGraphicDocumentGenerator#getPreferencesPageHeight()
	 */
	public String getPreferencesPageHeight() {
		return preferencesPageHeight;
	}
	
	/**
	 * Attempts to set the graphic height to the value. On failure,
	 * sets the graphic height to the default height.
	 * 
	 * @see IGraphicDocumentGenerator#setGraphicHeight(String)
	 */
	public void setGraphicHeight(String newGraphicHeight) {
		super.setGraphicHeight(newGraphicHeight);
		short height = 0;
		try {
			height = Short.parseShort(getGraphicHeight());
		}catch (Exception e) {}
					
		if (height == 0) {
			setGraphicHeight(DEFAULT_HEIGHT);
		}	
	}
	/**
	 * Attempts to set the graphic width to the value. On failure,
	 * sets the graphic width to the default width.
	 * 
	 * @see IGraphicDocumentGenerator#setGraphicWidth(String)
	 */
	public void setGraphicWidth(String newGraphicWidth) {
		super.setGraphicWidth(newGraphicWidth);
		short width = 0;
		try {
			width = Short.parseShort(getGraphicWidth());
		}catch (Exception e) {}			
		
		if (width == 0) {
			setGraphicWidth(DEFAULT_WIDTH);
		}		
	}
	
	/**
	 * Attempts to set the graphic font to the value. On failure,
	 * sets the graphic font to the default font.
	 * 
	 * @see IGraphicDocumentGenerator#setGraphicFont(String)
	 */
	public void setGraphicFont(String newGraphicFont) {
		super.setGraphicFont(newGraphicFont.trim());
		String font = getGraphicFont();		
		if (font != null) {
			super.setGraphicFont(font = font + ", " + DEFAULT_FONT);
		}
		else {
			super.setGraphicFont(DEFAULT_FONT);
		}	
	}	
	
	/** 
	 * @see IGraphicDocumentGenerator#setPreferencesPage(String)
	 */
	public void setPreferencesPage(String newPreferencesPage) {
		preferencesPage = newPreferencesPage.trim();
	}
	
	/**
	 * @see IGraphicDocumentGenerator#setPreferencesPageWidth(String)
	 */
	public void setPreferencesPageWidth(String newPreferencesPageWidth) {
		preferencesPageWidth = newPreferencesPageWidth;
	}
	
	/**
	 * @see IGraphicDocumentGenerator#setPreferencesPageHeight(String)
	 */
	public void setPreferencesPageHeight(String newPreferencesPageHeight) {
		preferencesPageHeight = newPreferencesPageHeight;
	}
	
	/**
	 * Creates the documented colour palettes for this object's graphic.  
	 * 
	 * @see IGraphicDocumentStyle
	 */
	protected void createGraphicColourPalettes() {
		graphicColourPalette = new Hashtable(6);
		graphicColourPalette.put(SEASIDE, SEASIDE_PALETTE);
		graphicColourPalette.put(SUMMER, SUMMER_PALETTE);
		graphicColourPalette.put(EARTH, EARTH_PALETTE);
		graphicColourPalette.put(SPRING, SPRING_PALETTE);
		graphicColourPalette.put(HARVEST, HARVEST_PALETTE);
		graphicColourPalette.put(PASTEL, PASTEL_PALETTE);				
	}	
	
	/**
	 * Creates the required document elements for this object's graphic.
	 * 
	 * @param dImplement implementation to use for document creation.
	 */
	protected void createSvgDocument(DOMImplementation dImplement) {		
		DocumentType dType = dImplement.createDocumentType("svg", SVG_VERSION, SVG_DTD);
					
		generatedDocument = dImplement.createDocument(null, "svg", dType);
		svgRoot = (Element) generatedDocument.getDocumentElement();	
		
		String width = getGraphicWidth();
		String height = getGraphicHeight();
		svgRoot.setAttribute("width", width);										   	
		svgRoot.setAttribute("height", height);
		svgRoot.setAttribute("viewbox", "0 0 " + width + " " + height);	
	}	
	
	/** 
	 * Adds the JavaScript functions required by this object's graphic. 
	 * 
	 * @param parent element to which the script-related elements are appended
	 * @param numberOfDataSets the number of data sets in the input
	 */
	protected void addJavaScriptFunctions(Element parent, int numberOfDataSets) {
		boolean hasLegend = !isLegendSuppressed();
		Element newElement = generatedDocument.createElement("script");
		newElement.setAttribute("type", JAVASCRIPT_VERSION);
		parent.appendChild(newElement);
	
		ECMAScriptGenerator jscript = new ECMAScriptGenerator();
		StringBuffer functions = new StringBuffer(jscript.generateGlobalVariables());				
		functions.append(jscript.generateClientSnifferVariables());		
			
		// generate the functions		
		functions.append(jscript.generateCookieFunctions());	
		functions.append(jscript.generateStorePreferencesFunction(graphicType));
		functions.append(jscript.generateGetPreferencesFunction(graphicType));				
		
		functions.append(jscript.generateInitFunction(graphicColourPalette, getGraphicColourPaletteName(), getGraphicFont(), graphicType));				
		functions.append(jscript.generateToggleVisibilityFunction(numberOfDataSets,graphicType));	
				
		functions.append(jscript.generateUpdateStylesFunction(numberOfDataSets, graphicType, hasLegend));		
		// generate legend-specific script
		if (hasLegend) {
			functions.append(jscript.generateUpdateLegendStylesFunction(numberOfDataSets, graphicType));			
		}
		functions.append(jscript.generateGetCurrentValuesFunction());	
		functions.append(jscript.generateResetStylesFunction());	
		functions.append(jscript.generateGetPalettesFunction());		
		
		// resourcesFolder should have been set by user.
		// preference page should have been set in widget CTOR and could have been 
		// overridden by user with 'setPreferencePage' - if resourcesFolder is null, 
		// page location has to be resolvable from folder containing the html page 
		// embedding the graphic.		
		if (preferencesPage != null) {			
			functions.append(jscript.generateOpenPreferencePageFunction(resourcesFolder, preferencesPage, graphicType, preferencesPageWidth, preferencesPageHeight));		
			// This is needed for basic types preference pages (i.e., those prefixed with "prefs_") only 
			// as long as support is needed for NS 4.7
			if (preferencesPage.indexOf(IGraphicDocumentStyle.PREFS) == 0) {
				functions.append(jscript.generateUseNetscapePreferencePage(resourcesFolder, preferencesPage, graphicType, preferencesPageWidth, preferencesPageHeight));	
			}		
		}				
		
		// for debugging purposes, comment out when final
		functions.append(jscript.generateInspectFunction());		
		
		CDATASection newCDATASection = generatedDocument.createCDATASection(functions.toString());
		newElement.appendChild(newCDATASection);
	}	
	
	/**
	 * Adds a border around this object's graphic and places preference icon.
	 * 
	 * @param parent element to which the border-related elements are appended.
	 */
	protected void addBorder(Element parent) {
		StringBuffer points = new StringBuffer();	
		String sWidth = getGraphicWidth();
		String sHeight = getGraphicHeight();
		
		// NumberFormatException not possible here		
		short width = Short.parseShort(sWidth);		
				
		final int titleXOffset = 10;
		final int titleYOffset = DEFAULT_FONTSIZE + TITLE_FONTSIZE_DELTA + 6;
		
		// preference icon location & size(w=29.1, h=23.3) 
		final int iconXOffset = width - 33;
		final int iconYOffset = 2;
		
		// separate chart title from rest of graphic
		final int borderLineYOffset = 28;		
	
		Element newElement = generatedDocument.createElement("rect");
		newElement.setAttribute("class", "st1, outline");
		newElement.setAttribute("width", sWidth);
		newElement.setAttribute("height", sHeight);
		parent.appendChild(newElement);
	
		newElement = generatedDocument.createElement("polyline");
	    newElement.setAttribute("class", "st1, keyline");
		points.append("0 ");    
	    points.append(borderLineYOffset);
	    points.append(" ");
	    points.append(width);
	    points.append(" ");
	    points.append(borderLineYOffset); 
	    newElement.setAttribute("points", points.toString());
	    parent.appendChild(newElement);
	    
	    // place the preference icon within the title area
	    newElement = generatedDocument.createElement("use");
	    newElement.setAttribute("xlink:href", "#preferences");
		newElement.setAttribute("x", Integer.toString(iconXOffset));
		newElement.setAttribute("y", Integer.toString(iconYOffset)); 
		// set onclick event handler to JavaScript function  
		newElement.setAttribute("onclick", "openPreferencePage()");
		parent.appendChild(newElement);
		
	}
			
	/**
	 * Adds the preferences icon to this object's graphic.
	 * 
	 * @param parent element to which the preference icon-related elements
	 * are appended
	 */
	protected void addPreferenceIcon(Element parent) {
		Element gElement_preferences = generatedDocument.createElement("g");
		gElement_preferences.setAttribute("id", "preferences");
		parent.appendChild(gElement_preferences);
		
		Element gElement = generatedDocument.createElement("g");
		gElement.setAttribute("id", "graph");
		gElement.setAttribute("style","fill-rule:nonzero;clip-rule:nonzero;fill:none;stroke:#999999;stroke-miterlimit:4;");
		gElement_preferences.appendChild(gElement);
		
		Element newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d", "M2.042,4.811H0.5v18h21v-6.542");
		gElement.appendChild(newElement);
		  
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d", "M2.042,4.811H0.5");
		gElement.appendChild(newElement);
		    
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d", "M2.042,10.801H0.5");
		gElement.appendChild(newElement);
		    		    
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d", "M2.042,16.8H0.5");
		gElement.appendChild(newElement);
		
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d", "M6.51,22.81v-18H3.521v18H6.51z");
		newElement.setAttribute("style", "fill:#000099;");
		gElement.appendChild(newElement);
				
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d","M12.51,22.81v-11H9.521v11h2.989z");
		newElement.setAttribute("style", "fill:#0066CC;");
		gElement.appendChild(newElement);
		
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d","M18.51,22.81v-5h-2.989v5h2.989z");
		newElement.setAttribute("style", "fill:#99CCFF;");
		gElement.appendChild(newElement);
		
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d","M21.532,7.842l-5.145-5.248c-1.295,1.295-2.096,3.083-2.096,5.06c0,3.953,3.286,7.156,7.238,7.156l0.002-6.968z");
		newElement.setAttribute("style", "fill:#0066CC;");
		gElement.appendChild(newElement);
		
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d","M21.557,14.81c3.952,0,7.047-3.11,7.047-7.063h-7.047v7.063z");
		newElement.setAttribute("style", "fill:#0066CC;");
		gElement.appendChild(newElement);
		
		newElement = generatedDocument.createElement("path");
		newElement.setAttribute("d","M28.604,7.654c0-3.953-3.204-7.156-7.156-7.156c-1.977,0-3.766,0.801-5.061,2.097l5.092,5.278h7.125");
		newElement.setAttribute("style", "fill:#99CCFF;");
		gElement.appendChild(newElement);
	
	}
	
	/**
	 * Adds the main title to this object's graphic for display
	 * in the border region.
	 * 
	 * @param parent element to which the title-related elements are appended
	 */
	protected void addTitle(Element parent) {		
		// add a font group for the title and then add the title		
		String title = getGraphicTitle();
		if (title != null) {
			String stylesheetClass = "graphTitle anchorAtStart";
			Element gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id", "graphTitle");
			gElement.setAttribute("class", stylesheetClass);
			gElement.setAttribute("font-family", getGraphicFont());
			parent.appendChild(gElement);				
			// default title font size
			int fontSize = DEFAULT_FONTSIZE + TITLE_FONTSIZE_DELTA;
			addLabel(gElement, title, null, 10, fontSize + 8, 0);
		}
			
	}	
	
	/**
	 * Adds the timestamp to this object's graphic
	 * 
	 * @param parent	element to which the timestamp-related elements are appended
	 * @param timestamp	timestamp (includes date)
	 */	
	protected void addTimeStamp(Element parent, String timestamp) {				
		String prefix = getGraphicTimestampPrefix();
		if (prefix != null) {
			timestamp = prefix + " " + timestamp;
		}	
		
		// font group		
		Element gElement = generatedDocument.createElement("g");
		gElement.setAttribute("id", "timeStamp");
		gElement.setAttribute("class", "timeStamp");
		gElement.setAttribute("font-family", getGraphicFont());
		
		// NumberFormatException not possible here
		double fontX = Double.parseDouble(getGraphicWidth()) - 5;
		double fontY = Double.parseDouble(getGraphicHeight()) - (double)DEFAULT_FONTSIZE;
		addLabel(gElement, timestamp, null, fontX, fontY, 0);
		
		parent.appendChild(gElement);
	}
	
	/**
	 * Adds the legend to this object's graphic.
	 * 
	 * @param parent	element to which the legend-related elements are appended 
	 * @param labels	legend text labels
	 * @param xOffset	starting x position
	 * @param yOffset	starting y position
	 */	
	protected void addLegend(Element parent, String [] labels, int xOffset, int yOffset) {
		int numberOfLabels = labels.length;
		
		// from IDocumentStyle - legendTitle, legendData classes
		int legendTitleFontSize = DEFAULT_FONTSIZE + 2;
		int legendLabelFontSize = DEFAULT_FONTSIZE + 1;
		
		// from definitions squareshape width
		int shapeWidth = 11;
		
		// eyeball positioning
		int legendMargin = 15;
		int legendLabelMargin = shapeWidth + 6;
		int delta = 16;
		
		int rotation = 0;
		int x = xOffset + legendMargin;
		int y = yOffset + legendTitleFontSize;
		// set up for label positions
		int legendLabelStartY = yOffset - legendLabelFontSize + 1;
		
		String title = getGraphicLegendTitle();
		if (title != null) {
			// add font group for legend title			
			Element gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id", "legendTitle");
			gElement.setAttribute("class", "legendTitle");
			gElement.setAttribute("font-family", getGraphicFont());
			parent.appendChild(gElement);				
			// add title to font group
			addLabel(gElement, title, null, x, y, rotation);
			// adjust for label positions
			legendLabelStartY = yOffset + legendTitleFontSize + 7;
		}
		// font group for legend labels		
		Element gElement = generatedDocument.createElement("g");
		gElement.setAttribute("id", "legendData");
		gElement.setAttribute("class", "legendData");
		gElement.setAttribute("font-family", getGraphicFont());
		parent.appendChild(gElement);	
	
		
		// add color squares and legend labels
		for (int i = 0, yPrevious = legendLabelStartY ; i < numberOfLabels; i++) {
 			addLegendShapes(parent, i, legendLabelFontSize, x, yPrevious);			
			// bump label over by margin of 15
			// labels are placed using lower left corner
			yPrevious += delta;
			// font style set on group				
			addLabel(gElement, labels[i], null, x + legendLabelMargin, yPrevious - 7, rotation);
		}
	}
	
	/**
	 * Adds the legend shapes to this object's graphic.
	 * 
	 * @param parent	element to which the legend-related elements are appended 
	 * @param index	dataset number
	 * @param fontSize	y shift factor
	 * @param x		upper left x
	 * @param y 		upper left y 
	 */	
	protected void addLegendShapes(Element parent, int index, int fontSize, int x, int y) {
		Element newElement, gColourElement;
		newElement = generatedDocument.createElement("use");
		newElement.setAttribute("xlink:href", "#legendsquareshapeWithoutLine");
		newElement.setAttribute("x", Integer.toString(x));
		// rectangles are placed using upper left corner
		newElement.setAttribute("y", Integer.toString(y + fontSize - 9));    
			
		// add the element to its color group for user preference changes
		gColourElement = generatedDocument.createElement("g");
		gColourElement.setAttribute("id", "legendsquareshapeWithoutLine" + index);
		gColourElement.setAttribute("class", "shape" + index);		
		if (graphicType.equals(IGraphicTypeConstants.LINE_CHART)) {
			gColourElement.setAttribute("onclick","toggleVisibility('dataColour" + index + "');");				
		}
		gColourElement.appendChild(newElement);
		parent.appendChild(gColourElement);	
	}
		
	/**
	 * Convenience method for adding text elements to this object's graphic.
	 * 
	 * @param parent element to which the label-related elements are appended.
	 * @param stylesheetClass CSS rule for this element
	 * @param x x location for the label
	 * @param y y location for the label
	 * @param rotation the rotation value (0 - 360)
	 */
	protected Element addLabel(Element parent, String title, String stylesheetClass, double x, double y, int rotation) {		
		Element newElement = createDOMTextElement(title, stylesheetClass, Double.toString(x), Double.toString(y), rotation);
		parent.appendChild(newElement);   		
		return newElement;
	}	
		
	/**
	 * Convenience method for creating DOM text nodes.
	 * 
	 * @param text text of the element
	 * @param styleClass CSS rule for the element
	 * @param x The x location for the text
	 * @param y The y location for the text
	 * @param rotation the rotation value (0 - 360)
	 */
	protected Element createDOMTextElement(String text, String styleClass, String x, String y, int rotation) {
		Element newElement = generatedDocument.createElement("text");
		if (styleClass != null) {
	   		newElement.setAttribute("class", styleClass);
		}
		newElement.setAttribute("font", getGraphicFont());
	    newElement.setAttribute("x", x);
	    newElement.setAttribute("y", y);
	    newElement.setAttribute("textLength", Integer.toString(text.length()));
	    if (rotation > 0 && rotation < 360) {
	    	newElement.setAttribute("transform", "rotate(" + rotation + "," + x + "," + y + ")");
	    }	
	    newElement.appendChild(generatedDocument.createTextNode(text));
	    return newElement;
	}	
}
