package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * <code>SVGLineChart</code> generates a line chart graphic DOM using Scalable Vector Graphics (SVG).
 * 
 * @version %I%
 */
public class SVGLineChart extends SVGXYChart implements IGraphicDocumentStyle, IDataInputProcessingExceptionCodes, Serializable {
	/**
	 * Sole constructor
	 */ 
	public SVGLineChart() {
		super();
		graphicType = IGraphicTypeConstants.LINE_CHART;
		setGraphicWidth(LINE_DEFAULT_WIDTH);
		setPreferencesPage(LINE_PREFS);
		setPreferencesPageWidth(PREFS_WIDTH);
		setPreferencesPageHeight(LINE_PREFS_HEIGHT);					
	}
		
	/**
     * @see IGraphicDocumentGenerator#generateGraphicDocument()
     */       
	public Document generateGraphicDocument() throws DataInputProcessingException {									
		String rangeMaxValue = null;
		String [] rangeMarkers = null;
		String [] categories = null;	
		int xTicks = 0, yTicks = 0, numberOfDatasets = 0;		
	
		// make sure there is some data, or exit
		try {
			numberOfDatasets = dataRetriever.getNumberOfDatasets();
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;		
		}		
		
		// !DO NOT REORDER THESE METHOD CALLS UNLESS NECESSARY!!
		createSvgDocument(dataDocument.getImplementation());
		
		// add the "onload" event handler function
		svgRoot.setAttribute("onload", "init()");
		
		addJavaScriptFunctions(svgRoot, numberOfDatasets);
 		addDefinitions(svgRoot, numberOfDatasets);
	    addBorder(svgRoot);
  
		try {
			rangeMarkers = dataRetriever.getRangeMarkers();
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;				
		}					
		
		if (rangeMarkers != null && rangeMarkers.length > 0) { 
			rangeMaxValue = (String)rangeMarkers[rangeMarkers.length - 1];
		}		
		yTicks = rangeMarkers.length;
		
		try {
			categories = dataRetriever.getCategories();
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;				
		}			
		xTicks = categories.length;
		
        addGrid(svgRoot, xTicks, yTicks, rangeMarkers);
        addAxisLabels(svgRoot, categories, rangeMarkers);
        addDataSets(svgRoot, dataRetriever, numberOfDatasets, rangeMaxValue, graphicType);
		
		try {
			Hashtable titles = dataRetriever.getTitles();		
 		    addTitles(svgRoot, titles);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());			
			throw e;
		}
		
		try {
			String timestamp = dataRetriever.getTimestamp();	
        	addTimeStamp(svgRoot, timestamp);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;
		}
		        
        // omit the legend?
        if (!isLegendSuppressed()) {      		
	  		try {
	       		String [] legendLabels = dataRetriever.getLegendLabels();		       
				addLegend(svgRoot, legendLabels, GRIDXOFFSET + xAxisLength, GRIDYOFFSET); 
	  		} catch (DataInputProcessingException e) {
	  			e.setWidgetType(graphicType);
	  			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());	  			
	  			throw e;
	  		} 		
    	}	      	
        return super.generateGraphicDocument();
	}	
	
	/**
     * @see SVGDocumentGenerator#addLegendShapes()
     */ 
	protected void addLegendShapes(Element parent, int index, int fontSize, int x, int y) {
		final String[] legendShapeType = {"#legendcircleshape", "#legendsquareshape", "#legendhorizontalrectshape", "#legendverticalrectshape", "#legendtriangleshape", "#legenddiamondshape"};
		Element useElementShape, useElementLine, gColourElement;
		
		// add the standard shape to the legend
		super.addLegendShapes(parent, index, fontSize, x, y);
		
 		useElementShape = generatedDocument.createElement("use");
		useElementShape.setAttribute("xlink:href", legendShapeType[index % (legendShapeType.length)]);
		useElementShape.setAttribute("x", Integer.toString(x));
		// rectangles are placed using upper left corner
		useElementShape.setAttribute("y", Integer.toString(y + fontSize - 8));    
		
		// add the element to its color group for user preference changes
		gColourElement = generatedDocument.createElement("g");
		gColourElement.setAttribute("id", "shape" + index);		
		gColourElement.setAttribute("onclick","toggleVisibility('dataColour" + index + "');");			
		gColourElement.setAttribute("class", "shape" + index);	
		gColourElement.appendChild(useElementShape);
		parent.appendChild(gColourElement);	
	}
	
	// specific to line chart 
	void addDataSets(Element parent, XYChartDataRetriever input, int numberOfLines, String yMax, String type) throws DataInputProcessingException {
		// NumberFormatException not possible here
		double yLabelsMaxValue = Double.parseDouble(yMax);
		String [] yValues = null;		
		Vector shapeYPosVec = new Vector();
		Vector yValueVec = new Vector();

		// add lines
		// need to compute y values for actual datapoints based on the number of pixels 
		// between each potential datapoint along y-axis		
		try {
			for (int i = 0; i < numberOfLines; i++) {
			 	yValues = input.getDatapointValuesFromDatasetWithPosition(String.valueOf(i));	
			 	if (yValues.length != xAxisValues.length) {
			 		// missing category 
					short code = MISSING_CATEGORY_NAMES;				
					String element = CATEGORY;				
					DataInputProcessingException e = new DataInputProcessingException(code, element);
					e.setWidgetType(type);
					throw e;
			 	}		
			 			 	
			 	// in here we add the line and compute the locations of the text boxes 		
		  	 	addLine(parent, i, (yAxisLength/yLabelsMaxValue), yValues, yValueVec, shapeYPosVec);
			}			
			// add exact values once we know the location of the boxes (so they line up properly)
			for (int i = 0; i < numberOfLines; i++) {				
				yValues = input.getDatapointValuesFromDatasetWithPosition(String.valueOf(i));					
				// new stuff
				String stylesheetClass = "dataValues anchorAtMiddle";		
				Element gToggleElement = generatedDocument.createElement("g");
				gToggleElement.setAttribute("id","textbox" + i);
				gToggleElement.setAttribute("class", stylesheetClass);
				gToggleElement.setAttribute("font-family", getGraphicFont());
				gToggleElement.setAttribute("visibility", "hidden");
				gToggleElement.setAttribute("onclick", "toggleVisibility('textbox" + i + "');");
				parent.appendChild(gToggleElement);	
				// end: added by Henry 				
				addTextBoxes(gToggleElement, i, (yAxisLength/yLabelsMaxValue), yValues, (Vector)yValueVec.get(i), (Vector)shapeYPosVec.get(i));
			}				
		} catch (DataInputProcessingException e) {
			e.setWidgetType(type);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;
		}
	}
	
	// add style element with default style sheet rules for the line chart
	void addDefaultInternalStylesheet(Element parent, int sets) {
		StringBuffer styleRules = new StringBuffer(COMMON_STYLE);
		styleRules.append(XYCHART_STYLE);
				
		String [] palette = (String [])graphicColourPalette.get(graphicColourPaletteName);
		// palette has limited number of colours, so loop if more sets
		int paletteLength = palette.length;		
		for (int i = 0, j = 0; i < sets; i++) {
		  j = (i + 1) % paletteLength;
		  j = (j == 0) ? paletteLength - 1 : j-1;			
		  styleRules.append(" .dataset" + i + "{stroke-linecap:round;stroke-width:0.75pt; stroke:" + palette[j] + ";}"); 
		  if (!isLegendSuppressed()) {
			  styleRules.append(" .shapeline" + i + "{stroke-width:0.75pt;stroke:" + palette[j] + ";fill:" + palette[j] + ";}");
			  styleRules.append(" .shape" + i + "{ fill:" + palette[j] + ";}");
		  }
		}
		
		CDATASection newCDATASection = generatedDocument.createCDATASection(styleRules.toString());
		
		Element newElement = generatedDocument.createElement("style");
		newElement.setAttribute("id", "defaultStyleRules");
		newElement.setAttribute("type", "text/css");
		newElement.appendChild(newCDATASection);
		parent.appendChild(newElement);
	}	
	
	// add defs element with line chart definitions 
	void addDefinitions(Element parent, int numberOfDatasets) {
		Element defsElement = generatedDocument.createElement("defs");
        parent.appendChild(defsElement);
        
        // default internal style rules       
		addDefaultInternalStylesheet(defsElement, numberOfDatasets);		
		
		
		if (!isLegendSuppressed()) {
			// id: legendsquareshapeWithoutLine
			Element gElement_legendsquareWithoutLine = generatedDocument.createElement("g");	
			defsElement.appendChild(gElement_legendsquareWithoutLine);
			gElement_legendsquareWithoutLine.setAttribute("id","legendsquareshapeWithoutLine");
			
	    	Element legendsquareElementWithoutLine = generatedDocument.createElement("rect");
	    	gElement_legendsquareWithoutLine.appendChild(legendsquareElementWithoutLine);
			legendsquareElementWithoutLine.setAttribute("width", "8");
			legendsquareElementWithoutLine.setAttribute("height", "8");	
			legendsquareElementWithoutLine.setAttribute("transform", "translate(3,2)");
			
			// id: legendsquareshape
			Element gElement_legendsquare = generatedDocument.createElement("g");	
			defsElement.appendChild(gElement_legendsquare);
			gElement_legendsquare.setAttribute("id","legendsquareshape");
			
	    	Element legendsquareElement = generatedDocument.createElement("rect");
	    	gElement_legendsquare.appendChild(legendsquareElement);
			legendsquareElement.setAttribute("width", "8");
			legendsquareElement.setAttribute("height", "8");	
			legendsquareElement.setAttribute("transform", "translate(3,0)");
			gElement_legendsquare.appendChild(addLegendShapeLine());
	
			// id: circleshape
			Element gElement_legendcircle = generatedDocument.createElement("g");
			defsElement.appendChild(gElement_legendcircle);
			gElement_legendcircle.setAttribute("id", "legendcircleshape");
			
			Element legendcircleElement = generatedDocument.createElement("circle");
			gElement_legendcircle.appendChild(legendcircleElement);
			legendcircleElement.setAttribute("r", "4");
			legendcircleElement.setAttribute("cx", "4");
			legendcircleElement.setAttribute("cy", "4");
			legendcircleElement.setAttribute("transform", "translate(3,0)");
			gElement_legendcircle.appendChild(addLegendShapeLine());
			
	
			// id: diamondshape
			Element gElement_legenddiamond = generatedDocument.createElement("g");
			defsElement.appendChild(gElement_legenddiamond);
			gElement_legenddiamond.setAttribute("id", "legenddiamondshape");
			
			Element legenddiamondElement = generatedDocument.createElement("polygon");
			gElement_legenddiamond.appendChild(legenddiamondElement);
			legenddiamondElement.setAttribute("points", "0,4 4,0 8,4 4,8");
			legenddiamondElement.setAttribute("transform", "translate(3,0)");
			gElement_legenddiamond.appendChild(addLegendShapeLine());
			
			
			// id: horizontalrectshape
			Element gElement_legendhorizontalrectshape = generatedDocument.createElement("g");
			defsElement.appendChild(gElement_legendhorizontalrectshape);
			gElement_legendhorizontalrectshape.setAttribute("id", "legendhorizontalrectshape");
			
			Element legendhorizontalrectElement = generatedDocument.createElement("rect");
			gElement_legendhorizontalrectshape.appendChild(legendhorizontalrectElement);
			legendhorizontalrectElement.setAttribute("width", "8");
			legendhorizontalrectElement.setAttribute("height", "4");
			legendhorizontalrectElement.setAttribute("transform", "translate(3,2)");
			gElement_legendhorizontalrectshape.appendChild(addLegendShapeLine());
			
			// id: verticalrectshape
			Element gElement_legendverticalrectshape = generatedDocument.createElement("g");
			defsElement.appendChild(gElement_legendverticalrectshape);
			gElement_legendverticalrectshape.setAttribute("id", "legendverticalrectshape");
			
			Element legendverticalrectElement = generatedDocument.createElement("rect");
			gElement_legendverticalrectshape.appendChild(legendverticalrectElement);
			legendverticalrectElement.setAttribute("width", "4");
			legendverticalrectElement.setAttribute("height", "8");
			legendverticalrectElement.setAttribute("transform", "translate(5,0)");	
			gElement_legendverticalrectshape.appendChild(addLegendShapeLine());
			
			
			// id: triangleshape 
			Element gElement_legendtriangleshape = generatedDocument.createElement("g");
			defsElement.appendChild(gElement_legendtriangleshape);
			gElement_legendtriangleshape.setAttribute("id", "legendtriangleshape");
			
			Element legendtriangleElement = generatedDocument.createElement("polygon");
			legendtriangleElement.setAttribute("points", "4 0, 0 8, 8 8");
			legendtriangleElement.setAttribute("transform", "translate(3,0)");
			gElement_legendtriangleshape.appendChild(legendtriangleElement);		
			gElement_legendtriangleshape.appendChild(addLegendShapeLine());
			
		}			
			// id: squareshape
		Element gElement_square = generatedDocument.createElement("g");	
		defsElement.appendChild(gElement_square);
		gElement_square.setAttribute("id","squareshape");
			
	    Element squareElement = generatedDocument.createElement("rect");
	    gElement_square.appendChild(squareElement);
		squareElement.setAttribute("width", "5.6");
		squareElement.setAttribute("height", "5.6");		
			
			
		// id: circleshape
		Element gElement_circle = generatedDocument.createElement("g");
		defsElement.appendChild(gElement_circle);
		gElement_circle.setAttribute("id", "circleshape");
			
		Element circleElement = generatedDocument.createElement("circle");
		gElement_circle.appendChild(circleElement);
		circleElement.setAttribute("r", "2.8");
		circleElement.setAttribute("cx", "2.8");
		circleElement.setAttribute("cy", "2.8");
	
		// id: diamondshape
		Element gElement_diamond = generatedDocument.createElement("g");
		defsElement.appendChild(gElement_diamond);
		gElement_diamond.setAttribute("id", "diamondshape");
		
		Element diamondElement = generatedDocument.createElement("polygon");
		gElement_diamond.appendChild(diamondElement);
		diamondElement.setAttribute("points", "0,2.8 2.8,0 5.6,2.8 2.8,5.6");
			
		// id: horizontalrectshape
		Element gElement_horizontalrectshape = generatedDocument.createElement("g");
		defsElement.appendChild(gElement_horizontalrectshape);
		gElement_horizontalrectshape.setAttribute("id", "horizontalrectshape");
			
		Element horizontalrectElement = generatedDocument.createElement("rect");
		gElement_horizontalrectshape.appendChild(horizontalrectElement);
		horizontalrectElement.setAttribute("width", "5.6");
		horizontalrectElement.setAttribute("height", "2.8");
			
			
		// id: verticalrectshape
		Element gElement_verticalrectshape = generatedDocument.createElement("g");
		defsElement.appendChild(gElement_verticalrectshape);
		gElement_verticalrectshape.setAttribute("id", "verticalrectshape");
			
		Element verticalrectElement = generatedDocument.createElement("rect");
		verticalrectElement.setAttribute("width", "2.8");
		verticalrectElement.setAttribute("height", "5.6");		
		gElement_verticalrectshape.appendChild(verticalrectElement);
			
		// id: triangleshape 
		Element gElement_triangleshape = generatedDocument.createElement("g");
		defsElement.appendChild(gElement_triangleshape);
		gElement_triangleshape.setAttribute("id", "triangleshape");
			
		Element triangleElement = generatedDocument.createElement("polygon");
		triangleElement.setAttribute("points", "2.8 0, 0 5.6, 5.6 5.6");
		gElement_triangleshape.appendChild(triangleElement);		
				
		addPreferenceIcon(defsElement);
	}	
	
	Element addLegendShapeLine() {
		Element lineElement = generatedDocument.createElement("polyline");
		lineElement.setAttribute("points", "0 4, 14 4");	
		return lineElement;
	}
	
	// Add a set of polyline elements for each dataset
	void addLine(Element parent, int linesetNumber, double yUnits, String [] input, Vector yValueVec, Vector shapeYPosVec) {
		final String[] shapeType = {"#circleshape", "#squareshape", "#horizontalrectshape", "#verticalrectshape", "#triangleshape", "#diamondshape"};
		final double[] shapeWidth = {5.6, 5.6, 5.6, 2.8, 5.6, 5.6};
		final double[] shapeHeight = {5.6, 5.6, 2.8, 5.6, 4.85, 5.6};
		final double BOX_HEIGHT = 18.0;

		int length = input.length;
		double xZero = GRIDXOFFSET;
		double yZero = GRIDYOFFSET + yAxisLength;
		
		double xPrevious = xZero;
		double yPrevious = yZero;
		double xNext = 0;
		double yNext = 0;
		
		Vector tmpYValueVec = new Vector();
		Vector tmpShapeYPosVec = new Vector();
		
		// group for data colour changes and toggling exact values visibility
		Element gColourElement = generatedDocument.createElement("g");
		gColourElement.setAttribute("id", "dataColour" + linesetNumber);
		gColourElement.setAttribute("class", "dataset" + linesetNumber);
		gColourElement.setAttribute("visibility", "visible");
		gColourElement.setAttribute("onclick", "toggleVisibility('textbox" + linesetNumber + "');");
		
		parent.appendChild(gColourElement);
		
		// draw a polyline with data points
		for (int i = 0; i < length; i++) {
			xNext = xAxisValues[i];
			String s = input[i]; // exact y value
			// NumberFormatException not possible here
			//tmpYValueVec.add(new Double(s));

			// NumberFormatException not possible here
			double y = yUnits*(Double.parseDouble(s));   // y position on the graph
			yNext = yZero - y;
			tmpYValueVec.add(new Double(yNext));
			
			Element newElement = generatedDocument.createElement("polyline");
			String points = xPrevious + " " + yPrevious + "," + xNext + " " + yNext;
			newElement.setAttribute("points", points);
			// add new polyline to group element	
			gColourElement.appendChild(newElement);
			
			// Add shapes				
			int shapeNumber = linesetNumber % shapeType.length;
			
			double shapeX = xNext - shapeWidth[shapeNumber] / 2.0;
			if (shapeX <= xZero) { // adjust shape x position if it crosses the left boundary
				shapeX += (shapeX - xZero);	
			} else if (shapeX + shapeWidth[shapeNumber] > xZero + xAxisLength) { // adjust shape x position if it crosses the right boundary
				shapeX -= (shapeX + shapeWidth[shapeNumber] - (xZero + xAxisLength));	
			}
			
			double shapeY = yNext - shapeHeight[shapeNumber] / 2.0;
			if (shapeY + shapeHeight[shapeNumber] >= yZero) {  // adjust the shape y position if it crosses the x-axis
				shapeY -= shapeY + shapeHeight[shapeNumber] - yZero;	
				shapeY--;
			}
			
			Element shapeElement = generatedDocument.createElement("use");
			shapeElement.setAttribute("xlink:href", shapeType[shapeNumber]);
			shapeElement.setAttribute("transform", "translate(" + shapeX + "," + shapeY + ")");
			shapeElement.setAttribute("visibility", "visible");
			gColourElement.appendChild(shapeElement);
						
			// add shapes on the data point
			xPrevious = xNext;
			yPrevious = yNext;	
			
			tmpShapeYPosVec.add(new Double(yNext - 3.25));

		}		
		shapeYPosVec.add(new Vector(tmpShapeYPosVec));
		yValueVec.add(new Vector(tmpYValueVec));
	}	
	// add a textbox for each datapoint		
	void addTextBoxes(Element parent, int id, double yUnits, String [] input, Vector yValueVec, Vector shapeYPosVec) {
		int length = input.length;
		int xZero = GRIDXOFFSET;
		int yZero = GRIDYOFFSET + yAxisLength;
	
		for (int i = 0; i < length; i++) {
			String exactValue = input[i]; 
			// determine text box position and dimensions	
					
			double textWidth = exactValue.length();
			double boxWidth = 16 + (textWidth - 1) * 6.0;
			double boxHeight = 18;
			
			double xPos = xAxisValues[i];
			double yPos = ((Double)yValueVec.get(i)).doubleValue();
			double shapeYPos = ((Double)shapeYPosVec.get(i)).doubleValue();
			
			// Set the box x, y position 	
			double boxXPos = xPos;
			double boxYPos = yPos;
			
			String crossXAxisFlag = "0";
			// Adjust text box positions if they cross the x-axis	
			if (yPos + boxHeight > yZero) { 
				boxYPos = yPos - boxHeight;					
				crossXAxisFlag = "1";
			} 
			// add the textbox					
			addEachTextBox(parent, boxXPos, boxYPos, boxWidth, boxHeight, exactValue, id, i, crossXAxisFlag);
		}	
	}		
	
}

