package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;

import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * <code>SVGMeter</code> generates a meter graphic DOM using Scalable Vector Graphics (SVG).
 * 
 * @version %I%
 */
public class SVGMeter extends SVGDocumentGenerator implements IGraphicDocumentStyle, IDataInputConstants, IDataInputProcessingExceptionCodes, Serializable {
	private MeterDataRetriever dataRetriever;
	
	/**
	 *  Sole constructor.
	 */
	public SVGMeter() {
		super();
		graphicType = IGraphicTypeConstants.METER;
		setGraphicWidth(METER_DEFAULT_WIDTH);
		setPreferencesPage(METER_PREFS);
		setPreferencesPageWidth(PREFS_WIDTH);
		setPreferencesPageHeight(METER_PREFS_HEIGHT);
		dataRetriever = new MeterDataRetriever();
	}
	
	/**
	 * Sets the document which is to be used for configuration data 
	 * retrieval and then retrieves the configuration data from that
	 * document. 
	 * 
	 * @see IGraphicDocumentGenerator#setConfigDocument(Document)
	 */
	public void setConfigDocument(Document newConfigDocument) throws DataInputProcessingException {
		super.setConfigDocument(newConfigDocument);
		dataRetriever.setConfigDocument(getConfigDocument());
		if (getConfigDocument() == null) {
			return;
		}	
		Hashtable configProperties = dataRetriever.getConfigurationAttributes();
		if (configProperties == null) {
			return;
		}		
		String width = (String)configProperties.get(WIDTH);
		String height = (String)configProperties.get(HEIGHT);
		String fontFamily = (String)configProperties.get(FONT);
		String title = (String)configProperties.get(TITLE);
		String timestampPrefix = (String)configProperties.get(TIMESTAMP_PREFIX);
							
		if (width != null) {
			setGraphicWidth(width);
		}	
		
		if (height != null) {
			setGraphicHeight(height);
		}	
		
		if (fontFamily != null) {
			setGraphicFont(fontFamily);			
		}			
		
		if (title != null) {
			setGraphicTitle(title);			
		}
			
		if (timestampPrefix != null) {
			setGraphicTimestampPrefix(timestampPrefix);			
		}			
	}
	
	/**
	 * Sets the document which is to be used for data update 
	 * retrieval and then, if <code>configDocument</code> has not been
	 * set, retrieves the configuration data from that document.
	 * 
	 * @see IGraphicDocumentGenerator#setDataDocument(Document)
	 */
	public void setDataDocument(Document newDataDocument) throws DataInputProcessingException {
		super.setDataDocument(newDataDocument);
		dataRetriever.setDataDocument(getDataDocument());
		
		// if we already have them, no need to get them
		if (getConfigDocument() != null) {
			return;
		}	
		
		Hashtable configProperties = dataRetriever.getConfigurationAttributes();
		if (configProperties == null) {
			return;
		}		
		String width = (String)configProperties.get(WIDTH);
		String height = (String)configProperties.get(HEIGHT);
		String fontFamily = (String)configProperties.get(FONT);
		String title = (String)configProperties.get(TITLE);
		String timestampPrefix = (String)configProperties.get(TIMESTAMP_PREFIX);
										
		if (width != null) {
			setGraphicWidth(width);
		}	
		
		if (height != null) {
			setGraphicHeight(height);
		}	
		
		if (fontFamily != null) {
			setGraphicFont(fontFamily);			
		}			
		
		if (title != null) {
			setGraphicTitle(title);			
		}
			
		if (timestampPrefix != null) {
			setGraphicTimestampPrefix(timestampPrefix);			
		}			
	}
	
	/**
	 * @see IGraphicDocumentGenerator#generateGraphicDocument()
	 */	
	public Document generateGraphicDocument() throws DataInputProcessingException {	        
		// !DO NOT REORDER THESE METHOD CALLS UNLESS NECESSARY!!		
		// create the document	
		if (dataDocument == null) {
				short code = NO_DATA_DOCUMENT;				
				String element = EMPTY_STRING;								
				DataInputProcessingException e = new DataInputProcessingException(code, element);
				e.setWidgetType(graphicType);
				throw e;
		}		
		createSvgDocument(dataDocument.getImplementation());
		
		// add the "onload" event handler function
		svgRoot.setAttribute("onload", "init()");		
		
		addJavaScriptFunctions(svgRoot, 0);
 		addDefinitions(svgRoot);
	   	addBorder(svgRoot);  
		addTitle(svgRoot);	
		
		// if bad data, exit
	   	try {
			Hashtable labels = dataRetriever.getLabels();		
			String [] criticalDivisions = dataRetriever.getRangeMarkers();				
			String needleValue = dataRetriever.getNeedleValue();
			addDial(svgRoot, labels, criticalDivisions, needleValue);  		    
		} catch (DataInputProcessingException e) {			
			e.setWidgetType(graphicType);			
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());	
			throw e;		
		}		
		
		try {
			String timestamp = dataRetriever.getTimestamp();	
        	addTimeStamp(svgRoot, timestamp);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);	
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;
		}  	
	   			
        return super.generateGraphicDocument();		
	}	
			
		
	// add style element with default style sheet rules for the meter
	void addDefaultInternalStylesheet(Element parent) {
		StringBuffer styleRules = new StringBuffer(COMMON_STYLE);
		styleRules.append(" ");
		styleRules.append(METER_STYLE);
		
		CDATASection newCDATASection = generatedDocument.createCDATASection(styleRules.toString());
		
		Element newElement = generatedDocument.createElement("style");
		newElement.setAttribute("id", "defaultStyleRules");
		newElement.setAttribute("type", "text/css");
		newElement.appendChild(newCDATASection);
		parent.appendChild(newElement);
	}	
	
	// add defs element with meter definitions 
	void addDefinitions(Element parent) {
		Element defsElement = generatedDocument.createElement("defs");
        parent.appendChild(defsElement);
        
        // if no stylesheet specified, use our default internal style rules
        //if (userStylesheet == null) {
		  addDefaultInternalStylesheet(defsElement);		
		//}
		
		// id: tickline		
		Element gElement_tick = generatedDocument.createElement("g");
		gElement_tick.setAttribute("id", "tickline");
		
		Element tickElement = generatedDocument.createElement("polyline");
		tickElement.setAttribute("class", "grid");
		tickElement.setAttribute("points", "0 0 5 0");
		gElement_tick.appendChild(tickElement);
		defsElement.appendChild(gElement_tick);
		
		
		addPreferenceIcon(defsElement);
	}	
	
	// add the dial
	void addDial(Element parent, Hashtable titles, String [] divisions, String needle)  throws DataInputProcessingException {
		double x, y;
		Element newElement, gElement;

	
		// centre points for everything
		// NumberFormatException not possible here
		int gWidth = Integer.parseInt(getGraphicWidth());
		int gHeight = Integer.parseInt(getGraphicHeight());
		double cx = gWidth / 2;
		double cy = gHeight / 2 + 5;		
		
		// get the smaller value of graphicHeight/graphicWidth
		int baseLength = gHeight >= gWidth ? gWidth : gHeight; 
		
		// outer rim
		newElement = generatedDocument.createElement("circle");
		newElement.setAttribute("class", "st1 outerRim");
		newElement.setAttribute("cx", Double.toString(cx));
		newElement.setAttribute("cy", Double.toString(cy));
		newElement.setAttribute("r", Double.toString(baseLength * 0.360));		
		parent.appendChild(newElement);

		// inner rim
		newElement = generatedDocument.createElement("circle");
		newElement.setAttribute("class", "st1 innerRim");
		newElement.setAttribute("cx", Double.toString(cx));
		newElement.setAttribute("cy", Double.toString(cy));
		newElement.setAttribute("r", Double.toString(baseLength * 0.352));			
		parent.appendChild(newElement);

		// meter background
		newElement = generatedDocument.createElement("circle");
		newElement.setAttribute("class", "st1 speedometerBg");
		newElement.setAttribute("cx", Double.toString(cx));
		newElement.setAttribute("cy", Double.toString(cy));
		newElement.setAttribute("r", Double.toString(baseLength * 0.344));
		newElement.setAttribute("fill", "#ffffff");		
		parent.appendChild(newElement);

		// actual rate box
		double boxWidth = baseLength * 0.262;
		double boxYOffset = baseLength * 0.1;
		short boxHeight = 14;
		newElement = generatedDocument.createElement("rect");
		newElement.setAttribute("class", "st1 actualRate");
		newElement.setAttribute("width", Double.toString(boxWidth));
		newElement.setAttribute("height", Short.toString(boxHeight));
		newElement.setAttribute("x", Double.toString(cx - (boxWidth/2)));
		newElement.setAttribute("y", Double.toString(cy + boxYOffset));		
		parent.appendChild(newElement);

		// actual rate		
		String actualRate = (String)titles.get(ACTUAL_RATE);	
		if (actualRate != null) {
			// group for font changes			
			gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id","minmaxRate");
			gElement.setAttribute("class", "minmaxRate anchorAtMiddle");		
			parent.appendChild(gElement);				
			addLabel(gElement, actualRate, null, cx, (cy + boxYOffset + boxHeight - 3), 0);					
		}
		
		// rate definition label
		String unitLabel = (String)titles.get(RATE_DEFINITION);
		// group for font changes		
		if (unitLabel != null && !unitLabel.equals(EMPTY_STRING)) {			
			gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id", "rateDefinition");
			gElement.setAttribute("class", "rateDef anchorAtMiddle");
			parent.appendChild(gElement);				
			addLabel(gElement, unitLabel, null, cx, (cy + 14), 0);				
		}

		//Draw the meter sections - divisions should have exactly 3 elements	
		int markers = divisions.length;
		
		if (markers > 3) {
			//throwException(MAXIMUM_NUMBER_OF_SEGMENT_MARKERS_EXCEEDED, RANGE);
			short code = MAXIMUM_NUMBER_OF_SEGMENT_MARKERS_EXCEEDED;				
			String element = RANGE;				
			DataInputProcessingException e = new DataInputProcessingException(code, element);
			throw e;	
		}	
		
		// hardcoded minRate 
		int min = 0;
				
		// need to sort these markers
		double [] sorter = new double[divisions.length];
		for (int i = 0; i < divisions.length; i++){
			if (divisions[i] != null) {
				// NumberFormatException not possible here
				sorter[i] = Double.parseDouble(divisions[i]);
			}	
		}		
		if (sorter.length > 1 ){
			Arrays.sort(sorter);
		}
		
		// get the markers
		double criticalMarker = 0;
		double safeMarker = 0;
		double max = 0;		
		if (sorter.length == 3) {
			criticalMarker = sorter[0];
			safeMarker = sorter[1];
			max = sorter[2];		
		} else if (sorter.length == 2) {
			criticalMarker = sorter[0];			
			max = sorter[1];	
		} else {
			max = sorter[0];
		}	
		double totalUnits = max;

		// calculate the number of units between the different ranges				
		double unitsCriticalRange = criticalMarker - min;		
		double unitsSemiCriticalRange = safeMarker - criticalMarker;			
		double unitsSafeRange = max - safeMarker;		
		double radians = 0.01745329252;
		
		double safeDegrees = 0;
		double semiDegrees = 0;
		double critDegrees = 0;
		if (unitsSafeRange > 0) {
			safeDegrees = unitsSafeRange / totalUnits * 180 * radians;
		}
		if (unitsSemiCriticalRange > 0) {
			semiDegrees = unitsSemiCriticalRange / totalUnits * 180 * radians;
		}
		if (unitsCriticalRange > 0) {
			critDegrees = unitsCriticalRange / totalUnits * 180 * radians;
		}

		// safe range slice		
		double radius = baseLength * 0.25;	
		String path = EMPTY_STRING;
		String translateString = "translate(" + cx + "," + cy + ")";
		if (safeDegrees > 0) {			
			x = Math.cos(safeDegrees-0.000001) * radius;
			y = Math.sin(safeDegrees-0.000001) * radius;		
			path =
				"M "
				+ radius
				+ " 0 A "
				+ radius
				+ " "
				+ radius
				+ " 0 0 0 "
				+ x
				+ " -"
				+ y
				+ " L 0 0 Z";
			newElement = generatedDocument.createElement("path");
			newElement.setAttribute("class", "st1 safeRange");
			newElement.setAttribute("transform", translateString);		
			newElement.setAttribute("d", path);
			parent.appendChild(newElement);
		}
				
		// semi-critical range slice
		if (semiDegrees > 0) {
			x = Math.cos(semiDegrees) * radius;
			y = Math.sin(semiDegrees) * radius;
			path =
				"M "
				+ radius
				+ " 0 A "
				+ radius
				+ " "
				+ radius
				+ " 0 0 0 "
				+ x
				+ " -"
				+ y
				+ " L 0 0 Z";
			newElement = generatedDocument.createElement("path");
			newElement.setAttribute("class", "st1 semiCritical");
			newElement.setAttribute("transform", translateString + " rotate(-" + safeDegrees / radians + ")");	
			newElement.setAttribute("d", path);
			parent.appendChild(newElement);
		}
		// critical range slice
		if (critDegrees > 0) {
			x = Math.cos(critDegrees) * radius;
			y = Math.sin(critDegrees) * radius;
			path =
				"M "
				+ radius
				+ " 0 A "
				+ radius
				+ " "
				+ radius
				+ " 0 0 0 "
				+ x
				+ " -"
				+ y
				+ " L 0 0 Z";		
			newElement = generatedDocument.createElement("path");
			newElement.setAttribute("class", "st1 critical");
			newElement.setAttribute("transform", translateString + " rotate(-" + (180 - (critDegrees / radians)) + ")");		
			newElement.setAttribute("d", path);
			parent.appendChild(newElement);
		}
		
		Element meterLabelElement = generatedDocument.createElement("g");
		meterLabelElement.setAttribute("id", "meterLabel");
		parent.appendChild(meterLabelElement);
		
		// min rate label 
		x = cx - radius - 2;
		y = cy + 2;	
		addLabel(meterLabelElement, Double.toString(min), "minmaxRate anchorAtEnd", x, y, 0); 

		// max rate label
		x = cx + radius + 1;
		y = cy + 2;
		addLabel(meterLabelElement, Double.toString(max), "minmaxRate anchorAtStart", x, y, 0); 
		
		// range of the meter
		double totalRange = max - min;

		// determine critical marker position and then draw it
		if (criticalMarker > 0) {
			ArrayList criticalMarkerInfo = calculateMarkerPosition(criticalMarker, totalRange, radius);
			String criticalMarkerX = (String)criticalMarkerInfo.get(0);
			String criticalMarkerY = (String)criticalMarkerInfo.get(1);
			String criticalMarkerStyle = (String)criticalMarkerInfo.get(2);
			addTranslationLabel(meterLabelElement, criticalMarkerX, criticalMarkerY, criticalMarker, translateString, criticalMarkerStyle);
		}
		// determine safe marker position and then draw it
		if (safeMarker > 0) {
			ArrayList safeMarkerInfo = calculateMarkerPosition(safeMarker, totalRange, radius);
			String safeMarkerX = (String)safeMarkerInfo.get(0);
			String safeMarkerY = (String)safeMarkerInfo.get(1);
			String safeMarkerStyle = (String)safeMarkerInfo.get(2);		
			addTranslationLabel(meterLabelElement, safeMarkerX, safeMarkerY, safeMarker, translateString, safeMarkerStyle);
		}
			
		// draw 19 ticks within the outline of the meter
		double angle = 9;
		String rotateString = EMPTY_STRING;
		for (int i = 0; i < 19; i++) {
			double rad = angle * Math.PI / 180.0;
			double xpos = (Math.cos(-1 * rad)) * (radius - 5);
			double ypos = (Math.sin(-1 * rad)) * (radius - 5);
			rotateString = "rotate(-" + angle + "," + xpos + "," + ypos + ")";
			// xlink the tick line 
			newElement = generatedDocument.createElement("use");
			newElement.setAttribute("xlink:href", "#tickline");
			newElement.setAttribute("x", Double.toString(xpos));
			newElement.setAttribute("y", Double.toString(ypos));  
			newElement.setAttribute("transform", translateString + rotateString);	  
			parent.appendChild(newElement);
			
			angle += 9;
		}	
		
		// needle
		// NumberFormatException not possible here
		double needleAngle = Double.parseDouble(needle) / max * 180 + 180;
		newElement = generatedDocument.createElement("rect");		
		newElement.setAttribute("class", "st1 needle");
		newElement.setAttribute("width", Double.toString(radius - 6));
		newElement.setAttribute("height", Double.toString(1.3));
		newElement.setAttribute("x", Double.toString(cx));
		newElement.setAttribute("y", Double.toString(cy));		
		newElement.setAttribute("transform", "rotate(" + needleAngle + "," + Double.toString(cx) +  "," + Double.toString(cy) + ")");				
		parent.appendChild(newElement);
		// needle outer circle
		newElement = generatedDocument.createElement("circle");
		newElement.setAttribute("class", "st1 outerCircle");
		newElement.setAttribute("cx", Double.toString(cx));
		newElement.setAttribute("cy", Double.toString(cy));		
		newElement.setAttribute("r", Double.toString(2.5));
		parent.appendChild(newElement);
		// needle inner circle
		newElement = generatedDocument.createElement("circle");
		newElement.setAttribute("class", "st1 innerCircle");
		newElement.setAttribute("cx", Double.toString(cx));
		newElement.setAttribute("cy", Double.toString(cy));		
		newElement.setAttribute("r", Double.toString(1.7));
		parent.appendChild(newElement);
        
        // needle center
        newElement = generatedDocument.createElement("circle");
		newElement.setAttribute("class", "st1 center");
		newElement.setAttribute("cx", Double.toString(cx));
		newElement.setAttribute("cy", Double.toString(cy));		
		newElement.setAttribute("r", Double.toString(0.5));
		parent.appendChild(newElement);
	}

	// add marker labels	
	void addTranslationLabel(Element parent, String x, String y, double label, String translateString, String styleSheetClass) {
		Element newElement;
		newElement = generatedDocument.createElement("text");
		newElement.setAttribute("class", "critDiv " + styleSheetClass);
		newElement.setAttribute("x", x);
		newElement.setAttribute("y", y);
		newElement.setAttribute("transform", translateString);	
		newElement.appendChild(generatedDocument.createTextNode(Double.toString(label)));
		parent.appendChild(newElement);
	}
	
	// Get marker information
	// @return an ArrayList with the following elements
	//  - the first element is the marker degree value
	//  - the second element is the marker x position
	//  - the third element is the marker y position
	//  - the fourth element is the marker anchor style
	ArrayList calculateMarkerPosition(double marker, double totalRange, double radius) {
		double markerDeg = (totalRange - marker) / totalRange * 180.0;
		double markerRad = markerDeg * Math.PI / 180.0;
		double markerX = (Math.cos(-1 * markerRad)) * (radius + 3);
		double markerY = (Math.sin(-1 * markerRad)) * (radius + 3);		
		
		String markerStyle = EMPTY_STRING;
		if (markerDeg < 90) 
			markerStyle = "anchorAtStart";
		else if (markerDeg == 90)
			markerStyle = "anchorAtMiddle";
		else if (markerDeg > 90) 
			markerStyle = "anchorAtEnd";

		ArrayList markerPosInfo = new ArrayList();
		markerPosInfo.add(Double.toString(markerX));
		markerPosInfo.add(Double.toString(markerY));
		markerPosInfo.add(new String(markerStyle));
		
		return markerPosInfo;	
	}
	

}

