package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
/**
 * <code>SVGPieChart</code> generates a pie chart graphic DOM using Scalable Vector Graphics (SVG). 
 * 
 * @version %I%
 */
public class SVGPieChart extends SVGDocumentGenerator implements IGraphicDocumentStyle, IDataInputProcessingExceptionCodes, IDataInputConstants, Serializable {
	private PieChartDataRetriever dataRetriever;
		
	// horizontally align the total value label and legend title
	private int yOffset;
	
	// pie centre and radius
	private double radius, cx, cy;	
		
	// String used to translate everything to pie centre coordinates
	private String translateString;		
	
	// constants for determining tick direction
	private final int NINETY_DEG = 1;
	private final int NEGATIVE_NINETY_DEG = 2;    
	private final int ONE_EIGHTY_DEG = 3;		  
	private final int ZERO_DEG = 4;
	
	/**
	 *  Sole constructor.
	 */
	public SVGPieChart() {
		super();
		graphicType = IGraphicTypeConstants.PIE_CHART;
		setGraphicWidth(PIE_DEFAULT_WIDTH);
		setPreferencesPage(PIE_PREFS);
		setPreferencesPageWidth(PREFS_WIDTH);
		setPreferencesPageHeight(PIE_PREFS_HEIGHT);
		dataRetriever = new PieChartDataRetriever();
	}
	
	/**
	 * Sets the document which is to be used for configuration data 
	 * retrieval and then retrieves the configuration data from that
	 * document. 
	 * 
	 * @see IGraphicDocumentGenerator#setConfigDocument(Document)
	 */
	public void setConfigDocument(Document newConfigDocument) throws DataInputProcessingException {
		super.setConfigDocument(newConfigDocument);
		dataRetriever.setConfigDocument(getConfigDocument());
		if (getConfigDocument() == null) {
			return;
		}	
		Hashtable configProperties = dataRetriever.getConfigurationAttributes();	
		if (configProperties == null) {
			return;
		}		
		String width = (String)configProperties.get(WIDTH);
		String height = (String)configProperties.get(HEIGHT);
		String fontFamily = (String)configProperties.get(FONT);
		String title = (String)configProperties.get(TITLE);
		String timestampPrefix = (String)configProperties.get(TIMESTAMP_PREFIX);
		String suppressLegend = (String)configProperties.get(SUPPRESS_LEGEND);
		String legendTitle = (String)configProperties.get(LEGEND_TITLE);
					
		if (width != null) {
			setGraphicWidth(width);
		}	
		
		if (height != null) {
			setGraphicHeight(height);
		}	
		
		if (fontFamily != null) {
			setGraphicFont(fontFamily);			
		}			
		
		if (title != null) {
			setGraphicTitle(title);			
		}
			
		if (timestampPrefix != null) {
			setGraphicTimestampPrefix(timestampPrefix);			
		}	
		
		if (suppressLegend != null) {
			setSuppressLegend(suppressLegend);			
		}	
		
		if (legendTitle != null) {
			setGraphicLegendTitle(legendTitle);			
		}	
	}
	
	/**
	 * Sets the document which is to be used for data update 
	 * retrieval and then, if <code>configDocument</code> has not been
	 * set, retrieves the configuration data from that document.
	 * 
	 * @see IGraphicDocumentGenerator#setDataDocument(Document)
	 */
	public void setDataDocument(Document newDataDocument) throws DataInputProcessingException {
		super.setDataDocument(newDataDocument);
		dataRetriever.setDataDocument(getDataDocument());
		
		// if we already have them, no need to get them
		if (getConfigDocument() != null) {
			return;
		}	
		
		Hashtable configProperties = dataRetriever.getConfigurationAttributes();	
		if (configProperties == null) {
			return;
		}	
		String width = (String)configProperties.get(WIDTH);
		String height = (String)configProperties.get(HEIGHT);
		String fontFamily = (String)configProperties.get(FONT);
		String title = (String)configProperties.get(TITLE);
		String timestampPrefix = (String)configProperties.get(TIMESTAMP_PREFIX);
		String suppressLegend = (String)configProperties.get(SUPPRESS_LEGEND);
		String legendTitle = (String)configProperties.get(LEGEND_TITLE);
					
		if (width != null) {
			setGraphicWidth(width);
		}	
		
		if (height != null) {
			setGraphicHeight(height);
		}	
		
		if (fontFamily != null) {
			setGraphicFont(fontFamily);			
		}			
		
		if (title != null) {
			setGraphicTitle(title);			
		}
			
		if (timestampPrefix != null) {
			setGraphicTimestampPrefix(timestampPrefix);			
		}	
		
		if (suppressLegend != null) {
			setSuppressLegend(suppressLegend);			
		}	
		
		if (legendTitle != null) {
			setGraphicLegendTitle(legendTitle);			
		}	
	}
	
	/**
	 * In addition to those thrown by <code>PieChartDataRetriever</code>, this
	 * method throws <code>DataInputProcessingException</code> for codes <code>MISSING_TOTALS,
	 * MISSING_PERCENTAGES</code>.
	 * 
	 * @see IGraphicDocumentGenerator#generateGraphicDocument()
	 */	
	public Document generateGraphicDocument() throws DataInputProcessingException {		
		// NumberFormatException not possible here				
		double gWidth = Double.parseDouble(getGraphicWidth());
		double gHeight = Double.parseDouble(getGraphicHeight());
		radius = gHeight >= gWidth ? gWidth * 0.173 : gHeight * 0.233; 
		
		yOffset = (int)(gHeight * 0.152);	
		cx = gWidth * 0.35;
		cy = gHeight * .52;		
		translateString = "translate(" + cx + "," + cy + ")";		
		
		int numberOfDatasets = 0;
		String [] datasetTotals = null;
		String [] datasetPercentages = null;
			
		// if there's no data, nothing to do		
		try {		
			numberOfDatasets = dataRetriever.getNumberOfDatasets();				
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + ": " + e.getMessage());
			throw e;			
		}		
		
		// !DO NOT REORDER THESE METHOD CALLS UNLESS NECESSARY!!		
		// create the document		
		createSvgDocument(dataDocument.getImplementation());
		
		// add the "onload" event handler function
		svgRoot.setAttribute("onload", "init()");
		
		addJavaScriptFunctions(svgRoot, numberOfDatasets);
 		addDefinitions(svgRoot, numberOfDatasets);
	   	addBorder(svgRoot);  
	   	   
		try {
			datasetTotals = dataRetriever.getDatasetTotals();		
			if (datasetTotals.length != numberOfDatasets) {
				// missing totals
				short code = MISSING_TOTALS;				
				String element = DATASET;				
				DataInputProcessingException e = new DataInputProcessingException(code, element);
				throw e;				
			}	
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + ": " + e.getMessage());
			throw e;		 	
		}		 
		try {		
			datasetPercentages = dataRetriever.getDatasetPercentages();	
			if (datasetPercentages.length != numberOfDatasets) {				
				// missing percentages
				short code = MISSING_PERCENTAGES;				
				String element = DATASET;				
				DataInputProcessingException e = new DataInputProcessingException(code, element);
				throw e;
				
			}				
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + ": " + e.getMessage());
			throw e;		 	
		}
		 
		addPieSlices(svgRoot, datasetTotals, datasetPercentages);  	 
	   	
	   	try {
			Hashtable titles = dataRetriever.getTitles();		
 		    addTitles(svgRoot, titles);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + ": " + e.getMessage());			
			throw e;
		}
		
		try {
			String timestamp = dataRetriever.getTimestamp();	
        	addTimeStamp(svgRoot, timestamp);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + ": " + e.getMessage());
			throw e;
		}  	 	      
     
	     // omit the legend?   
        if (!isLegendSuppressed()) {      		
	  		try {
	       		String [] legendLabels = dataRetriever.getLegendLabels();		       
				addLegend(svgRoot, legendLabels, (int)(gWidth*0.75), yOffset); 
	  		} catch (DataInputProcessingException e) {
	  			e.setWidgetType(graphicType);
	  			//System.out.println(e.getWidgetType() + " " + e.getElement() + ": " + e.getMessage());	  			
	  			throw e;
	  		} 		
    	}      	    	
        return super.generateGraphicDocument();		
	}	
	
	protected void addJavaScriptFunctions(Element parent, int numberOfSets) {		
		super.addJavaScriptFunctions(parent, numberOfSets);
		
		Element newElement = generatedDocument.createElement("script");
		newElement.setAttribute("type", JAVASCRIPT_VERSION);
		parent.appendChild(newElement);
	
		ECMAScriptGenerator jscript = new ECMAScriptGenerator();
		StringBuffer functions = new StringBuffer(jscript.generatePieChartHelperFunctions());							
		CDATASection newCDATASection = generatedDocument.createCDATASection(functions.toString());

		newElement.appendChild(newCDATASection);
	}	
	
	/**
	 * Adds the titles to the SVG graphic. 
	 * 
	 * @param parent element to which the title-related elements are appended
	 * @param input the titles
	 */
	protected void addTitles(Element parent, Hashtable input) {				
		int rotation = 0;
		int xOffset = 10;
		Element gElement = generatedDocument.createElement("g");
		String title = null;
		
		// add the main title
		addTitle(parent);
		
		// add a font group for the labels and then add the labels
		// pie slice value title
		title = (String)input.get(VALUE_TITLE);
		if (title != null) {			
			gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id", "labelsValues0");
			gElement.setAttribute("class", "labelsValues, anchorAtMiddle");
			gElement.setAttribute("font-family", getGraphicFont());
			parent.appendChild(gElement);				
			// need the centre point of the circle
			// NumberFormatException not possible here
			addLabel(gElement, title, null, (Double.parseDouble(getGraphicWidth())*0.35), (Double.parseDouble(getGraphicHeight()) * 0.9), rotation);
		}
		// total value title
		title = (String)input.get(TOTAL_TITLE);
		if (title != null) {			
			gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id", "labelsValues1");
			gElement.setAttribute("class", "legendTitle, anchorAtStart");
			gElement.setAttribute("font-family", getGraphicFont());
			parent.appendChild(gElement);	
			// magic numbers for positioning			
			addLabel(gElement, title, null, xOffset, yOffset + 10, rotation);
		}
	}	
	
	// add style element with default style sheet rules for the pie chart
	void addDefaultInternalStylesheet(Element parent, int sets) {
		StringBuffer styleRules = new StringBuffer(COMMON_STYLE);
		styleRules.append(" ");
		styleRules.append(PIE_STYLE);
		
		String [] palette = (String [])graphicColourPalette.get(graphicColourPaletteName);
		// palette has limited number of colours, so loop if more sets
		int paletteLength = palette.length;		  
		for (int i = 0, j = 0; i < sets; i++) {
		  j = (i + 1) % paletteLength;
		  j = (j == 0) ? paletteLength - 1 : j-1;			
		  styleRules.append(" .dataset" + i + "{stroke:black; fill:" + palette[j] + ";}");	
		  if (!isLegendSuppressed()) {	  
		  	styleRules.append(" .shape" + i + "{stroke:none; fill:" + palette[j] + ";}");
		  }
		}
		
		CDATASection newCDATASection = generatedDocument.createCDATASection(styleRules.toString());
		
		Element newElement = generatedDocument.createElement("style");
		newElement.setAttribute("id", "defaultStyleRules");
		newElement.setAttribute("type", "text/css");
		newElement.appendChild(newCDATASection);
		parent.appendChild(newElement);
	}	
	
	// add defs element with pie chart definitions 
	void addDefinitions(Element parent, int numberOfDatasets) {
		Element defsElement = generatedDocument.createElement("defs");
        parent.appendChild(defsElement);
       
		addDefaultInternalStylesheet(defsElement, numberOfDatasets);		
				
		if (!isLegendSuppressed()) {
			// id: legendsquareshapeWithoutLine
			Element gElement_legendsquareWithoutLine = generatedDocument.createElement("g");	
			defsElement.appendChild(gElement_legendsquareWithoutLine);
			gElement_legendsquareWithoutLine.setAttribute("id","legendsquareshapeWithoutLine");
		
		
	    	Element legendsquareElementWithoutLine = generatedDocument.createElement("rect");
	    	gElement_legendsquareWithoutLine.appendChild(legendsquareElementWithoutLine);
			legendsquareElementWithoutLine.setAttribute("width", "8");
			legendsquareElementWithoutLine.setAttribute("height", "8");	
			legendsquareElementWithoutLine.setAttribute("x", "3");
			legendsquareElementWithoutLine.setAttribute("y", "0");
			legendsquareElementWithoutLine.setAttribute("transform", "translate(0,2)");					
		}	
		
		// id: tickline		
		Element gElement_tick = generatedDocument.createElement("g");
		gElement_tick.setAttribute("id", "tickline");
			
		Element tickElement = generatedDocument.createElement("polyline");
		//tickElement.setAttribute("class", "st1, wedgeTick");
		tickElement.setAttribute("class", "grid");
		tickElement.setAttribute("points", "0 0 8 0");
		gElement_tick.appendChild(tickElement);
		defsElement.appendChild(gElement_tick);
		
		addPreferenceIcon(defsElement);
	}	
	
	// add the slices to the pie	
	void addPieSlices(Element parent, String [] slices, String [] percentages) {				
		double pieValue = 0;
		double sliceValue = 0;
		double percent = 0;	
		double originalDegree = 0;
		double previousDegree = 0;
		double degree = 0;
		double xPosition = 0;
		double yPosition = 0;		
		Element newElement, gColourElement, gToggleValueElement, gToggleBothValuesElement;
		Element gTogglePercentElement;
		
		// groups for toggling values and percentages visibility		
		// slice value
		gToggleValueElement = generatedDocument.createElement("g");
		gToggleValueElement.setAttribute("id","exactValues");			
		gToggleValueElement.setAttribute("onclick","toggleVisibility(\"exactValues\")");
		gToggleValueElement.setAttribute("visibility", "hidden");
		// add it to the colour group for color changes
		parent.appendChild(gToggleValueElement);	
		
		// percentage value
		gTogglePercentElement = generatedDocument.createElement("g");
		gTogglePercentElement.setAttribute("id", "percentValues");
		gTogglePercentElement.setAttribute("onclick","toggleVisibility(\"percentValues\")");
		gTogglePercentElement.setAttribute("visibility", "hidden");
		parent.appendChild(gTogglePercentElement);
		
		// both values
		gToggleBothValuesElement = generatedDocument.createElement("g");
		gToggleBothValuesElement.setAttribute("id", "bothValues");
		gToggleBothValuesElement.setAttribute("onclick","toggleVisibility(\"bothValues\")");
		parent.appendChild(gToggleBothValuesElement);

									
		// caluclate the total value of the pie
		for (int i = 0; i < slices.length; i++) {
			// NumberFormatException not possible here
			pieValue += Double.parseDouble(slices[i]);
		}	
				
		// for each slice, calculate the slice arc and position from the sliceValue percentage
		for (int i = 0; i < slices.length; i++) {	
			// NumberFormatException not possible here								
			sliceValue = Double.parseDouble(slices[i]);								
			percent = (pieValue > 0) ? sliceValue / pieValue : 0;				
			// calculate the degrees of the percentage
			originalDegree = percent * 360;
			//double degtemp = percent * 360;
			degree = (180 - (originalDegree)) * 0.01745329252;
			if (degree < 0) {
				degree = degree * -1;
			}
							
			// find the x and y position based on the radius and the degrees
			xPosition = (Math.cos(degree)) * radius;
			yPosition = (Math.sin(degree)) * radius;
			
			// calculate the arc path			
			String path = (originalDegree > 90) ? 
				("M "
					+ radius
					+ " 0 A "
					+ radius
					+ " "
					+ radius
					+ " 0 0 0 -"
					+ xPosition
					+ " -"
					+ yPosition
					+ " L 0 0  Z") 
					
				: ("M "
					+ radius
					+ " 0 A "
					+ radius
					+ " "
					+ radius
					+ " 0 0 0 "
					+ (xPosition * -1)
					+ " -"
					+ yPosition
					+ " L 0 0  Z");
					
			// set the slice transformation					
			String transformation =(i ==0) ? translateString : translateString + " rotate(-" + previousDegree + ")";
				
			// group element for data colour changes and toggling exact values visibility
			gColourElement = generatedDocument.createElement("g");
			gColourElement.setAttribute("id", "pieColour" + i);
			gColourElement.setAttribute("class", "dataset" + i);				
			gColourElement.setAttribute("onclick", "toggleVisibility(\"pieColour\");");
			parent.appendChild(gColourElement);										
				
			newElement = generatedDocument.createElement("path");						
			newElement.setAttribute("transform", transformation);						
			newElement.setAttribute("d", path);											
			gColourElement.appendChild(newElement);		
			
			// add the value and percentage labels for the slice
			addLabels(gToggleBothValuesElement, gToggleValueElement, gTogglePercentElement, i, percentages, percent, previousDegree, slices[i]);
			// set up for loop
			previousDegree += originalDegree;
		}		
	}
	
	// add value and percentage to each pie slice
	void addLabels(Element bothValuesElement, Element valueElement, Element percentElement, int index, String [] percentageLabels, double percentage, double previous, String sliceValue) {		
		Element newElement;
		double degree = percentage * 360;					
		double rad = (degree / 2 + previous) * 0.01745329252;
		double xpos = (Math.cos(-1 * rad)) * radius;
		double ypos = (Math.sin(-1 * rad)) * radius;		
		
		String rotateString = EMPTY_STRING;
		int rotateFlag = 0;
				
		// adjust the tick rotation value
		if (cy + ypos > cy && cy + ypos >= cy + 7.0/8.0 * radius) {
			// ypos is near the bottom of the pie chart
			rotateString = "rotate(" + (90) + "," + xpos + "," + ypos + ")";
			rotateFlag = NINETY_DEG;	
		} else if (cy + ypos < cy && cy + ypos <= cy - 7.0/8.0 * radius) {  
			 // ypos is near the top of the pie chart
			rotateString = "rotate(" + (-90) + "," + xpos + "," + ypos + ")";
			rotateFlag = NEGATIVE_NINETY_DEG;
		} else if (cx + xpos < cx) {  
			// if the tick is on the left side of the pie chart
			rotateString = "rotate(" + (180) + "," + xpos + "," + ypos + ")";	
			rotateFlag = ONE_EIGHTY_DEG;
		} else {
			rotateFlag = ZERO_DEG;	
		}
				
		// xlink the tick line 
		newElement = generatedDocument.createElement("use");
		newElement.setAttribute("xlink:href", "#tickline");
		newElement.setAttribute("x", Double.toString(xpos));
		newElement.setAttribute("y", Double.toString(ypos));  
		newElement.setAttribute("transform", translateString + rotateString);	  
		valueElement.appendChild(newElement);
		percentElement.appendChild(newElement);   // newly added
		bothValuesElement.appendChild(newElement);
		
		// create the group for toggling visibility
		String stylesheetClass;
		
		// all text labels will be anchored at start if they are located on
		// right side of the circle; otherwise, the labels will be anchored 
		// at end.
		if (cx + xpos >= cx) {
			stylesheetClass = "pieValues anchorAtStart";							
		} else {
			stylesheetClass = "pieValues anchorAtEnd";
		}		
		
		// percentage and value - need to separate these			
		String label = percentageLabels[index] + "%,  " + sliceValue;
		
		// value and percentage labels
		String percentLabel = percentageLabels[index] + "%";
		String sliceValueLabel = sliceValue;
		
		// determine label x and y positions
		Vector labelPosVec = calculateLabelPositions(cx, xpos, ypos, rotateFlag, label);
		// NumberFormatException not possible here
		double labelXPos = Double.parseDouble(labelPosVec.firstElement().toString());
		double labelYPos = Double.parseDouble(labelPosVec.lastElement().toString());
		
		// draw percentage label
		newElement = generatedDocument.createElement("text");	
		newElement.setAttribute("x", Double.toString(labelXPos));
		newElement.setAttribute("y", Double.toString(-1 * labelYPos));
		newElement.setAttribute("transform", translateString);		
		newElement.setAttribute("class", stylesheetClass + " shape" + index);
		newElement.setAttribute("font-family", getGraphicFont());
		newElement.appendChild(generatedDocument.createTextNode(percentLabel));
		percentElement.appendChild(newElement);

		// draw value label
		newElement = generatedDocument.createElement("text");	
		newElement.setAttribute("x", Double.toString(labelXPos));	
		newElement.setAttribute("y", Double.toString(-1 * labelYPos));
		newElement.setAttribute("transform", translateString);		
		newElement.setAttribute("class", stylesheetClass + " shape" + index);
		newElement.setAttribute("font-family", getGraphicFont());
		newElement.appendChild(generatedDocument.createTextNode(sliceValueLabel));
		valueElement.appendChild(newElement);							
		
		// draw both values
		newElement = generatedDocument.createElement("text");	
		newElement.setAttribute("x", Double.toString(labelXPos));	
		newElement.setAttribute("y", Double.toString(-1 * labelYPos));
		newElement.setAttribute("transform", translateString);		
		newElement.setAttribute("class", stylesheetClass + " shape" + index);
		newElement.setAttribute("font-family", getGraphicFont());
		newElement.appendChild(generatedDocument.createTextNode(label));
		bothValuesElement.appendChild(newElement);							
	}	

	// determine value and percentage label position	
	Vector calculateLabelPositions(double cx, double xpos, double ypos, int rotateFlag, String label) {
		double labelXPos = 0;
		double labelYPos = 0;		
		
		// store the label x and y positions into a vector
		Vector labelVec = new Vector(2);  
			
		if (cx + xpos < cx) {   // if the label is located on the left side of the circle				
			if (rotateFlag == NEGATIVE_NINETY_DEG) {
				labelXPos = xpos + 2; 
				labelYPos = -ypos + 12;
			} else if (rotateFlag == NINETY_DEG) {
				labelXPos = xpos + 2;
				labelYPos = -ypos - 20;	
			} else if (rotateFlag == ONE_EIGHTY_DEG) {	
				labelXPos = xpos - 10;
				labelYPos = -ypos - 1;
			}
		} else {   // if the label is located on the right side of the circle
			if (rotateFlag == NEGATIVE_NINETY_DEG) {
				labelXPos = xpos - 3;
				labelYPos = -ypos + 12;	
			} else if (rotateFlag == NINETY_DEG) {
				labelXPos = xpos - 3;
				labelYPos = -ypos - 20;	
			} else if (rotateFlag == ZERO_DEG) {
				labelXPos = xpos + 10;
				labelYPos = -ypos - 1 ;
			}
		}
		
		labelVec.add(new Double(labelXPos));
		labelVec.add(new Double(labelYPos));
		
		return labelVec;
	}

}

