package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
/**
 * <code>SVGStackBarChart</code> generates a stacked bar chart graphic DOM using Scalable Vector Graphics (SVG).
 * 
 * @version %I%
 */
public class SVGStackBarChart extends SVGXYChart implements IGraphicDocumentStyle, IDataInputProcessingExceptionCodes, Serializable {
	/**
	 * Sole constructor.
	 */
	public SVGStackBarChart() {
		super();
		graphicType = IGraphicTypeConstants.STACKBAR_CHART;
		setGraphicWidth(STACKBAR_DEFAULT_WIDTH);
		setPreferencesPage(STACKBAR_PREFS);
		setPreferencesPageWidth(PREFS_WIDTH);
		setPreferencesPageHeight(STACKBAR_PREFS_HEIGHT);		
	}
	
	/**
	 * @see IGraphicDocumentGenerator#generateGraphicDocument()
	 */	
	public Document generateGraphicDocument() throws DataInputProcessingException {								
		String [] rangeMarkers = null;
		String [] categories = null;	
		String rangeMaxValue = null;
		int xTicks = 0, yTicks = 0, numberOfDatasets = 0;
		
		// if no data, exit
		try {
			numberOfDatasets = dataRetriever.getNumberOfDatasets();
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;		
		}		
		
		// !DO NOT REORDER THESE METHOD CALLS UNLESS NECESSARY!!
		createSvgDocument(dataDocument.getImplementation());
		
		// add the "onload" event handler function
		svgRoot.setAttribute("onload", "init()");
		
		addJavaScriptFunctions(svgRoot, numberOfDatasets);
 		addDefinitions(svgRoot, numberOfDatasets);
	    addBorder(svgRoot);
  
		try {
			rangeMarkers = dataRetriever.getRangeMarkers();
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;				
		}					
		
		if (rangeMarkers != null && rangeMarkers.length > 0) { 
			rangeMaxValue = (String)rangeMarkers[rangeMarkers.length - 1];
		}		
		yTicks = rangeMarkers.length;
		
		try {
			categories = dataRetriever.getCategories();
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;				
		}				
		xTicks = categories.length;
     
        addGrid(svgRoot, xTicks, yTicks, rangeMarkers);
        addAxisLabels(svgRoot, categories, rangeMarkers);
        addDataSets(svgRoot, dataRetriever, xTicks, numberOfDatasets, rangeMaxValue, graphicType); 	
         		
		try {
			Hashtable titles = dataRetriever.getTitles();		
 		    addTitles(svgRoot, titles);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());	
			throw e;		
		}
		
		try {
			String timestamp = dataRetriever.getTimestamp();	
        	addTimeStamp(svgRoot, timestamp);
		} catch (DataInputProcessingException e) {
			e.setWidgetType(graphicType);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;
		}
		        
        // omit the legend?
        if (!isLegendSuppressed()) {      		
	  		try {
	       		String [] legendLabels = dataRetriever.getLegendLabels();		       
				addLegend(svgRoot, legendLabels, GRIDXOFFSET + xAxisLength, GRIDYOFFSET); 
	  		} catch (DataInputProcessingException e) {
	  			e.setWidgetType(graphicType);
	  			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());	  			
	  			throw e;
	  		} 		
    	}      
		return super.generateGraphicDocument();
	}
	
	// Go through each x position and then draw the total value of each bar
	private void addAllBarValues(Element parent, double yUnits, int xTicks, String [] barTotals)	{
		int length = barTotals.length;		
		int xZero = GRIDXOFFSET;
		int yZero = GRIDYOFFSET + yAxisLength;
		int id = 0;
		double xIntervalLength = (double)xAxisLength/xTicks;
		
		for (int i = 0; i < length; i++) {			
			String barTotal = barTotals[i];			
			double barWidth = xIntervalLength * 0.5;    // width of the bar
			// NumberFormatException not possible here
			double barHeight = yUnits * Double.parseDouble(barTotal);		 // height of the bar
			double barX = xZero + xIntervalLength * (((double)i) + 0.5) - barWidth * 0.5;  // bar x position
			double barY = yZero - barHeight; 			
			
			//double textX = barX + barWidth / 2.0 - barTotal.length() * 2.0;  // text x position 
			double textX = barX + barWidth / 2.0;
			double textY = barY - 5.0;   // text y position
			addExactValue(parent, textX, textY, barTotal);
		}	
	}
	
	/*
	 * 	Draw bars for each barset
	 *  @param parent barchart 
	 *  @param barsetNumber number for each barset, e.g., 0 for pass, 1 for fail, 2 for soft fail, etc.
	 *  @param yUnits y interval length
	 *  @param Hashtable -> key: xposition  value: xposition Value
	 */
	private Vector addBarset(Element parent, int barsetNumber, double yUnits, String [] input, int xTicks, Vector prevVec, Vector boxYPosVec, Boolean firstBarSet, int numberOfBarSets) {
		int length = input.length;  // number of bars for each barset
		
		// At coordinate 0
		int xZero = GRIDXOFFSET;
		int yZero = GRIDYOFFSET + yAxisLength - 1;  // move up the whole bar in each x position by 1 unit
		double xIntervalLength = (double)xAxisLength/xTicks;

		Vector barStartYValueVec = new Vector();
				
		if (prevVec.size() == 0) {
			for (int i = 0; i < length; i++) {
				barStartYValueVec.add(new Double(yZero));	
			}	
		} else {
			barStartYValueVec = prevVec;
		}

		Vector newVec = new Vector();		
		Vector tmpBoxYPos = new Vector();
		
		Element oneBarSetElement = generatedDocument.createElement("g");
		oneBarSetElement.setAttribute("id", "dataColour" + barsetNumber);
		oneBarSetElement.setAttribute("class", "shape" + barsetNumber);
		oneBarSetElement.setAttribute("onclick","toggleVisibility(\"exactValues\")");
		parent.appendChild(oneBarSetElement);		
		
		// start drawing one barSet
		for (int i = 0; i < length; i++) {
			String exactValue = input[i];  // exact height of a bar on the y axis
			double barWidth = xIntervalLength * 0.5;
			// NumberFormatException not possible here
			double barHeight = yUnits * Double.parseDouble(exactValue) - 0.9 / ((double)numberOfBarSets);
			double barX = xZero + xIntervalLength * (i  + 0.5) - barWidth * 0.5;
			double barY = Double.parseDouble(barStartYValueVec.get(i).toString()) - barHeight;
			
			Element newElement = generatedDocument.createElement("rect");//			
			newElement.setAttribute("x", Double.toString(barX));
			newElement.setAttribute("y", Double.toString(barY));
			newElement.setAttribute("width", Double.toString(barWidth));
			newElement.setAttribute("height", Double.toString(barHeight));

			oneBarSetElement.appendChild(newElement);
			
			newVec.add(new Double(barY));
			
			// Store textbox y positions in a vector
			double boxHeight = 18;
			double boxYPos;			
			
			Vector previousBoxYPos = new Vector();
			int vecSize = boxYPosVec.size();
			
			// Get the previous textbox y position
			if (firstBarSet == Boolean.FALSE)
				previousBoxYPos = (Vector)boxYPosVec.get(boxYPosVec.size()-1);
				
			// Adjust textbox y position, if the height of a bar is too small
			if ( (barY + boxHeight) > yZero && firstBarSet == Boolean.TRUE) {
				// adjust the bar position if it crosses the x-axis     
				boxYPos	= yZero - boxHeight;
			} else if (firstBarSet == Boolean.TRUE) {
				boxYPos = barY + barHeight / 2.0 - boxHeight / 2.0;	
			} else {
				double prevYPos = ((Double)previousBoxYPos.get(i)).doubleValue();
				boxYPos = barY + barHeight / 2.0 - boxHeight / 2.0;	
					
				// adjust the textbox height if it overlaps with the textbox underneath
				if (boxYPos + boxHeight >= prevYPos) {
					boxYPos = prevYPos - boxHeight;	
				}	
			}
			
			
			// Store previous barset y positions	
			tmpBoxYPos.add(new Double(boxYPos));	
		}
		
		boxYPosVec.add(new Vector(tmpBoxYPos));
		firstBarSet = Boolean.FALSE;
		
		return newVec;
	}
	
	// add bars
	private void addDataSets(Element parent, XYChartDataRetriever input, int xTicks, int numberOfBarsets, String yMax, String type) throws DataInputProcessingException {
		
		Vector boxYPosVec = new Vector();
		Vector prevVec = new Vector();
		Boolean firstBarSet = Boolean.TRUE;
		
		// need to compute y values for actual datapoints based on the number of pixels 
		// between each potential datapoint along y-axis
		// NumberFormatException not possible here
		double yLabelsMaxValue = Double.parseDouble(yMax);
		
		// draw bars for each barset
		try {
			for (int i = 0; i < numberOfBarsets; i++) {
				String [] yValues = input.getDatapointValuesFromDatasetWithPosition(String.valueOf(i));	
				if (yValues.length != xAxisValues.length) {
			 		// missing category 
					short code = MISSING_CATEGORY_NAMES;				
					String element = CATEGORY;				
					DataInputProcessingException e = new DataInputProcessingException(code, element);
					e.setWidgetType(type);
					throw e;
			 	}							
				// check if the barset is the first one
				firstBarSet = (i == 0 ? Boolean.TRUE : Boolean.FALSE);
				// store the latest barset
				prevVec = addBarset(parent, i, (yAxisLength/yLabelsMaxValue), yValues, xTicks, prevVec, boxYPosVec, firstBarSet, numberOfBarsets);				
			}
		
			Element gValueElement = generatedDocument.createElement("g");
			String stylesheetClass = "dataValues anchorAtMiddle";		
			gValueElement.setAttribute("id","exactValues");
			gValueElement.setAttribute("class", stylesheetClass);
			gValueElement.setAttribute("font-family", getGraphicFont());
			gValueElement.setAttribute("visibility", "hidden");
			gValueElement.setAttribute("onclick","toggleVisibility(\"exactValues\")");
			parent.appendChild(gValueElement);
		
			// draw textboxes for each barset
			for (int i = 0; i < boxYPosVec.size(); i++) {
				String [] yValues = input.getDatapointValuesFromDatasetWithPosition(String.valueOf(i));				

				Element gToggleElement = generatedDocument.createElement("g");
				String boxStylesheetClass = "dataValues anchorAtMiddle";		
				gToggleElement.setAttribute("id","textbox" + i);
				gToggleElement.setAttribute("class", boxStylesheetClass);
				gToggleElement.setAttribute("font-family", getGraphicFont());
				gToggleElement.setAttribute("visibility", "hidden");
				gToggleElement.setAttribute("onclick","toggleVisibility(\"exactValues\")");
				parent.appendChild(gToggleElement);

				addTextBoxes(gToggleElement, i, (yAxisLength/yLabelsMaxValue), xTicks, yValues, (Vector)boxYPosVec.get(i));
			}
			// add total values over each bar	
			String [] barYValues = input.getCategoryTotals();		
			addAllBarValues(gValueElement,(yAxisLength/yLabelsMaxValue), xTicks, barYValues);		
		} catch (DataInputProcessingException e) {
			e.setWidgetType(type);
			//System.out.println(e.getWidgetType() + " " + e.getElement() + " " + e.getMessage());
			throw e;
		}		
	}

	// specific to stack bar chart
	private void addDefaultInternalStylesheet(Element parent, int sets) {
		StringBuffer styleRules = new StringBuffer(COMMON_STYLE);
		styleRules.append(XYCHART_STYLE);
		
		String [] palette = (String [])graphicColourPalette.get(graphicColourPaletteName);
		// palette has limited number of colours, so loop if more sets
		int paletteLength = palette.length;		
		boolean hasLegend = !isLegendSuppressed();
		for (int i = 0, j = 0; i < sets; i++) {
		  j = (i + 1) % paletteLength;
		  j = (j == 0) ? paletteLength - 1 : j-1;			  			  
		  styleRules.append(" .dataset" + i + "{stroke-width:0.75pt; stroke:" + palette[j] + ";}"); 
		  if (hasLegend) {
		  	styleRules.append(" .shape" + i + "{stroke:none; fill:" + palette[j] + ";}");
		  }
		}
		
		CDATASection newCDATASection = generatedDocument.createCDATASection(styleRules.toString());
		
		Element newElement = generatedDocument.createElement("style");
		newElement.setAttribute("id", "defaultStyleRules");
		newElement.setAttribute("type", "text/css");
		newElement.appendChild(newCDATASection);
		parent.appendChild(newElement);
	}	
	
	// specific to stack bar chart
	private void addDefinitions(Element parent, int numberOfDatasets) {
		Element defsElement = generatedDocument.createElement("defs");
        parent.appendChild(defsElement);
        
        // default internal style rules
		addDefaultInternalStylesheet(defsElement, numberOfDatasets);		
	
		if (!isLegendSuppressed()){
			// id: squareshape
			Element gElement_square = generatedDocument.createElement("g");	
			gElement_square.setAttribute("id","legendsquareshapeWithoutLine");
		
    		Element squareElement = generatedDocument.createElement("rect");
			squareElement.setAttribute("width", "11");
			squareElement.setAttribute("height", "11");		
			gElement_square.appendChild(squareElement);
			defsElement.appendChild(gElement_square);		
		}
		
		addPreferenceIcon(defsElement);
	}	
		
	// add total value over each bar
	private void addExactValue(Element parent, double x, double y, String value) {
		addLabel(parent, value, "dataValues, anchorAtMiddle", x, y, 0);
	}
	
	// add text boxes besides each barset
	private void addTextBoxes(Element parent, int id, double yUnits, int xTicks, String [] input, Vector vect) {
		int length = input.length;  // number of bars for each barset
		
		// At coordinate 0
		int xZero = GRIDXOFFSET;
		int yZero = GRIDYOFFSET + yAxisLength;
		double xIntervalLength = (double)xAxisLength/xTicks;
		
		for (int i = 0; i < length; i++) {
			// determine text box position and dimensions
			double barWidth = xIntervalLength * 0.5;
			double barX = xZero + xIntervalLength * (i  + 0.5) - barWidth * 0.5;
			String exactValue = input[i]; 
			
			double textWidth = exactValue.length();
			double boxWidth = 16 + (textWidth - 1) * 6.0;
			double boxHeight = 18;
			
			double boxXPos = barX + barWidth - 8.0;
			double boxYPos = ((Double)vect.get(i)).doubleValue();
			
			// add text box
			addEachTextBox(parent, boxXPos, boxYPos, boxWidth, boxHeight, exactValue, id, i, "0");
		}
	}	
}
