package com.ibm.etools.pd.widget.viewer;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;

import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * <code>SVGXYChart</code> is an abstract base class providing implementation of methods
 * common to classes generating SVG graphics which have x,y axes.
 * 
 * @version %I%
 */
public abstract class SVGXYChart extends SVGDocumentGenerator implements IDataInputConstants, Serializable {
	/**
	 * Provides methods for retrieving the data from
	 * the configuration and data documents.
	 */
	protected XYChartDataRetriever dataRetriever;
	
	// determine grid dimensions from ratio
	protected final double XRATIO = 0.65;
	protected final double YRATIO = 0.65;
	
	// start drawing grid here
	protected static int GRIDXOFFSET = 60;
	protected final int GRIDYOFFSET = 50;
	
	// default chart dimensions
	protected int xAxisLength;
	protected int yAxisLength;

	// x axis label things we need to keep track of
	protected double [] xAxisValues;
	
	// default axis data label font size
	protected int axisLabelFontSize = DEFAULT_FONTSIZE;
	protected int xAxisLabelYValue;	
	
	// y axis maximum value (get from input)
	protected double yMaxValue;
	
	/**
	 * Sole constructor.	  
	 */ 
	SVGXYChart() {
		super();
		dataRetriever = new XYChartDataRetriever();
		xAxisLength = (int) (graphicWidth * XRATIO);
		yAxisLength = (int) (graphicHeight * YRATIO);	
		xAxisLabelYValue = GRIDYOFFSET + yAxisLength + axisLabelFontSize + 5;	
	}	
	
	/**
	 * Sets the document which is to be used for configuration data 
	 * retrieval and then retrieves the configuration data from that
	 * document. 
	 * 
	 * @see IGraphicDocumentGenerator#setConfigDocument(Document)
	 */
	public void setConfigDocument(Document newConfigDocument) throws DataInputProcessingException {
		super.setConfigDocument(newConfigDocument);
		if (getConfigDocument() == null) {
			return;
		}	
		dataRetriever.setConfigDocument(getConfigDocument());
				
		Hashtable configProperties = dataRetriever.getConfigurationAttributes();	
		if (configProperties == null) {
			return;  	
		}		
		
		String width = (String)configProperties.get(WIDTH);
		String height = (String)configProperties.get(HEIGHT);
		String fontFamily = (String)configProperties.get(FONT);
		String title = (String)configProperties.get(TITLE);
		String timestampPrefix = (String)configProperties.get(TIMESTAMP_PREFIX);
		String suppressLegend = (String)configProperties.get(SUPPRESS_LEGEND);
		String legendTitle = (String)configProperties.get(LEGEND_TITLE);
					
		if (width != null) {
			setGraphicWidth(width);
		}	
		
		if (height != null) {
			setGraphicHeight(height);
		}	
		
		if (fontFamily != null) {
			setGraphicFont(fontFamily);			
		}	
		
		if (title != null) {
			setGraphicTitle(title);			
		}	
		
		if (timestampPrefix != null) {
			setGraphicTimestampPrefix(timestampPrefix);			
		}	
		
		if (suppressLegend != null) {
			setSuppressLegend(suppressLegend);			
		}	
		
		if (legendTitle != null) {
			setGraphicLegendTitle(legendTitle);			
		}	
	}
	
	/**
	 * Sets the document which is to be used for data update 
	 * retrieval and then, if <code>configDocument</code> has not been
	 * set, retrieves the configuration data from that document.
	 * 
	 * @see IGraphicDocumentGenerator#setDataDocument(Document)
	 */
	public void setDataDocument(Document newDataDocument) throws DataInputProcessingException {
		super.setDataDocument(newDataDocument);
		dataRetriever.setDataDocument(getDataDocument());
		
		if (getConfigDocument() != null) {
			return;
		}	
		
		Hashtable configProperties = dataRetriever.getConfigurationAttributes();	
		if (configProperties == null) {
			return;  	
		}		
			
		String width = (String)configProperties.get(WIDTH);
		String height = (String)configProperties.get(HEIGHT);
		String fontFamily = (String)configProperties.get(FONT);
		String title = (String)configProperties.get(TITLE);
		String timestampPrefix = (String)configProperties.get(TIMESTAMP_PREFIX);
		String suppressLegend = (String)configProperties.get(SUPPRESS_LEGEND);
		String legendTitle = (String)configProperties.get(LEGEND_TITLE);
					
		if (width != null) {
			setGraphicWidth(width);
		}	
		
		if (height != null) {
			setGraphicHeight(height);
		}	
		
		if (fontFamily != null) {
			setGraphicFont(fontFamily);			
		}			
		
		if (title != null) {
			setGraphicTitle(title);			
		}
			
		if (timestampPrefix != null) {
			setGraphicTimestampPrefix(timestampPrefix);			
		}	
		
		if (suppressLegend != null) {
			setSuppressLegend(suppressLegend);			
		}	
		
		if (legendTitle != null) {
			setGraphicLegendTitle(legendTitle);			
		}	
	}
		
	/**
	 * Sets the graphic width and then computes the length of
	 * the x-axis as a factor of the graphic width.
	 * 
	 * @see IGraphicDocumentGenerator#setGraphicWidth(String)
	 */
	public void setGraphicWidth(String newGraphicWidth){
		super.setGraphicWidth(newGraphicWidth);
		xAxisLength = (int) (graphicWidth * XRATIO);
	}		
	
	/**
	 * Sets the graphic height and then computes the length of
	 * the y-axis as a factor of the graphic height.
	 * 
	 * @see IGraphicDocumentGenerator#setGraphicHeight(String)
	 */
	public void setGraphicHeight(String newGraphicHeight) {
		super.setGraphicHeight(newGraphicHeight);
		yAxisLength = (int) (graphicHeight * YRATIO);
		xAxisLabelYValue = GRIDYOFFSET + yAxisLength + axisLabelFontSize + 5;
	}	
	
	
	protected void addJavaScriptFunctions(Element parent, int numberOfSets) {		
		super.addJavaScriptFunctions(parent, numberOfSets);
		
		Element newElement = generatedDocument.createElement("script");
		newElement.setAttribute("type", JAVASCRIPT_VERSION);
		parent.appendChild(newElement);
	
		ECMAScriptGenerator jscript = new ECMAScriptGenerator();
		StringBuffer functions = new StringBuffer(jscript.generateXYChartHelperFunctions());				
			
		CDATASection newCDATASection = generatedDocument.createCDATASection(functions.toString());
		newElement.appendChild(newCDATASection);
	}	
	
	/**
	 * Add one text box (with a value inside it)
	 * 
	 * @param parent element to which the textbox-related elements are appended
	 * @param x x-coordinate of the text box
	 * @param y y-coordinate of the text box
	 * @param width width of the text box
	 * @param height height of the text box
	 */
	protected void addEachTextBox(Element parent, double x, double y, double width, double height, String value, int dataSet, int next, String crossXAxisFlag) {
		String stylesheetClass = "dataValues anchorAtMiddle";
		
		Element rectElement = generatedDocument.createElement("rect");
		rectElement.setAttribute("x", Double.toString(x));
		rectElement.setAttribute("y", Double.toString(y));
		rectElement.setAttribute("width", Double.toString(width));
		rectElement.setAttribute("height", Double.toString(height));
		rectElement.setAttribute("fill", "#FFFFCC");
		rectElement.setAttribute("stroke", "black");
		rectElement.setAttribute("stroke-width", "1");
		rectElement.setAttribute("id", "textboxRect" + Integer.toString(dataSet) + Integer.toString(next));
		rectElement.setAttribute("crossXAxisFlag", crossXAxisFlag);
		parent.appendChild(rectElement);

		// Set the text position inside the box		
		double textX = x + width / 2.0;
		double textY = y + 13;

		Element textElement = addLabel(parent,value, stylesheetClass, textX, textY, 0);
		textElement.setAttribute("id", "textboxText" + dataSet + next);
	}	
		
	/**
	 * Adds the grid for the x,y axes and draws tick lines on them,
	 * positioning them based on the y-axis marker values.
	 * 
	 * @param parent element to which the grid-related elements are appended
	 * @param xTicks number of ticks to draw on the x-axis
	 * @param yTicks number of ticks to draw on the y-axis
	 * @param yAxisLabels the set of y-axis range marker values
	 */
	protected void addGrid(Element parent, int xTicks, int yTicks, String [] yAxisLabels) {
		if (xTicks < 1 || yTicks < 1)
			return;		
		
		double deltaY = (double)yAxisLength/yTicks;
		double deltaX = (double)xAxisLength/xTicks;
		double nextY = 0;
		double nextX = 0;
		double offsetY = 0;
		double prevY = 0;
		
		double yZero = GRIDYOFFSET + yAxisLength;
		// NumberFormatException not possible here
		double rangeMaxValue = Double.parseDouble(yAxisLabels[yTicks - 1]);
		double yUnitLength = (double)yAxisLength/rangeMaxValue;		
		double maxY = yZero - rangeMaxValue * yUnitLength;
		
		// solid grid line group
		Element newElement = generatedDocument.createElement("g");
		newElement.setAttribute("id", "solidgridline");
		Element gElement = (Element)parent.appendChild(newElement);
		
		newElement = generatedDocument.createElement("polyline");
		newElement.setAttribute("class", "gridline");
		newElement.setAttribute("points", GRIDXOFFSET + " " + maxY + " " +
		                                  (GRIDXOFFSET + xAxisLength) + " " + maxY);
		gElement.appendChild(newElement);
		
		// xlink the rest of the solid line
		for (int i = 0; i < (yTicks - 1); i++) {
			// NumberFormatException not possible here
			double yLabelValue = Double.parseDouble(yAxisLabels[i]);
			nextY = yZero - yUnitLength * yLabelValue;
			offsetY = nextY - maxY;
			newElement = generatedDocument.createElement("use");
			newElement.setAttribute("xlink:href", "#solidgridline");
			newElement.setAttribute("x", "0");
			newElement.setAttribute("y", Double.toString(offsetY));    
			parent.appendChild(newElement);
		}	
		
		// tick line group		
		newElement = generatedDocument.createElement("g");
		newElement.setAttribute("id", "tickline");
		gElement = (Element)parent.appendChild(newElement);
		// left side
		newElement = generatedDocument.createElement("polyline");
		newElement.setAttribute("class", "grid");
		newElement.setAttribute("points", GRIDXOFFSET + " " + maxY + " " +
		                                  (GRIDXOFFSET + 5) + " " + maxY);
		gElement.appendChild(newElement);
		// right side
		newElement = generatedDocument.createElement("polyline");
		newElement.setAttribute("class", "grid");
		newElement.setAttribute("points", 
		                     (GRIDXOFFSET + xAxisLength) + " " + maxY + " " + (GRIDXOFFSET + xAxisLength - 5) + " " + maxY);
		gElement.appendChild(newElement);
		
		//xlink the rest of the tick lines
		for (int i = 0; i < (yTicks - 1); i++) {
			// NumberFormatException not possible here
			double yLabelValue = Double.parseDouble(yAxisLabels[i]);
			nextY = yZero - yUnitLength * yLabelValue;
			offsetY = nextY - maxY;
				
			newElement = generatedDocument.createElement("use");
			newElement.setAttribute("xlink:href", "#tickline");
			newElement.setAttribute("x", "0");
			newElement.setAttribute("y", Double.toString(offsetY));    
			parent.appendChild(newElement);
		}
		
		Vector dashGridYPos = calculateDashGridYPositions(yAxisLabels);
		// NumberFormatException not possible here
		double maxDashY = 0;
		if (dashGridYPos != null && dashGridYPos.size() > 0) {
			double maxDashYLabel = Double.parseDouble((String)dashGridYPos.lastElement());		
			maxDashY = yZero - yUnitLength * maxDashYLabel;


			// dashed grid line group
			newElement = generatedDocument.createElement("g");
			newElement.setAttribute("id", "dashgridline");
			gElement = (Element)parent.appendChild(newElement);
		
			newElement = generatedDocument.createElement("polyline");
			newElement.setAttribute("class", "griddashline");
			newElement.setAttribute("points", 
            	 GRIDXOFFSET + " " + maxDashY + " " + (GRIDXOFFSET + xAxisLength) + " " + maxDashY);
			gElement.appendChild(newElement);
		}


		//xlink the rest of the dashed grid lines
		for (int i = 0; i < (yTicks - 1); i++) {
			// NumberFormatException not possible here
			double yDashValue = Double.parseDouble((String)dashGridYPos.get(i));
			nextY = yZero - yUnitLength * yDashValue;
			offsetY = nextY - maxDashY;
			
			newElement = generatedDocument.createElement("use");
			newElement.setAttribute("xlink:href", "#dashgridline");
			newElement.setAttribute("x", "0");
			newElement.setAttribute("y", Double.toString(offsetY));    
			parent.appendChild(newElement);
		} 
		
		// grid border lines
		newElement = generatedDocument.createElement("polyline");
		newElement.setAttribute("class", "grid");
		newElement.setAttribute("fill", "none");
		newElement.setAttribute("points", GRIDXOFFSET + " " + GRIDYOFFSET + ",  " + 
		                                  GRIDXOFFSET + " " + (GRIDYOFFSET + yAxisLength) + ",  " +
		                                  (GRIDXOFFSET + xAxisLength) + " " + (GRIDYOFFSET + yAxisLength) + ", " + 
		                                  (GRIDXOFFSET + xAxisLength) + " " + GRIDYOFFSET);
	
		parent.appendChild(newElement);
		
		// xAxis tick line group
		newElement = generatedDocument.createElement("g");
		newElement.setAttribute("id", "xtickline");
		gElement = (Element)parent.appendChild(newElement);
		
		newElement = generatedDocument.createElement("polyline");
		newElement.setAttribute("class", "grid");
		newElement.setAttribute("points", (GRIDXOFFSET + deltaX) + " " + (GRIDYOFFSET + yAxisLength - 5) + " " + 
		                                  (GRIDXOFFSET + deltaX) + " " + (GRIDYOFFSET + yAxisLength));
		gElement.appendChild(newElement);
		
		//xlink the rest of the tick lines
		for (int i = 0; i < ( xTicks - 2); i++) {
			nextX = nextX + deltaX;
			newElement = generatedDocument.createElement("use");
			newElement.setAttribute("xlink:href", "#xtickline");
			newElement.setAttribute("x", Double.toString(nextX));
			newElement.setAttribute("y", "0");    
			parent.appendChild(newElement);
		}		
	}	         
	
	
	/**
	 * Adds the titles to the SVG graphic. This includes the main title,
	 * legend title (if appropriate), x-axis label and y-axis label.
	 * 
	 * @param parent element to which the title-related elements are appended
	 * @param input the titles
	 */	
	protected void addTitles(Element parent, Hashtable input) {		
		String title, stylesheetClass = null;		
		int fontSize, rotation = 0;
		Element gElement = generatedDocument.createElement("g");
		
		// add a font group for the title and then add the title
		title = getGraphicTitle();
		if (title != null) {
			stylesheetClass = "graphTitle anchorAtStart";
			gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id", "graphTitle");
			gElement.setAttribute("class", stylesheetClass);
			gElement.setAttribute("font-family", getGraphicFont());
			parent.appendChild(gElement);				
			// default title font size
			fontSize = DEFAULT_FONTSIZE + TITLE_FONTSIZE_DELTA;
			addLabel(gElement, title, null, 10, fontSize + 8, rotation);
		}
		
		// add a font group for the labels and then add the labels
		// x-axis title
		title = (String)input.get(X_TITLE);
		if (title != null) {
			stylesheetClass = "labelsValues anchorAtMiddle";
			gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id", "labelsValues0");
			gElement.setAttribute("class", stylesheetClass);
			gElement.setAttribute("font-family", getGraphicFont());
			parent.appendChild(gElement);				
			// default axis label font size
			fontSize = DEFAULT_FONTSIZE + LABEL_FONTSIZE_DELTA;
			addLabel(gElement, title, null, GRIDXOFFSET + xAxisLength/2, xAxisLabelYValue + fontSize + 10, rotation);
		}
		// y-axis title
		title = (String)input.get(Y_TITLE);
		if (title != null) {
			stylesheetClass = "labelsValues anchorAtMiddle";
			gElement = generatedDocument.createElement("g");
			gElement.setAttribute("id", "labelsValues1");
			gElement.setAttribute("class", stylesheetClass);
			gElement.setAttribute("font-family", getGraphicFont());
			parent.appendChild(gElement);				
			rotation = 270;
			addLabel(gElement, title, null, 25, GRIDYOFFSET + yAxisLength / 2.0, rotation);
		}
	}	


	/**
	 * Adds the axis data labels to the x- and y-axes.
	 * 
	 * @param parent element to which the label-related elements are appended
	 * @param xAxisLabels data labels for the x-axis
	 * @param yAxisLabels data labels for the y-axis
	 */
	protected void addAxisLabels(Element parent, String [] xAxisLabels, String [] yAxisLabels) {
		// get from stylesheet !!!!
		String XstylesheetClass = "dataValues anchorAtMiddle";
		String YstylesheetClass = "dataValues anchorAtEnd";		
		int rotation = 0;	
		int xLabels = xAxisLabels.length;
		int yLabels = yAxisLabels.length;
		
		Element gElement = null;		
		double delta = 0;
		double next = 0;
		
		// x Axis labels:
		// add font group and then add axis data labels to it
		gElement = generatedDocument.createElement("g");
		gElement.setAttribute("id", "dataValues0");
		gElement.setAttribute("class", XstylesheetClass);
		gElement.setAttribute("font-family", getGraphicFont());
		parent.appendChild(gElement);	
		XstylesheetClass = null;
		// needed for use in addLine method
		xAxisValues = new double[xLabels];
		delta = (double)xAxisLength/xLabels;
		next = GRIDXOFFSET + (delta/2.0);		
		int y = xAxisLabelYValue;
		// needed as an offset for x axis title placement
		xAxisLabelYValue = y;
		for (int i = 0; i < xLabels; i++) {	
			addLabel(gElement, xAxisLabels[i], XstylesheetClass, next, y, rotation);		
			xAxisValues[i] = next;
			next = next + delta;
		}
		
		// y Axis labels:
		// add font group and then add axis data labels to it
		gElement = generatedDocument.createElement("g");
		gElement.setAttribute("id", "dataValues1");
		gElement.setAttribute("class", YstylesheetClass);
		gElement.setAttribute("font-family", getGraphicFont());
		parent.appendChild(gElement);	
		YstylesheetClass = null;
		
		
		// Max y axis value
		// NumberFormatException not possible here
		double rangeMaxValue = Double.parseDouble(yAxisLabels[yLabels - 1]);
		double yUnitLength = (double)yAxisLength/rangeMaxValue;
		
		double yZero = GRIDYOFFSET + yAxisLength;
		next = yZero + (DEFAULT_FONTSIZE / 2.0);
		int x = GRIDXOFFSET - 5;
		
		// do the origin first
		addLabel(gElement, "0", YstylesheetClass, x, next, rotation);
		for (int i = 0; i < yLabels; i++) {	
			// NumberFormatException not possible here
			double yLabelValue = Double.parseDouble(yAxisLabels[i]);
			next = yZero - yUnitLength * yLabelValue + DEFAULT_FONTSIZE / 2.0;
			addLabel(gElement, yAxisLabels[i], YstylesheetClass, x, next, rotation);		
		}				
	}
	
	/**
	 * Calculate the y position of all dash grid lines
	 * 
	 * @param yAxisLabels the set of y-axis range marker values
	 */
	protected Vector calculateDashGridYPositions(String [] yAxisLabels) {
		double prevY = 0;
		double yLabelValue = 0;		
		double nextDashY = 0;
		Vector dashY = new Vector(); 
		
		// Calculate the y position for each dashed grid line
		for (int i = 0; i < yAxisLabels.length; i++) {	
			// NumberFormatException not possible here
			yLabelValue = Double.parseDouble(yAxisLabels[i]);
			nextDashY = (prevY + yLabelValue) / 2.0;
			dashY.add(Double.toString(nextDashY));
			prevY = yLabelValue;
		}
		return dashY;
	}
}
