package reports;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;

import com.ibm.etools.comptest.definition.TestcaseDefinition;
import com.ibm.etools.comptest.extension.ReportGeneratorImpl;
import com.ibm.etools.comptest.instance.ExecutionAttempt;
import com.ibm.etools.comptest.instance.ExecutionContainer;
import com.ibm.etools.comptest.instance.ExecutionRecord;
import com.ibm.etools.comptest.instance.TestcaseInstance;

/**
 * This class generates text reports for a test case definition and for one or more
 * executions.  Basically it navigates these objects collecting data to be presented as
 * the reports.
 * 
 * <p>For the selected test case definition, this class will generate a report that
 * presents the name, and the location of the test case and of its test cases instances.  For the
 * instances it will also presente the number of executions.
 * 
 * <p>For the selected executions, this class will generate a report that
 * presents the name, the location, the status of each test element in the execution and
 * the status and the timestamp of each event collected.
 * 
 * <p>A quick way of testing and integrating this report into the Component Test tool is to 
 * register it in the <b>Local report definition</b> preference page.  Please refer to the 
 * documentation for the full details on how to do this.  The class name of this report is 
 * <b>reports.TextReport</b> and the classpath is the full path - relative to the file system 
 * - of the <b>bin</b> directory  of this project.
 */

public class TextReport
extends ReportGeneratorImpl
{
	/**
	 * Returns the extension of the report file that will be created for a specific object type, 
	 * and whether one or many objects of that type have been selected..
	 * <br>org.eclipse.core.resources.IProject - Workbench project 
	 * <br>org.eclipse.core.resources.IFolder - Workbech folder 
	 * <br>com.ibm.etools.comptest.definition.TestcaseDefinition - Test case definition 
	 * <br>com.ibm.etools.comptest.instance.TestcaseInstance - Test case instance 
	 * <br>com.ibm.etools.comptest.instance.ExecutionContainer - Execution object 
	 * 
	 * @param String The object type of the report to be created
	 * @param boolean Indicates whether the report is for a single object of the selected type
	 * @return String The resource extension
	 */
	public String getReportFileExtension(String id, boolean single) 
	{
		if(ExecutionContainer.class.getName().equals(id))
			return "txt";

		if(TestcaseDefinition.class.getName().equals(id))
			return "txt";
		
		return super.getReportFileExtension(id, single);
	}
	
	/**
	 * This method is executed if:
	 * <br>This report is registered to handle a selection of one testcase definition 
	 * <br>The user selected a valid testcase definition 
	 * 
	 * @param IFile The report file that <b>will</b> be created by the Component Test Plugin
	 * @param IContainer The input for the report
	 * @return InputStream The content of the report
	 * @exception Exception Any exception thrown by this method will be treated.
	 */
	public InputStream generate(IFile reportFile, TestcaseDefinition testcaseDefinition) 
	throws Exception
	{
		StringBuffer sb = new StringBuffer();

		//Testcase Definition attributes
		sb.append("Testcase Definition:");
		sb.append("\n\tname: ").append(testcaseDefinition.getName());
		sb.append("\n\tURI: ").append(getLocation(testcaseDefinition));
		
		//Iterating throught all the Testcase Instances
		for (Iterator i = testcaseDefinition.getTaskInstances().iterator(); i.hasNext();)
		{
			sb.append("\n\t\tTestcase Instance:");
			
			TestcaseInstance testcaseInstance = (TestcaseInstance) i.next();
			sb.append("\n\t\t\tName: ").append(testcaseInstance.getName());
			sb.append("\n\t\t\tURI: ").append(getLocation(testcaseInstance));
			sb.append("\n\t\t\t# Executions: ").append(testcaseInstance.getExecutionContainers().size());
		}
		
		return new ByteArrayInputStream(sb.toString().getBytes());		
	}

	/**
	 * This method is executed if:
	 * <br>This report is registered to handle a selection of one ExecutionContainer
	 * <br>The user selected an execution
	 * 
	 * @param IFile The report file that <b>will</b> be created by the Component Test Plugin
	 * @param IContainer The input for the report
	 * @return InputStream The content of the report
	 * @exception Exception Any exception thrown by this method will be treated.
	 */
	public InputStream generate(IFile reportFile, ExecutionContainer executionContainer) 
	throws Exception
	{
		return generate(reportFile, new ExecutionContainer[]{executionContainer});
	}

	/**
	 * This method is executed if:
	 * <br>This report is registered to handle a selection of more than one ExecutionContainer
	 * <br>The user selected a set of valid executions
	 * 
	 * @param IFile The report file that <b>will</b> be created by the Component Test Plugin
	 * @param IContainer[] The input for the report
	 * @return InputStream The content of the report
	 * @exception Exception Any exception thrown by this method will be treated.
	 */
	public InputStream generate(IFile reportFile, ExecutionContainer[] executionContainers)
	throws Exception
	{
		StringBuffer sb = new StringBuffer();

		for(int k = 0, max = executionContainers.length; k < max; k++)
		{
			//Execution Container attributes
			sb.append("(" + (k+1) + ") Execution Container:");
			sb.append("\n\tname: ").append(executionContainers[k].getName());
			sb.append("\n\tURI: ").append(getLocation(executionContainers[k]));
	
			//Getting all the Execution Attempts
			List executionAttempts = executionContainers[k].getExecutionAttempts();
	
			//Iterating throught all the Execution Attempts
			for (Iterator i = executionAttempts.iterator(); i.hasNext();)
			{
				ExecutionAttempt executionAttempt = (ExecutionAttempt) i.next();
	
				//Execution Attempt attribues			
				sb.append("\n\tExecution Attempt:");
				if(executionAttempt.getStatus() != null)
					sb.append("\n\t\tStatus:").append(executionAttempt.getStatus().getName());
				else
					sb.append("\n\t\tStatus:").append("No status set");
	
				//Getting all the Execution Records of the Execution Attempt
				List executionRecords = executionAttempt.getExecutionRecords();
				for (Iterator j = executionRecords.iterator(); j.hasNext();)
				{
					ExecutionRecord executionRecord = (ExecutionRecord) j.next();
	
					//Execution Record attribues			
					sb.append("\n\t\tExecution Record:");
					sb.append("\n\t\t\tModel Timestamp:").append(executionRecord.getModelTimestamp());
					if(executionRecord.getStatus() != null)
						sb.append("\n\t\t\tStatus:").append(executionRecord.getStatus().getName());
					else
						sb.append("\n\t\t\tStatus:").append("No status set");
				}
			}
			
			if(k < (max-1))
				sb.append("\n\n");
		}
		
		return new ByteArrayInputStream(sb.toString().getBytes());
	}		
}
