package reports.svg;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;

import com.ibm.etools.comptest.instance.ExecutionStatus;
import com.ibm.etools.comptest.instance.TestcaseInstance;

/**
 * This class contains 2 classes that are used to simplify the graph creation
 * by aggregating the required information in a convinent way.
 */
public class DayHelper
{
	/**
	 * The instances of this class contain the information that will be 
	 * plotted for a day in the graph.
	 */
	public static class GraphDay
	implements Cloneable
	{
		public static final int EXPECTED_ATTEMPT = -107;
		public static final int EXPECTED_PASS = -106;
		public static final int EXPECTED_FAIL = -105;
		public static final int EXPECTED_SOFT_FAIL = -104;
		
		public static final int ATTEMPT = -103;
		public static final int PASS = -102;
		public static final int FAIL = -101;
		public static final int SOFT_FAIL = -100;
		
		private static boolean hasExpectedAttempt = false;
		private static boolean hasExpectedPass = false;
		private static boolean hasExpectedFail = false;
		private static boolean hasExpectedSoftFail = false;
		
		private static boolean hasAttempt = false;
		private static boolean hasPass = false;
		private static boolean hasFail = false;
		private static boolean hasSoftFail = false;
		
		private String day;

		private int expectedAttempt;
		private int expectedPass;
		private int expectedFail;
		private int expectedSoftFail;

		private int attempt;
		private int pass;
		private int fail;
		private int softFail;
		
		/**
		 * Constructor for GraphDay
		 * 
		 * @param String The day in the format "yyyyMMdd"
		 */
		public GraphDay(String day)
		{
			try
			{
				SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
				this.day = format.format(format.parse(day));
				if(!this.day.equals(day))
					throw new ParseException("Invalid date format.", 0);
			}
			catch (ParseException pe)
			{
				throw new RuntimeException(pe.getMessage());
			}			
		}
		
		public static void reset()
		{
			hasExpectedAttempt = false;
			hasExpectedPass = false;
			hasExpectedFail = false;
			hasExpectedSoftFail = false;
			
			hasAttempt = false;
			hasPass = false;
			hasFail = false;
			hasSoftFail = false;
		}
		
		/**
		 * Returns the day in the yyyyMMdd format.
		 * @return String
		 */
		public String getDay()
		{
			return day;
		}
		
		public String getDayOfMonth()
		{
			return day.substring("yyyyMM".length());
		}
		
		/**
		 * Returns the day in the DateFormat.SHORT format.
		 * @return String
		 */
		public String getFormattedDay()
		{
			String day = getDay();
			SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
			try
			{
				Date dateObj = formatter.parse(day);
				
				// Display format: SHORT
				DateFormat df = SimpleDateFormat.getDateInstance(DateFormat.SHORT);
				return df.format(dateObj);
			}
			catch(ParseException pe)
			{
			}
			
			return day;
		}

		/**
		 * Returns the statistic value for the indicated flag.
		 * @param int statistic flag
		 * @return int
		 */
		public int getStatistic(int flag)
		{
			switch(flag)
			{
				case EXPECTED_ATTEMPT:
					return expectedAttempt;
				
				case EXPECTED_PASS:
					return expectedPass;
				
				case EXPECTED_FAIL:
					return expectedFail;
					
				case EXPECTED_SOFT_FAIL:
					return expectedSoftFail;
				
				case ATTEMPT:
					return attempt;
				
				case PASS:
					return pass;
					
				case FAIL:
					return fail;
					
				case SOFT_FAIL:
					return softFail;
					
				default:
					return -1;
			}
		}

		/**
		 * Returns true if the specied statistic value was set.
		 * @param int statistic flag
		 * @return boolean
		 */
		public static boolean hasStatistic(int flag)
		{
			switch(flag)
			{
				case EXPECTED_ATTEMPT:
					return hasExpectedAttempt;
				
				case EXPECTED_PASS:
					return hasExpectedPass;
				
				case EXPECTED_FAIL:
					return hasExpectedFail;
					
				case EXPECTED_SOFT_FAIL:
					return hasExpectedSoftFail;
				
				case ATTEMPT:
					return hasAttempt;
				
				case PASS:
					return hasPass;
					
				case FAIL:
					return hasFail;
					
				case SOFT_FAIL:
					return hasSoftFail;
					
				default:
					return false;
			}
		}
		
		/**
		 * Adds a value to the statistic value for the indicated flag.
		 * @param int statistic flag
		 * 
		 */
		public void addStatistic(int flag, int value)
		{
			switch(flag)
			{
				case EXPECTED_ATTEMPT:
					expectedAttempt+=value;
					hasExpectedAttempt = (expectedAttempt>0);
					break;
				
				case EXPECTED_PASS:
					expectedPass+=value;
					hasExpectedPass = (expectedPass>0);
					break;
				
				case EXPECTED_FAIL:
					expectedFail+=value;
					hasExpectedFail = (expectedFail>0);
					break;
					
				case EXPECTED_SOFT_FAIL:
					expectedSoftFail+=value;
					hasExpectedSoftFail = (expectedSoftFail>0);
					break;
				
				case ATTEMPT:
					attempt+=value;
					hasAttempt = (attempt>0);
					break;
				
				case PASS:
					pass+=value;
					hasPass = (pass>0);
					break;
					
				case FAIL:
					fail+=value;
					hasFail = (fail>0);
					break;
					
				case SOFT_FAIL:
					softFail+=value;
					hasSoftFail = (softFail>0);
					break;
			}
		}

		/**
		 * @return GraphDay the current GraphDay after the day increment
		 */
		public GraphDay incrementDay()
		{
			try
			{
				SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
				Date date = format.parse(day);
				format.getCalendar().add(Calendar.DAY_OF_MONTH, 1);
				day = format.format(format.getCalendar().getTime());
			}
			catch(ParseException pe)
			{
				throw new RuntimeException(pe.getMessage());
			}
			
			return this;
		}

		/**
		 * Creates a clone of the GraphDay instance.
		 * 
		 * @see java.lang.Object#clone()
		 */
		public Object clone()
		{
			GraphDay graphDay = new GraphDay(day);

			graphDay.expectedAttempt = expectedAttempt;
			graphDay.expectedPass = expectedPass;
			graphDay.expectedFail = expectedFail;
			graphDay.expectedSoftFail = expectedSoftFail;

			graphDay.attempt = attempt;
			graphDay.pass = pass;
			graphDay.fail = fail;
			graphDay.softFail = softFail;
			
			return graphDay;
		}	
	}

	/**
	 * The instances of this class contain the testcase instance status per day.
	 */
	public static class DataDay
	implements Comparable
	{
		private String day;
		private Hashtable executionStatusByTestcaseInstance;
		
		/**
		 * Constructor for DataDay
		 * 
		 * @param String The day in the format "yyyyMMdd"
		 */
		public DataDay(String day)
		{
			try
			{
				SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
				this.day = format.format(format.parse(day));
				if(!this.day.equals(day))
					throw new ParseException("Invalid date format.", 0);
			}
			catch (ParseException pe)
			{
				throw new RuntimeException(pe.getMessage());
			}
				
			executionStatusByTestcaseInstance = new Hashtable();
		}
		
		/**
		 * Returns the day in the format yyyyMMdd
		 * @return String
		 */
		public String getDay()
		{
			return day;
		}
		
		/**
		 * Adds the status for a testcase instance.  Only the last added status will be
		 * considered.
		 * 
		 * @param TestcaseInstance
		 * @param ExecutionStatus
		 */
		public void addStatus(TestcaseInstance testcaseInstance, ExecutionStatus status)
		{
			executionStatusByTestcaseInstance.put(testcaseInstance, status);
		}
		
		/**
		 * Returns the status of a testcase instance.
		 * 
		 * @param TestcaseInstance
		 * @return ExecutionStatus
		 */
		public ExecutionStatus getStatus(TestcaseInstance testcaseInstance)
		{
			return (ExecutionStatus)executionStatusByTestcaseInstance.get(testcaseInstance);
		}
		
	    /**
	     * Returns an iterator over the testcase instances in this day.  The elements are
	     * returned in no particular order.
	     *
	     * @return an iterator over the testcase instances in this day.
	     */
		public Iterator testcaseInstanceIterator()
		{
			return executionStatusByTestcaseInstance.keySet().iterator();
		}
		
		/**
		 * @see java.lang.Object#equals(Object)
		 */
		public boolean equals(Object object)
		{
			if(this == object)
				return true;
				
			if((object != null) && (object instanceof DataDay))
				return (hashCode() == object.hashCode());
				
			return false;
		}
		
		/**
		 * @see java.lang.Object#hashCode()
		 */
		public int hashCode()
		{
			return day.hashCode();
		}
		
		/**
		 * @see java.lang.Comparable#compareTo(Object)
		 */
		public int compareTo(Object object)
		{
			DataDay graphDay = (DataDay)object;
			if(this.equals(graphDay))
				return 0;
				
			if(graphDay == null)
				return 1;
				
			return ((hashCode()-graphDay.hashCode())>0)?1:-1;
		}
	}
}
