package reports.svg;

/*
 * Disclaimer:
 * The following source code is sample code created by IBM Corporation. 
 * This sample code is not part of any standard IBM product and is provided 
 * to you solely for the purpose of assisting you in the development of your 
 * applications. The code is provided 'AS IS', without warranty or condition 
 * of any kind. IBM shall not be liable for any damages arising out of your 
 * use of the sample code, even if IBM has been advised of the possibility of 
 * such damages.
 */

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.ibm.etools.comptest.extension.ReportGeneratorImpl;
import com.ibm.etools.comptest.instance.ExecutionAttempt;
import com.ibm.etools.comptest.instance.ExecutionContainer;
import com.ibm.etools.comptest.instance.ExecutionStatus;
import com.ibm.etools.comptest.instance.TestcaseInstance;
import com.ibm.etools.pd.widget.viewer.SVGLineChart;

import reports.svg.DayHelper.DataDay;
import reports.svg.DayHelper.GraphDay;

/**
 * This class generates an SVG report for a set of test case instances.
 * 
 * <p>For the selected test case instances, this class will generate a report that
 * presents a graph with the number of execution attempts, and how many succeeded, failed or 
 * soft failed, ordered by day.
 * 
 * <p>To use the report, you must integrate the generator into Component Test. A quick way to do this is by
 * registering it in the <b>Local report definition</b> preference page.  Please refer to the 
 * documentation for the full details on how to do this.  The class name of this report is 
 * reports.svg.SVGReport and the classpath is the full path - relative to the file system 
 * - of the <b>bin</b> directory  of this project.
 * 
 * <p><b>IMPORTANT:</b> Read the comments for the method getPreferencePageDirectory() if
 * you are experiencing problems trying to open the SVG preference page.
 */

public class SVGReport 
extends ReportGeneratorImpl
{
	/**
	 * Returns the extension of the report file that will be created for a specific object type, 
	 * and whether one or many objects of that type have been selected..
	 * <br>org.eclipse.core.resources.IProject - Workbench project 
	 * <br>org.eclipse.core.resources.IFolder - Workbech folder 
	 * <br>com.ibm.etools.comptest.definition.TestcaseDefinition - Test case definition 
	 * <br>com.ibm.etools.comptest.instance.TestcaseInstance - Test case instance 
	 * <br>com.ibm.etools.comptest.instance.ExecutionContainer - Execution object 
	 * 
	 * @param String The object type of the report to be created
	 * @param boolean Indicates whether the report is for a single object of the selected type
	 * @return String The resource extension
	 */
	public String getReportFileExtension(String id, boolean single)
	{
		if (TestcaseInstance.class.getName().equals(id) && (!single))
			return "html";

		return super.getReportFileExtension(id, single);
	}

	/**
	 * This method is executed if:
	 * <br>This report is registered to handle a selection of more than one TestcaseInstance
	 * <br>The user selected a valid set of executions
	 * 
	 * @param IFile The report file that <b>will</b> be created by the Component Test Plugin
	 * @param TestcaseInstance[] The input for the report
	 * @return InputStream The content of the report
	 * @exception Exception Any exception thrown by this method will be treated.
	 */
	public InputStream generate(IFile reportFile, TestcaseInstance[] testcaseInstances) 
	throws Exception
	{
		//Resets the GraphDay static information
		GraphDay.reset();
		
		// Retrieve the test instances status per day
		ArrayList dataDays = analyseTestcaseInstances(testcaseInstances);
		if(dataDays.isEmpty())
			return null;
		
		// Sort the days, fills the missing days with a previous information and 
		// carries the results from previous days
		GraphDay[] graphDays = createGraphDays(dataDays);
		
		// Loads the statistics from file with the expected execution values
		int numberOfTestcaseInstances = loadExpectedExecutionValues(reportFile.getParent(), graphDays);

		// Counts the number of testcase instances with data for the graph
		int numberOfTestcaseInstancesWithData = countTestcaseInstancesWithData(testcaseInstances);
		if(numberOfTestcaseInstances < numberOfTestcaseInstancesWithData)
			numberOfTestcaseInstances = numberOfTestcaseInstancesWithData;
		
		// Returns the report content as an InputStream and creates the
		// svg and css files
		return generate(reportFile, graphDays, numberOfTestcaseInstances);
	}
		
		
		
	//
	// Methods for collecting the testcase instances data 
	//	
		

		
	/**
	 * Analyses the test case instances array collecting the daily status.
	 * 
	 * <p>Note: Only the last execution of a test case instance of a particular day will
	 * be considered.
	 * 
	 * @param TestcaseInstance[] the TestcaseInstances to be analysed.
	 * @return ArrayList List of DataDay - the testcase instances status per day;
	 */
	protected ArrayList analyseTestcaseInstances(TestcaseInstance[] testcaseInstances)
	{
		Hashtable dataDayByDay = new Hashtable();
		
		for(int i = 0; i < testcaseInstances.length; i++)
		{
			Hashtable lastExecutionByDay = getLastExecutionByDay(testcaseInstances[i], true);
			for(Iterator iter = lastExecutionByDay.keySet().iterator(); iter.hasNext();)
			{
				String day = (String)iter.next();
				ExecutionContainer executionContainer = (ExecutionContainer)lastExecutionByDay.get(day);
				ExecutionAttempt executionAttempt = getTestcaseInstanceExecutionAttempt(executionContainer);

				DataDay dataDay = (DataDay)dataDayByDay.get(day);
				if(dataDay == null)
				{
					dataDay = new DataDay(day);
					dataDayByDay.put(day, dataDay);
				}
				dataDay.addStatus(testcaseInstances[i], executionAttempt.getStatus());
			}
		}
		
		return new ArrayList(dataDayByDay.values());
	}
	
	/**
	 * This methods executes 3 operations:
	 * <br>1.Sorts the execution days
	 * <br>2.Keep the last status of a testcase instance in order to repeat it on the next
	 * day if a testcase instance was not executed.
	 * <br>3.Navigates the ArrayList looking for missing days in the key collection
	 * adding the previous status value
	 * 
	 * @param Collection the testcase instance status per day.
	 * @param GraphDay[] the sorted daily status
	 */
	protected GraphDay[] createGraphDays(ArrayList dataDays)
	{
		DataDay[] dataDayArray = (DataDay[])dataDays.toArray(new DataDay[dataDays.size()]);
		Arrays.sort(dataDayArray);
		
		ArrayList graphDays = new ArrayList();
		Hashtable lastStatusByTestcaseInstance = new Hashtable();

		for(int i=0,max=dataDayArray.length; i<max; i++)
		{
			for(Iterator iter = dataDayArray[i].testcaseInstanceIterator(); iter.hasNext();)
			{
				TestcaseInstance testcaseInstance = (TestcaseInstance)iter.next();
				lastStatusByTestcaseInstance.put(testcaseInstance, dataDayArray[i].getStatus(testcaseInstance));
			}
			
			GraphDay graphDay = new GraphDay(dataDayArray[i].getDay());
			graphDays.add(graphDay);
			for(Iterator iter = lastStatusByTestcaseInstance.values().iterator(); iter.hasNext();)
			{
				ExecutionStatus executionStatus = (ExecutionStatus) iter.next();

				graphDay.addStatistic(GraphDay.ATTEMPT, 1);
				switch(executionStatus.getValue())
				{
					case ExecutionStatus.PASS:
						graphDay.addStatistic(GraphDay.PASS, 1);
						break;
						
					case ExecutionStatus.FAIL:
						graphDay.addStatistic(GraphDay.FAIL, 1);
						break;

					case ExecutionStatus.SOFT_FAIL:
						graphDay.addStatistic(GraphDay.SOFT_FAIL, 1);
						break;					
				}
				
			}
			
			if(i < max-1)
			{
				GraphDay repeatedGraphDay = ((GraphDay)graphDay.clone()).incrementDay();
				
				//Repeats the current GraphDay for all the days that don't have
				//data
				while(repeatedGraphDay.getDay().compareTo(dataDayArray[i+1].getDay()) < 0)
				{
					graphDays.add((GraphDay)repeatedGraphDay.clone());
					repeatedGraphDay.incrementDay();
				}
			}
		}
		
		return (GraphDay[])graphDays.toArray(new GraphDay[graphDays.size()]);
	}
		
	/**
	 * Loads the expected results for the testcase instances execution from the 
	 * <b>expected_execution.xml</b> file in the specified container (project or folder).
	 * 
	 * <p>Expected Result File format:
	 * 
	 * 	<ComponentTest_ExpectedExecution>
	 * 		<day index="1" attempt="3" pass="2" fail="1" softFail="0"/>
	 * 		<day index="2" attempt="5" pass="2" fail="2" softFail="1"/>
	 * 		<day index="4" attempt="8" pass="5" fail="2" softFail="1"/>
	 * 	</ComponentTest_ExpectedExecution>
	 * 
	 * <b>Each day entry specifies the values for a particular day.  
	 * In the example above the day 3 will have the same information of the day 2.
	 * 
	 * @param IContainer the container (project or folder)
	 * @param ArrayList the list of dailyStatistics where the exepected results will be loaded.
	 * @return int the greater number of testcase instances executions in one day;
	 */
	protected int loadExpectedExecutionValues(IContainer container, GraphDay[] graphDays)
	{
		IFile file = container.getFile(new Path("expected_execution.xml"));
		if((file == null) || (!file.exists()))
			return 0;
			
		Element topElement = null;
		Document document = XMLUtil.getXmlDom(file.getLocation().makeAbsolute().toOSString());
		NodeList list = document.getElementsByTagName("ComponentTest_ExpectedExecution");
		if((list.getLength() != 1) || (!(list.item(0) instanceof Element)))
			return 0;
		
		int maxTestcaseInstaceCount = 0;
		
		int attempt = 0;
		int pass = 0;
		int fail = 0;
		int sofFail = 0;
		int oldIndex = 0;
		
		topElement = (Element)list.item(0);
		list = topElement.getElementsByTagName("day");
		for(int i=0, max=list.getLength(); i<max; i++)
		{
			Object item = list.item(i);
			if(item instanceof Element)
			{
				Element dayElement = (Element)item;
				int index = XMLUtil.getIntAttribute(dayElement, "index");
				if(index <= 0)
					continue;
				
				index--;	
				if(index >= graphDays.length)
					break;
					
				if(index < graphDays.length)
				{
					for(int j=oldIndex; j<index; j++)
					{
						graphDays[j].addStatistic(GraphDay.EXPECTED_ATTEMPT, attempt);
						graphDays[j].addStatistic(GraphDay.EXPECTED_PASS, pass);
						graphDays[j].addStatistic(GraphDay.EXPECTED_FAIL, fail);
						graphDays[j].addStatistic(GraphDay.EXPECTED_SOFT_FAIL, sofFail);
					}

					attempt = XMLUtil.getIntAttribute(dayElement, "attempt");
					if(maxTestcaseInstaceCount < attempt)
						maxTestcaseInstaceCount = attempt;
					
					pass = XMLUtil.getIntAttribute(dayElement, "pass");
					if(maxTestcaseInstaceCount < pass)
						maxTestcaseInstaceCount = pass;

					fail = XMLUtil.getIntAttribute(dayElement, "fail");
					if(maxTestcaseInstaceCount < fail)
						maxTestcaseInstaceCount = fail;

					sofFail = XMLUtil.getIntAttribute(dayElement, "softFail");
					if(maxTestcaseInstaceCount < sofFail)
						maxTestcaseInstaceCount = sofFail;
					
					oldIndex = index;
				}
			}
		}			

		for(int j=oldIndex; j<graphDays.length; j++)
		{
			graphDays[j].addStatistic(GraphDay.EXPECTED_ATTEMPT, attempt);
			graphDays[j].addStatistic(GraphDay.EXPECTED_PASS, pass);
			graphDays[j].addStatistic(GraphDay.EXPECTED_FAIL, fail);
			graphDays[j].addStatistic(GraphDay.EXPECTED_SOFT_FAIL, sofFail);
		}
		
		return maxTestcaseInstaceCount;
	} 
		
	protected int countTestcaseInstancesWithData(TestcaseInstance[] testcaseInstances)
	{
		int counter=0;
		for(int i = 0; i < testcaseInstances.length; i++)
		{
			if(!testcaseInstances[i].getExecutionContainers().isEmpty())
				counter++;
		}
		return counter;
	}
		
		
	//
	// Methods for generating the report content and extra files
	//	
		

	/**
	 * Generates the report content and the required files - "report".svg and "report".css
	 * where "report" is the report file full path minus the extension.
	 * 
	 * @param IFile The report file that <b>will</b> be created by the Component Test Plugin
	 * @param ArrayList the sorted daily statistics
	 * @return InputStream the report content
	 */ 
	protected InputStream generate(IFile reportFile, GraphDay[] graphDays, int numberOfTestcaseInstances)
	throws Exception
	{
		// these are not the best values
		String width = "750";
		String height = "500";

		// Defines a new color palette and initializes it with white in all the positions
		ArrayList colourPalette = new ArrayList(); 

		// creates a Document by collecting information from the Component Test objects
		Document newDoc = generateDataDocument(graphDays, numberOfTestcaseInstances, colourPalette);
		if(newDoc == null)
			return null;

		// turn the DOM into an SVG chart
		SVGLineChart svgLineChart = new SVGLineChart();
		svgLineChart.setDataDocument(newDoc);

		svgLineChart.setGraphicWidth(width);
		svgLineChart.setGraphicHeight(height);

		String preferenceDir = getSVGPreferencePageDirectory();
		if (preferenceDir != null)
			svgLineChart.setResourcesFolder("file://" + preferenceDir.replace('\\', '/') + "/");

		svgLineChart.addGraphicColourPalette("Test Status", (String[])colourPalette.toArray(new String[colourPalette.size()]));
		svgLineChart.setGraphicColourPaletteName("Test Status");

		// create the line chart SVG
		svgLineChart.generateGraphicDocument();

		// write the SVG to a file
		String svgFileName = createFile(reportFile, "svg", svgLineChart.serializeGeneratedDocumentToString()).getName();

		// create the HTML to return to caller
		StringBuffer sb = new StringBuffer();
		sb.append("<html>\n").append("<head>\n").append("<title>").append(getUserText("chart.Title")).append("</title>").append("</head>\n");
		sb.append("<body>\n").append("<object type=\"image/svg+xml\" data=\"");
		sb.append(svgFileName).append("\" ").append("width=\"").append(width).append("\" ");
		sb.append("height=\"").append(height).append("\" ");
		sb.append("name=\"mySvgDocument\"");
		sb.append("/>\n");
		sb.append("</body>").append("</html>");

		return new ByteArrayInputStream(sb.toString().getBytes());
	}
	
	/**
	 * Generates the Document that will be used as the input for the
	 * SVG widgets API to produce the report graph.
	 * 
	 * @param GraphDay[] the sorted daily statistics
	 * @param int the number of Testcase Instances
	 * @param ArrayList the colour palette
	 * @return Document the graph input document
	 */
	protected Document generateDataDocument(GraphDay[] graphDays, int numberOfTestcaseInstances, ArrayList colourPalette)
	throws Exception
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document document = builder.newDocument();

		// create the root element, i.e. configuration
		Element root = (Element)document.createElement("configuration");
		document.appendChild(root);
		
		root.setAttribute("title", getUserText("chart.Title"));
		root.setAttribute("legendtitle", getUserText("chart.Legend"));
		root.setAttribute("timestampPrefix", getUserText("chart.timestampPrefix"));
		

		// Create the element dataUpdate
		Element dataUpdateElement = (Element)document.createElement("dataUpdate");
		root.appendChild(dataUpdateElement);
		Date dateNow = new Date();
		SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd zzz hh:mm:ss");	
		dataUpdateElement.setAttribute("timestamp", dateFormatter.format(dateNow));

		/* Create a list of dataset elements (each dataset correponds to a line in the graph) inside the 
		 * dataUpdate element.  If the line is added then the colourPalette will be changed so it can be
		 * drawn in a specific color,
		 */
		int position = 0;			
		if(addLineData(document, dataUpdateElement, position, getUserText("chart.expectedAttempt"), GraphDay.EXPECTED_ATTEMPT, graphDays))
		{
			colourPalette.add("#D3D3D3"); //grey
			position++;
		}
		if(addLineData(document, dataUpdateElement, position, getUserText("chart.expectedPass"), GraphDay.EXPECTED_PASS, graphDays))
		{
			colourPalette.add("#ADFF2F"); //light green
			position++;
		}
		if(addLineData(document, dataUpdateElement, position, getUserText("chart.expectedFail"), GraphDay.EXPECTED_FAIL, graphDays))
		{
			colourPalette.add("#DB7093"); //light red
			position++;
		}
		if(addLineData(document, dataUpdateElement, position, getUserText("chart.expectedSoftFail"), GraphDay.EXPECTED_SOFT_FAIL, graphDays))
		{
			colourPalette.add("#FAFAD2"); //light yellow
			position++;
		}
		if(addLineData(document, dataUpdateElement, position, getUserText("chart.attempt"), GraphDay.ATTEMPT, graphDays))
		{
			colourPalette.add("#000000"); //black
			position++;
		}
		if(addLineData(document, dataUpdateElement, position, getUserText("chart.pass"), GraphDay.PASS, graphDays))
		{
			colourPalette.add("#008000"); //green
			position++;
		}
		if(addLineData(document, dataUpdateElement, position, getUserText("chart.fail"), GraphDay.FAIL, graphDays))
		{
			colourPalette.add("#FF0000"); //red
			position++;
		}
		if(addLineData(document, dataUpdateElement, position, getUserText("chart.softFail"), GraphDay.SOFT_FAIL, graphDays))
		{
			colourPalette.add("#FFFF00"); //yellow
			position++;
		}
		
		// Create the categorization element which contains the information
		// related X axis
		createXData(document, dataUpdateElement, graphDays);
		
		// Create the categorization element which contains the information
		// related Y axis
		createYData(document, dataUpdateElement, numberOfTestcaseInstances);

		return document;
	}
	
	/**
	 * Adds a line data to the document.  The line will represent one of the values
	 * of the execution statistics.
	 * 
	 * @param Document the Document
	 * @param Element the data element
	 * @param int the position of the line
	 * @param String the name of the line
	 * @param int the type of statistic value will be represented on the line
	 * @param DayStatistic[] the sorted daily statistics
	 * @return boolean if the line data was added
	 */
	protected boolean addLineData(Document document, Element dataUpdateElement, int position, String name, int statisticFlag, GraphDay[] graphDays)
	{
		if(!GraphDay.hasStatistic(statisticFlag))
			return false;
			
		Element dataset = (Element)document.createElement("dataset");
		dataUpdateElement.appendChild(dataset);
		dataset.setAttribute("name",name);
		dataset.setAttribute("position", new Integer(position).toString());
		
		for(int i = 0; i < graphDays.length; i++)
		{
			Element data = (Element)document.createElement("datapoint");
			dataset.appendChild(data);
			
			data.setAttribute("category", "_" + Integer.toString(i));
			data.setAttribute("value", Integer.toString(graphDays[i].getStatistic(statisticFlag)));
		}
		
		return true;
	}
	
	/**
	 * Creates the data that is presented at the X axis.
	 * 
	 * @param Document the Document
	 * @param Element the data element
	 * @param DayStatistic[] the sorted daily statistics
	 */
	protected void createXData(Document document, Element dataUpdateElement, GraphDay[] graphDays)
	{
		Element categorizationElement = (Element)document.createElement("categorization");
		dataUpdateElement.appendChild(categorizationElement);
		
		// Title information
		StringBuffer xTitle = new StringBuffer();
		xTitle.append(getUserText("chart.Xtitle"));
		xTitle.append(" ").append(graphDays[0].getFormattedDay());
		categorizationElement.setAttribute("description", xTitle.toString());
		
		// Add the X axis points
		for(int i = 0; i < graphDays.length; i++)
		{
			Element categoryElement = (Element)document.createElement("category");
			categorizationElement.appendChild(categoryElement);
			String day = graphDays[i].getDayOfMonth();

			categoryElement.setAttribute("id", "_" + i);
			categoryElement.setAttribute("name", day);
			categoryElement.setAttribute("position", Integer.toString(i));				
		}
	}

	/**
	 * Creates the data that is presented at the Y axis.
	 * 
	 * @param Document the Document
	 * @param Element the data element
	 */
	protected void createYData(Document document, Element dataUpdateElement, int numberOfTestcaseInstances)
	{
		Element rangeElement = (Element)document.createElement("range");
		dataUpdateElement.appendChild(rangeElement);
		
		// Define the Y axis range
		rangeElement.setAttribute("context", getUserText("chart.Ytitle"));
		rangeElement.setAttribute("minValue", "0");
		rangeElement.setAttribute("maxValue", Integer.toString(numberOfTestcaseInstances));

		// Add the Y axis points
		for (int i = 0; i < numberOfTestcaseInstances; i++)
		{
			Element segmentMarkerElement = (Element)document.createElement("segmentMarker");
			rangeElement.appendChild(segmentMarkerElement);
			
			segmentMarkerElement.setAttribute("position", Integer.toString(i));
			segmentMarkerElement.setAttribute("value", Integer.toString(i+1));
		}
	}
		
	/**
	 * 	Save all stylesheet content in a css file, the css file name is the same as
	 *  the html name, e.g. if the html file name is test1.html, the css name is 
	 *  test1.css
	 * 
	 * @param	file	a html file 
	 * @return a css file name
	 */
	protected String createStyleSheet(IFile file) throws CoreException
	{
		// Generate a string buffer in which all stylesheet content resides
		StringBuffer sb = new StringBuffer();

		sb.append("rect.outline{stroke-width:0.75pt; stroke:#666666; fill:#ffffff; }\n");
		sb.append("polyline.keyline{stroke-width:0.75pt; stroke:#666666;}\n");
		sb.append("text.title{font-family:Arial; font-style:bold; font-size:11; text-anchor:left;}\n");
		sb.append("text.xtitle{font-family:Arial; font-style:regular; font-size:10; text-anchor:middle;}\n");
		sb.append("text.ytitle{font-family:Arial; font-style:regular; font-size:10; text-anchor:start; writing-mode:tb;}\n");
		sb.append("text.axislabel{font-family:Arial; font-style:regular; font-size:8; text-anchor:middle;}\n");
		sb.append("text.ydata{font-family:Arial; font-style:regular; font-size:8; text-anchor:middle;}\n");
		sb.append("text.legendtitle{font-family:Arial; font-style:regular; font-size:9; text-anchor:start;}\n");
		sb.append("text.legendlabel{font-family:Arial; font-style:regular; font-size:8; text-anchor:start;}\n");
		sb.append("polyline.grid{stroke-width:0.75pt; stroke:#000000;}\n");
		sb.append("polyline.gridline{stroke-width:0.75pt; stroke:#cccccc;}\n");
		sb.append("polyline.griddashline{stroke-width:0.75pt; stroke:#cccccc; stroke-dasharray:2; fill:none;}\n");
		sb.append("polyline.dataset0{stroke-width:0.75pt; stroke:#003399;}\n");
		sb.append("polyline.dataset1{stroke-width:0.75pt; stroke:#0066CC;}\n");
		sb.append("polyline.dataset2{stroke-width:0.75pt; stroke:#3399FF;}\n");
		sb.append("polyline.dataset3{stroke-width:0.75pt; stroke:#66CCFF;}\n");
		sb.append(".shape0{stroke:black; fill:#003399;}\n");
		sb.append(".shape1{stroke:black; fill:#0066CC;}\n");
		sb.append(".shape2{stroke:black; fill:#3399FF;}\n");
		sb.append(".shape3{stroke:black; fill:#66CCFF;}\n");

		return createFile(file, "css", sb.toString()).getName();
	}

	/**
	 * Returns the directory where the preference page for the SVG graph is
	 * stored.
	 * 
	 * @return String The absolute path of the preference page directory.
	 */
	protected String getSVGPreferencePageDirectory()
	{
		String classpath = getUsedClasspath();
		if(classpath == null)
			return null;
			
		int index = classpath.indexOf(File.pathSeparator);
		if(index >= 0)
			classpath.substring(0, index);
			
		File directory = new File(classpath + "/preferences");
		if(directory.exists() && directory.isDirectory())
			return directory.getAbsolutePath();
			
		return null;
	}

	/**
	 * Creates a file in the same location and name of the <i>originalFile</i> but with a different
	 * extension.
	 * 
	 * @param IFile originalFile The file that will be used to define the location and name of the new file
	 * @param String extension The extension of the new file
	 * @param String content The content of the new file.
	 * @return IFile The created file;
	 */
	protected IFile createFile(IFile originalFile, String extension, String content) throws CoreException
	{
		// Generate the file handle where the svg file will be saved
		IPath newPath = originalFile.getProjectRelativePath().removeFileExtension().addFileExtension(extension);
		IFile newFile = originalFile.getProject().getFile(newPath);

		// Deletes any existing file with the same path
		if (newFile.exists())
			newFile.delete(true, true, null);

		// Creates the svg file in the path
		newFile.create(new ByteArrayInputStream(content.getBytes()), true, null);
		return newFile;
	}
	
	/**
	 * Returns the values that will be presented for the user.
	 * 
	 * @param String key The key for the value
	 * @return String the value
	 */
	protected String getUserText(String key)
	{
		if("chart.Title".equals(key))
			return "Testcase Status By Date";
		
		if("chart.Legend".equals(key))
			return "Status";
			
		if("chart.timestampPrefix".equals(key))
			return "Last update:";
		
		if("chart.Xtitle".equals(key))	
			return "Date starting at:";
			
		if("chart.Ytitle".equals(key))
			return "Testcases";


		if("chart.attempt".equals(key))
			return "Attempt";
			
		if("chart.pass".equals(key))
			return "Pass";
		
		if("chart.fail".equals(key))	
			return "Fail";
			
		if("chart.softFail".equals(key))
			return "Soft Fail";


		if("chart.expectedAttempt".equals(key))
			return "Expected Attempt";
			
		if("chart.expectedPass".equals(key))
			return "Expected Pass";
		
		if("chart.expectedFail".equals(key))	
			return "Expected Fail";
			
		if("chart.expectedSoftFail".equals(key))
			return "Expected Soft Fail";
		
		return key;
	}
}
